@@ -25,25 +25,24 @@
  * two-part external storage URLs. Used for represent efficient concatenated
  * storage, and migration-related pointer objects.
  */
-interface HistoryBlob
-{
+interface HistoryBlob {
 	/**
 	 * Adds an item of text, returns a stub object which points to the item.
 	 * You must call setLocation() on the stub object before storing it to the
 	 * database
 	 *
-	 * @param $text string
+	 * @param string $text
 	 *
-	 * @return String: the key for getItem()
+	 * @return string The key for getItem()
 	 */
 	function addItem( $text );
 
 	/**
 	 * Get item by key, or false if the key is not present
 	 *
-	 * @param $key string
+	 * @param string $key
 	 *
-	 * @return String or false
+	 * @return string|bool
 	 */
 	function getItem( $key );
 
@@ -55,14 +54,14 @@ interface HistoryBlob
 	 *
 	 * Default text is not required for two-part external storage URLs.
 	 *
-	 * @param $text string
+	 * @param string $text
 	 */
 	function setText( $text );
 
 	/**
 	 * Get default text. This is called from Revision::getRevisionText()
 	 *
-	 * @return String
+	 * @return string
 	 */
 	function getText();
 }
@@ -71,9 +70,8 @@ interface HistoryBlob
  * Concatenated gzip (CGZ) storage
  * Improves compression ratio by concatenating like objects before gzipping
  */
-class ConcatenatedGzipHistoryBlob implements HistoryBlob
-{
-	public $mVersion = 0, $mCompressed = false, $mItems = array(), $mDefaultHash = '';
+class ConcatenatedGzipHistoryBlob implements HistoryBlob {
+	public $mVersion = 0, $mCompressed = false, $mItems = [], $mDefaultHash = '';
 	public $mSize = 0;
 	public $mMaxSize = 10000000;
 	public $mMaxCount = 100;
@@ -83,12 +81,13 @@ class ConcatenatedGzipHistoryBlob implements HistoryBlob
 	 */
 	public function __construct() {
 		if ( !function_exists( 'gzdeflate' ) ) {
-			throw new MWException( "Need zlib support to read or write this kind of history object (ConcatenatedGzipHistoryBlob)\n" );
+			throw new MWException( "Need zlib support to read or write this "
+				. "kind of history object (ConcatenatedGzipHistoryBlob)\n" );
 		}
 	}
 
 	/**
-	 * @param $text string
+	 * @param string $text
 	 * @return string
 	 */
 	public function addItem( $text ) {
@@ -102,7 +101,7 @@ class ConcatenatedGzipHistoryBlob implements HistoryBlob
 	}
 
 	/**
-	 * @param $hash string
+	 * @param string $hash
 	 * @return array|bool
 	 */
 	public function getItem( $hash ) {
@@ -115,7 +114,7 @@ class ConcatenatedGzipHistoryBlob implements HistoryBlob
 	}
 
 	/**
-	 * @param $text string
+	 * @param string $text
 	 * @return void
 	 */
 	public function setText( $text ) {
@@ -134,7 +133,7 @@ class ConcatenatedGzipHistoryBlob implements HistoryBlob
 	/**
 	 * Remove an item
 	 *
-	 * @param $hash string
+	 * @param string $hash
 	 */
 	public function removeItem( $hash ) {
 		$this->mSize -= strlen( $this->mItems[$hash] );
@@ -166,7 +165,7 @@ class ConcatenatedGzipHistoryBlob implements HistoryBlob
 	 */
 	function __sleep() {
 		$this->compress();
-		return array( 'mVersion', 'mCompressed', 'mItems', 'mDefaultHash' );
+		return [ 'mVersion', 'mCompressed', 'mItems', 'mDefaultHash' ];
 	}
 
 	function __wakeup() {
@@ -190,18 +189,25 @@ class ConcatenatedGzipHistoryBlob implements HistoryBlob
  */
 class HistoryBlobStub {
 	/**
-	 * One-step cache variable to hold base blobs; operations that
-	 * pull multiple revisions may often pull multiple times from
-	 * the same blob. By keeping the last-used one open, we avoid
-	 * redundant unserialization and decompression overhead.
+	 * @var array One-step cache variable to hold base blobs; operations that
+	 * pull multiple revisions may often pull multiple times from the same
+	 * blob. By keeping the last-used one open, we avoid redundant
+	 * unserialization and decompression overhead.
 	 */
-	protected static $blobCache = array();
+	protected static $blobCache = [];
 
-	var $mOldId, $mHash, $mRef;
+	/** @var int */
+	public $mOldId;
+
+	/** @var string */
+	public $mHash;
+
+	/** @var string */
+	public $mRef;
 
 	/**
-	 * @param string $hash the content hash of the text
-	 * @param $oldid Integer the old_id for the CGZ object
+	 * @param string $hash The content hash of the text
+	 * @param int $oldid The old_id for the CGZ object
 	 */
 	function __construct( $hash = '', $oldid = 0 ) {
 		$this->mHash = $hash;
@@ -210,6 +216,7 @@ class HistoryBlobStub {
 	/**
 	 * Sets the location (old_id) of the main object to which this object
 	 * points
+	 * @param int $id
 	 */
 	function setLocation( $id ) {
 		$this->mOldId = $id;
@@ -217,6 +224,7 @@ class HistoryBlobStub {
 
 	/**
 	 * Sets the location (old_id) of the referring object
+	 * @param string $id
 	 */
 	function setReferrer( $id ) {
 		$this->mRef = $id;
@@ -224,6 +232,7 @@ class HistoryBlobStub {
 
 	/**
 	 * Gets the location of the referring object
+	 * @return string
 	 */
 	function getReferrer() {
 		return $this->mRef;
@@ -237,10 +246,16 @@ class HistoryBlobStub {
 			$obj = self::$blobCache[$this->mOldId];
 		} else {
 			$dbr = wfGetDB( DB_SLAVE );
-			$row = $dbr->selectRow( 'text', array( 'old_flags', 'old_text' ), array( 'old_id' => $this->mOldId ) );
+			$row = $dbr->selectRow(
+				'text',
+				[ 'old_flags', 'old_text' ],
+				[ 'old_id' => $this->mOldId ]
+			);
+
 			if ( !$row ) {
 				return false;
 			}
+
 			$flags = explode( ',', $row->old_flags );
 			if ( in_array( 'external', $flags ) ) {
 				$url = $row->old_text;
@@ -248,9 +263,10 @@ class HistoryBlobStub {
 				if ( !isset( $parts[1] ) || $parts[1] == '' ) {
 					return false;
 				}
-				$row->old_text = ExternalStore::fetchFromUrl( $url );
+				$row->old_text = ExternalStore::fetchFromURL( $url );
 
 			}
+
 			if ( !in_array( 'object', $flags ) ) {
 				return false;
 			}
@@ -271,8 +287,9 @@ class HistoryBlobStub {
 			// Save this item for reference; if pulling many
 			// items in a row we'll likely use it again.
 			$obj->uncompress();
-			self::$blobCache = array( $this->mOldId => $obj );
+			self::$blobCache = [ $this->mOldId => $obj ];
 		}
+
 		return $obj->getItem( $this->mHash );
 	}
 
@@ -295,10 +312,11 @@ class HistoryBlobStub {
  * on conversion if $wgLegacySchemaConversion is set to true.
  */
 class HistoryBlobCurStub {
-	var $mCurId;
+	/** @var int */
+	public $mCurId;
 
 	/**
-	 * @param $curid Integer: the cur_id pointed to
+	 * @param int $curid The cur_id pointed to
 	 */
 	function __construct( $curid = 0 ) {
 		$this->mCurId = $curid;
@@ -308,7 +326,7 @@ class HistoryBlobCurStub {
 	 * Sets the location (cur_id) of the main object to which this object
 	 * points
 	 *
-	 * @param $id int
+	 * @param int $id
 	 */
 	function setLocation( $id ) {
 		$this->mCurId = $id;
@@ -319,7 +337,7 @@ class HistoryBlobCurStub {
 	 */
 	function getText() {
 		$dbr = wfGetDB( DB_SLAVE );
-		$row = $dbr->selectRow( 'cur', array( 'cur_text' ), array( 'cur_id' => $this->mCurId ) );
+		$row = $dbr->selectRow( 'cur', [ 'cur_text' ], [ 'cur_id' => $this->mCurId ] );
 		if ( !$row ) {
 			return false;
 		}
@@ -332,50 +350,43 @@ class HistoryBlobCurStub {
  * Requires xdiff 1.5+ and zlib
  */
 class DiffHistoryBlob implements HistoryBlob {
-	/** Uncompressed item cache */
-	var $mItems = array();
+	/** @var array Uncompressed item cache */
+	public $mItems = [];
 
-	/** Total uncompressed size */
-	var $mSize = 0;
+	/** @var int Total uncompressed size */
+	public $mSize = 0;
 
 	/**
-	 * Array of diffs. If a diff D from A to B is notated D = B - A, and Z is
-	 * an empty string:
+	 * @var array Array of diffs. If a diff D from A to B is notated D = B - A,
+	 * and Z is an empty string:
 	 *
 	 *              { item[map[i]] - item[map[i-1]]   where i > 0
 	 *    diff[i] = {
 	 *              { item[map[i]] - Z                where i = 0
 	 */
-	var $mDiffs;
+	public $mDiffs;
 
-	/** The diff map, see above */
-	var $mDiffMap;
+	/** @var array The diff map, see above */
+	public $mDiffMap;
 
-	/**
-	 * The key for getText()
+	/** @var int The key for getText()
 	 */
-	var $mDefaultKey;
+	public $mDefaultKey;
+
+	/** @var string Compressed storage */
+	public $mCompressed;
+
+	/** @var bool True if the object is locked against further writes */
+	public $mFrozen = false;
 
 	/**
-	 * Compressed storage
-	 */
-	var $mCompressed;
-
-	/**
-	 * True if the object is locked against further writes
-	 */
-	var $mFrozen = false;
-
-	/**
-	 * The maximum uncompressed size before the object becomes sad
+	 * @var int The maximum uncompressed size before the object becomes sad
 	 * Should be less than max_allowed_packet
 	 */
-	var $mMaxSize = 10000000;
+	public $mMaxSize = 10000000;
 
-	/**
-	 * The maximum number of text items before the object becomes sad
-	 */
-	var $mMaxCount = 100;
+	/** @var int The maximum number of text items before the object becomes sad */
+	public $mMaxCount = 100;
 
 	/** Constants from xdiff.h */
 	const XDL_BDOP_INS = 1;
@@ -390,7 +401,7 @@ class DiffHistoryBlob implements HistoryBlob {
 
 	/**
 	 * @throws MWException
-	 * @param $text string
+	 * @param string $text
 	 * @return int
 	 */
 	function addItem( $text ) {
@@ -405,7 +416,7 @@ class DiffHistoryBlob implements HistoryBlob {
 	}
 
 	/**
-	 * @param $key string
+	 * @param string $key
 	 * @return string
 	 */
 	function getItem( $key ) {
@@ -413,7 +424,7 @@ class DiffHistoryBlob implements HistoryBlob {
 	}
 
 	/**
-	 * @param $text string
+	 * @param string $text
 	 */
 	function setText( $text ) {
 		$this->mDefaultKey = $this->addItem( $text );
@@ -443,21 +454,22 @@ class DiffHistoryBlob implements HistoryBlob {
 		}
 
 		// Create two diff sequences: one for main text and one for small text
-		$sequences = array(
-			'small' => array(
+		$sequences = [
+			'small' => [
 				'tail' => '',
-				'diffs' => array(),
-				'map' => array(),
-			),
-			'main' => array(
+				'diffs' => [],
+				'map' => [],
+			],
+			'main' => [
 				'tail' => '',
-				'diffs' => array(),
-				'map' => array(),
-			),
-		);
+				'diffs' => [],
+				'map' => [],
+			],
+		];
 		$smallFactor = 0.5;
 
-		for ( $i = 0; $i < count( $this->mItems ); $i++ ) {
+		$mItemsCount = count( $this->mItems );
+		for ( $i = 0; $i < $mItemsCount; $i++ ) {
 			$text = $this->mItems[$i];
 			if ( $i == 0 ) {
 				$seqName = 'main';
@@ -480,8 +492,8 @@ class DiffHistoryBlob implements HistoryBlob {
 
 		// Knit the sequences together
 		$tail = '';
-		$this->mDiffs = array();
-		$this->mDiffMap = array();
+		$this->mDiffs = [];
+		$this->mDiffMap = [];
 		foreach ( $sequences as $seq ) {
 			if ( !count( $seq['diffs'] ) ) {
 				continue;
@@ -493,7 +505,8 @@ class DiffHistoryBlob implements HistoryBlob {
 				$this->mDiffs[] = $this->diff( $tail, $head );
 			}
 			$this->mDiffMap[] = $seq['map'][0];
-			for ( $i = 1; $i < count( $seq['diffs'] ); $i++ ) {
+			$diffsCount = count( $seq['diffs'] );
+			for ( $i = 1; $i < $diffsCount; $i++ ) {
 				$this->mDiffs[] = $seq['diffs'][$i];
 				$this->mDiffMap[] = $seq['map'][$i];
 			}
@@ -502,29 +515,29 @@ class DiffHistoryBlob implements HistoryBlob {
 	}
 
 	/**
-	 * @param $t1
-	 * @param $t2
+	 * @param string $t1
+	 * @param string $t2
 	 * @return string
 	 */
 	function diff( $t1, $t2 ) {
 		# Need to do a null concatenation with warnings off, due to bugs in the current version of xdiff
 		# "String is not zero-terminated"
-		wfSuppressWarnings();
+		MediaWiki\suppressWarnings();
 		$diff = xdiff_string_rabdiff( $t1, $t2 ) . '';
-		wfRestoreWarnings();
+		MediaWiki\restoreWarnings();
 		return $diff;
 	}
 
 	/**
-	 * @param $base
-	 * @param $diff
+	 * @param string $base
+	 * @param string $diff
 	 * @return bool|string
 	 */
 	function patch( $base, $diff ) {
 		if ( function_exists( 'xdiff_string_bpatch' ) ) {
-			wfSuppressWarnings();
+			MediaWiki\suppressWarnings();
 			$text = xdiff_string_bpatch( $base, $diff ) . '';
-			wfRestoreWarnings();
+			MediaWiki\restoreWarnings();
 			return $text;
 		}
 
@@ -580,7 +593,7 @@ class DiffHistoryBlob implements HistoryBlob {
 	 * the bytes backwards and initialised with 0 instead of 1. See bug 34428.
 	 *
 	 * @param string $s
-	 * @return string|bool: false if the hash extension is not available
+	 * @return string|bool False if the hash extension is not available
 	 */
 	function xdiffAdler32( $s ) {
 		if ( !function_exists( 'hash' ) ) {
@@ -603,7 +616,8 @@ class DiffHistoryBlob implements HistoryBlob {
 			return;
 		}
 		$tail = '';
-		for ( $diffKey = 0; $diffKey < count( $this->mDiffs ); $diffKey++ ) {
+		$mDiffsCount = count( $this->mDiffs );
+		for ( $diffKey = 0; $diffKey < $mDiffsCount; $diffKey++ ) {
 			$textKey = $this->mDiffMap[$diffKey];
 			$text = $this->patch( $tail, $this->mDiffs[$diffKey] );
 			$this->mItems[$textKey] = $text;
@@ -630,16 +644,16 @@ class DiffHistoryBlob implements HistoryBlob {
 				$map .= $i - $prev;
 				$prev = $i;
 			}
-			$info = array(
+			$info = [
 				'diffs' => $this->mDiffs,
 				'map' => $map
-			);
+			];
 		}
 		if ( isset( $this->mDefaultKey ) ) {
 			$info['default'] = $this->mDefaultKey;
 		}
 		$this->mCompressed = gzdeflate( serialize( $info ) );
-		return array( 'mCompressed' );
+		return [ 'mCompressed' ];
 	}
 
 	function __wakeup() {
@@ -667,7 +681,7 @@ class DiffHistoryBlob implements HistoryBlob {
 			// New format
 			$map = explode( ',', $info['map'] );
 			$cur = 0;
-			$this->mDiffMap = array();
+			$this->mDiffMap = [];
 			foreach ( $map as $i ) {
 				$cur += $i;
 				$this->mDiffMap[] = $cur;
