@@ -55,7 +55,7 @@ class MovePage {
 		// Convert into a Status object
 		if ( $errors ) {
 			foreach ( $errors as $error ) {
-				call_user_func_array( array( $status, 'fatal' ), $error );
+				call_user_func_array( [ $status, 'fatal' ], $error );
 			}
 		}
 
@@ -64,25 +64,13 @@ class MovePage {
 			$status->fatal( 'spamprotectiontext' );
 		}
 
-		# The move is allowed only if (1) the target doesn't exist, or
-		# (2) the target is a redirect to the source, and has no history
-		# (so we can undo bad moves right after they're done).
-
-		if ( $this->newTitle->getArticleID() ) { # Target exists; check for validity
-			if ( !$this->isValidMoveTarget() ) {
-				$status->fatal( 'articleexists' );
-			}
-		} else {
-			$tp = $this->newTitle->getTitleProtection();
-			if ( $tp !== false ) {
-				if ( !$user->isAllowed( $tp['permission'] ) ) {
-					$status->fatal( 'cantmove-titleprotected' );
-				}
-			}
+		$tp = $this->newTitle->getTitleProtection();
+		if ( $tp !== false && !$user->isAllowed( $tp['permission'] ) ) {
+				$status->fatal( 'cantmove-titleprotected' );
 		}
 
 		Hooks::run( 'MovePageCheckPermissions',
-			array( $this->oldTitle, $this->newTitle, $user, $reason, $status )
+			[ $this->oldTitle, $this->newTitle, $user, $reason, $status ]
 		);
 
 		return $status;
@@ -125,6 +113,13 @@ class MovePage {
 			$status->fatal( 'badarticleerror' );
 		}
 
+		# The move is allowed only if (1) the target doesn't exist, or
+		# (2) the target is a redirect to the source, and has no history
+		# (so we can undo bad moves right after they're done).
+		if ( $this->newTitle->getArticleID() && !$this->isValidMoveTarget() ) {
+			$status->fatal( 'articleexists' );
+		}
+
 		// Content model checks
 		if ( !$wgContentHandlerUseDB &&
 			$this->oldTitle->getContentModel() !== $this->newTitle->getContentModel() ) {
@@ -146,7 +141,7 @@ class MovePage {
 		}
 
 		// Hook for extensions to say a title can't be moved for technical reasons
-		Hooks::run( 'MovePageIsValidMove', array( $this->oldTitle, $this->newTitle, $status ) );
+		Hooks::run( 'MovePageIsValidMove', [ $this->oldTitle, $this->newTitle, $status ] );
 
 		return $status;
 	}
@@ -233,7 +228,7 @@ class MovePage {
 	public function move( User $user, $reason, $createRedirect ) {
 		global $wgCategoryCollation;
 
-		Hooks::run( 'TitleMove', array( $this->oldTitle, $this->newTitle, $user ) );
+		Hooks::run( 'TitleMove', [ $this->oldTitle, $this->newTitle, $user ] );
 
 		// If it is a file, move it first.
 		// It is done before all other moving stuff is done because it's hard to revert.
@@ -243,7 +238,7 @@ class MovePage {
 			$file->load( File::READ_LATEST );
 			if ( $file->exists() ) {
 				$status = $file->move( $this->newTitle );
-				if ( !$status->isOk() ) {
+				if ( !$status->isOK() ) {
 					return $status;
 				}
 			}
@@ -252,12 +247,15 @@ class MovePage {
 			RepoGroup::singleton()->clearCache( $this->newTitle ); # clear false negative cache
 		}
 
-		$dbw->begin( __METHOD__ ); # If $file was a LocalFile, its transaction would have closed our own.
+		$dbw->startAtomic( __METHOD__ );
+
+		Hooks::run( 'TitleMoveStarting', [ $this->oldTitle, $this->newTitle, $user ] );
+
 		$pageid = $this->oldTitle->getArticleID( Title::GAID_FOR_UPDATE );
 		$protected = $this->oldTitle->isProtected();
 
 		// Do the actual move
-		$this->moveToInternal( $user, $this->newTitle, $reason, $createRedirect );
+		$nullRevision = $this->moveToInternal( $user, $this->newTitle, $reason, $createRedirect );
 
 		// Refresh the sortkey for this row.  Be careful to avoid resetting
 		// cl_timestamp, which may disturb time-based lists on some sites.
@@ -265,8 +263,8 @@ class MovePage {
 		// from LinksUpdate::getCategoryInsertions() and friends.
 		$prefixes = $dbw->select(
 			'categorylinks',
-			array( 'cl_sortkey_prefix', 'cl_to' ),
-			array( 'cl_from' => $pageid ),
+			[ 'cl_sortkey_prefix', 'cl_to' ],
+			[ 'cl_from' => $pageid ],
 			__METHOD__
 		);
 		if ( $this->newTitle->getNamespace() == NS_CATEGORY ) {
@@ -280,15 +278,15 @@ class MovePage {
 			$prefix = $prefixRow->cl_sortkey_prefix;
 			$catTo = $prefixRow->cl_to;
 			$dbw->update( 'categorylinks',
-				array(
+				[
 					'cl_sortkey' => Collation::singleton()->getSortKey(
 							$this->newTitle->getCategorySortkey( $prefix ) ),
 					'cl_collation' => $wgCategoryCollation,
 					'cl_type' => $type,
-					'cl_timestamp=cl_timestamp' ),
-				array(
+					'cl_timestamp=cl_timestamp' ],
+				[
 					'cl_from' => $pageid,
-					'cl_to' => $catTo ),
+					'cl_to' => $catTo ],
 				__METHOD__
 			);
 		}
@@ -298,20 +296,20 @@ class MovePage {
 		if ( $protected ) {
 			# Protect the redirect title as the title used to be...
 			$dbw->insertSelect( 'page_restrictions', 'page_restrictions',
-				array(
+				[
 					'pr_page' => $redirid,
 					'pr_type' => 'pr_type',
 					'pr_level' => 'pr_level',
 					'pr_cascade' => 'pr_cascade',
 					'pr_user' => 'pr_user',
 					'pr_expiry' => 'pr_expiry'
-				),
-				array( 'pr_page' => $pageid ),
+				],
+				[ 'pr_page' => $pageid ],
 				__METHOD__,
-				array( 'IGNORE' )
+				[ 'IGNORE' ]
 			);
-			# Update the protection log
-			$log = new LogPage( 'protect' );
+
+			// Build comment for log
 			$comment = wfMessage(
 				'prot_1movedto2',
 				$this->oldTitle->getPrefixedText(),
@@ -320,44 +318,47 @@ class MovePage {
 			if ( $reason ) {
 				$comment .= wfMessage( 'colon-separator' )->inContentLanguage()->text() . $reason;
 			}
-			// @todo FIXME: $params?
-			$logId = $log->addEntry(
-				'move_prot',
-				$this->newTitle,
-				$comment,
-				array( $this->oldTitle->getPrefixedText() ),
-				$user
-			);
 
 			// reread inserted pr_ids for log relation
 			$insertedPrIds = $dbw->select(
 				'page_restrictions',
 				'pr_id',
-				array( 'pr_page' => $redirid ),
+				[ 'pr_page' => $redirid ],
 				__METHOD__
 			);
-			$logRelationsValues = array();
+			$logRelationsValues = [];
 			foreach ( $insertedPrIds as $prid ) {
 				$logRelationsValues[] = $prid->pr_id;
 			}
-			$log->addRelations( 'pr_id', $logRelationsValues, $logId );
+
+			// Update the protection log
+			$logEntry = new ManualLogEntry( 'protect', 'move_prot' );
+			$logEntry->setTarget( $this->newTitle );
+			$logEntry->setComment( $comment );
+			$logEntry->setPerformer( $user );
+			$logEntry->setParameters( [
+				'4::oldtitle' => $this->oldTitle->getPrefixedText(),
+			] );
+			$logEntry->setRelations( [ 'pr_id' => $logRelationsValues ] );
+			$logId = $logEntry->insert();
+			$logEntry->publish( $logId );
 		}
 
 		// Update *_from_namespace fields as needed
 		if ( $this->oldTitle->getNamespace() != $this->newTitle->getNamespace() ) {
 			$dbw->update( 'pagelinks',
-				array( 'pl_from_namespace' => $this->newTitle->getNamespace() ),
-				array( 'pl_from' => $pageid ),
+				[ 'pl_from_namespace' => $this->newTitle->getNamespace() ],
+				[ 'pl_from' => $pageid ],
 				__METHOD__
 			);
 			$dbw->update( 'templatelinks',
-				array( 'tl_from_namespace' => $this->newTitle->getNamespace() ),
-				array( 'tl_from' => $pageid ),
+				[ 'tl_from_namespace' => $this->newTitle->getNamespace() ],
+				[ 'tl_from' => $pageid ],
 				__METHOD__
 			);
 			$dbw->update( 'imagelinks',
-				array( 'il_from_namespace' => $this->newTitle->getNamespace() ),
-				array( 'il_from' => $pageid ),
+				[ 'il_from_namespace' => $this->newTitle->getNamespace() ],
+				[ 'il_from' => $pageid ],
 				__METHOD__
 			);
 		}
@@ -368,15 +369,35 @@ class MovePage {
 		$oldsnamespace = MWNamespace::getSubject( $this->oldTitle->getNamespace() );
 		$newsnamespace = MWNamespace::getSubject( $this->newTitle->getNamespace() );
 		if ( $oldsnamespace != $newsnamespace || $oldtitle != $newtitle ) {
-			WatchedItem::duplicateEntries( $this->oldTitle, $this->newTitle );
+			$store = WatchedItemStore::getDefaultInstance();
+			$store->duplicateAllAssociatedEntries( $this->oldTitle, $this->newTitle );
 		}
 
-		$dbw->commit( __METHOD__ );
-
 		Hooks::run(
-			'TitleMoveComplete',
-			array( &$this->oldTitle, &$this->newTitle, &$user, $pageid, $redirid, $reason )
+			'TitleMoveCompleting',
+			[ $this->oldTitle, $this->newTitle,
+				$user, $pageid, $redirid, $reason, $nullRevision ]
 		);
+
+		$dbw->endAtomic( __METHOD__ );
+
+		$params = [
+			&$this->oldTitle,
+			&$this->newTitle,
+			&$user,
+			$pageid,
+			$redirid,
+			$reason,
+			$nullRevision
+		];
+		// Hold onto $user to avoid HHVM bug where it no longer
+		// becomes a reference (T118683)
+		$dbw->onTransactionIdle( function () use ( $params, $dbw, &$user ) {
+			// Keep each single hook handler atomic
+			$dbw->setFlag( DBO_TRX ); // flag is automatically reset by DB layer
+			Hooks::run( 'TitleMoveComplete', $params );
+		} );
+
 		return Status::newGood();
 	}
 
@@ -390,6 +411,7 @@ class MovePage {
 	 * @param string $reason The reason for the move
 	 * @param bool $createRedirect Whether to leave a redirect at the old title. Does not check
 	 *   if the user has the suppressredirect right
+	 * @return Revision the revision created by the move
 	 * @throws MWException
 	 */
 	private function moveToInternal( User $user, &$nt, $reason = '', $createRedirect = true ) {
@@ -421,6 +443,13 @@ class MovePage {
 			$redirectContent = null;
 		}
 
+		// Figure out whether the content model is no longer the default
+		$oldDefault = ContentHandler::getDefaultModelFor( $this->oldTitle );
+		$contentModel = $this->oldTitle->getContentModel();
+		$newDefault = ContentHandler::getDefaultModelFor( $nt );
+		$defaultContentModelChanging = ( $oldDefault !== $newDefault
+			&& $oldDefault === $contentModel );
+
 		// bug 57084: log_page should be the ID of the *moved* page
 		$oldid = $this->oldTitle->getArticleID();
 		$logTitle = clone $this->oldTitle;
@@ -429,10 +458,10 @@ class MovePage {
 		$logEntry->setPerformer( $user );
 		$logEntry->setTarget( $logTitle );
 		$logEntry->setComment( $reason );
-		$logEntry->setParameters( array(
+		$logEntry->setParameters( [
 			'4::target' => $nt->getPrefixedText(),
 			'5::noredir' => $redirectContent ? '0': '1',
-		) );
+		] );
 
 		$formatter = LogFormatter::newFromEntry( $logEntry );
 		$formatter->setContext( RequestContext::newExtraneousContext( $this->oldTitle ) );
@@ -458,7 +487,7 @@ class MovePage {
 			# by definition if we've got here it's rather uninteresting.
 			# We have to remove it so that the next step doesn't trigger
 			# a conflict on the unique namespace+title index...
-			$dbw->delete( 'page', array( 'page_id' => $newid ), __METHOD__ );
+			$dbw->delete( 'page', [ 'page_id' => $newid ], __METHOD__ );
 
 			$newpage->doDeleteUpdates( $newid, $newcontent );
 		}
@@ -473,11 +502,11 @@ class MovePage {
 
 		# Change the name of the target page:
 		$dbw->update( 'page',
-			/* SET */ array(
+			/* SET */ [
 				'page_namespace' => $nt->getNamespace(),
 				'page_title' => $nt->getDBkey(),
-			),
-			/* WHERE */ array( 'page_id' => $oldid ),
+			],
+			/* WHERE */ [ 'page_id' => $oldid ],
 			__METHOD__
 		);
 
@@ -493,10 +522,20 @@ class MovePage {
 		$newpage->updateRevisionOn( $dbw, $nullRevision );
 
 		Hooks::run( 'NewRevisionFromEditComplete',
-			array( $newpage, $nullRevision, $nullRevision->getParentId(), $user ) );
+			[ $newpage, $nullRevision, $nullRevision->getParentId(), $user ] );
 
 		$newpage->doEditUpdates( $nullRevision, $user,
-			array( 'changed' => false, 'moved' => true, 'oldcountable' => $oldcountable ) );
+			[ 'changed' => false, 'moved' => true, 'oldcountable' => $oldcountable ] );
+
+		// If the default content model changes, we need to populate rev_content_model
+		if ( $defaultContentModelChanging ) {
+			$dbw->update(
+				'revision',
+				[ 'rev_content_model' => $contentModel ],
+				[ 'rev_page' => $nt->getArticleID(), 'rev_content_model IS NULL' ],
+				__METHOD__
+			);
+		}
 
 		if ( !$moveOverRedirect ) {
 			WikiPage::onArticleCreate( $nt );
@@ -509,25 +548,27 @@ class MovePage {
 			$newid = $redirectArticle->insertOn( $dbw );
 			if ( $newid ) { // sanity
 				$this->oldTitle->resetArticleID( $newid );
-				$redirectRevision = new Revision( array(
+				$redirectRevision = new Revision( [
 					'title' => $this->oldTitle, // for determining the default content model
 					'page' => $newid,
 					'user_text' => $user->getName(),
 					'user' => $user->getId(),
 					'comment' => $comment,
-					'content' => $redirectContent ) );
+					'content' => $redirectContent ] );
 				$redirectRevision->insertOn( $dbw );
 				$redirectArticle->updateRevisionOn( $dbw, $redirectRevision, 0 );
 
 				Hooks::run( 'NewRevisionFromEditComplete',
-					array( $redirectArticle, $redirectRevision, false, $user ) );
+					[ $redirectArticle, $redirectRevision, false, $user ] );
 
-				$redirectArticle->doEditUpdates( $redirectRevision, $user, array( 'created' => true ) );
+				$redirectArticle->doEditUpdates( $redirectRevision, $user, [ 'created' => true ] );
 			}
 		}
 
 		# Log the move
 		$logid = $logEntry->insert();
 		$logEntry->publish( $logid );
+
+		return $nullRevision;
 	}
 }
