@@ -26,6 +26,8 @@
  * @file
  */
 
+use MediaWiki\Logger\LoggerFactory;
+
 /**
  * A simple method to retrieve the plain source of an article,
  * using "action=raw" in the GET request string.
@@ -33,14 +35,6 @@
  * @ingroup Actions
  */
 class RawAction extends FormlessAction {
-	/**
-	 * @var bool Does the request include a gen=css|javascript parameter
-	 * @deprecated This used to be a string for "css" or "javascript" but
-	 * it is no longer used. Setting this parameter results in empty content
-	 * being served
-	 */
-	private $gen = false;
-
 	public function getName() {
 		return 'raw';
 	}
@@ -56,6 +50,7 @@ class RawAction extends FormlessAction {
 	function onView() {
 		$this->getOutput()->disable();
 		$request = $this->getRequest();
+		$response = $request->response();
 		$config = $this->context->getConfig();
 
 		if ( !$request->checkUrlExtension() ) {
@@ -66,64 +61,108 @@ class RawAction extends FormlessAction {
 			return; // Client cache fresh and headers sent, nothing more to do.
 		}
 
-		# special case for 'generated' raw things: user css/js
-		# This is deprecated and will only return empty content
 		$gen = $request->getVal( 'gen' );
-		$smaxage = $request->getIntOrNull( 'smaxage' );
-
 		if ( $gen == 'css' || $gen == 'js' ) {
 			$this->gen = true;
-			if ( $smaxage === null ) {
-				$smaxage = $config->get( 'SquidMaxage' );
-			}
 		}
 
 		$contentType = $this->getContentType();
 
-		# Force caching for CSS and JS raw content, default: 5 minutes.
-		# Note: If using a canonical url for userpage css/js, we send an HTCP purge.
+		$maxage = $request->getInt( 'maxage', $config->get( 'SquidMaxage' ) );
+		$smaxage = $request->getIntOrNull( 'smaxage' );
 		if ( $smaxage === null ) {
 			if ( $contentType == 'text/css' || $contentType == 'text/javascript' ) {
+				// CSS/JS raw content has its own CDN max age configuration.
+				// Note: Title::getCdnUrls() includes action=raw for css/js pages,
+				// so if using the canonical url, this will get HTCP purges.
 				$smaxage = intval( $config->get( 'ForcedRawSMaxage' ) );
 			} else {
+				// No CDN cache for anything else
 				$smaxage = 0;
 			}
 		}
 
-		$maxage = $request->getInt( 'maxage', $config->get( 'SquidMaxage' ) );
-
-		$response = $request->response();
-
 		// Set standard Vary headers so cache varies on cookies and such (T125283)
 		$response->header( $this->getOutput()->getVaryHeader() );
-		if ( $config->get( 'UseXVO' ) ) {
-			$response->header( $this->getOutput()->getXVO() );
+		if ( $config->get( 'UseKeyHeader' ) ) {
+			$response->header( $this->getOutput()->getKeyHeader() );
 		}
 
-		$response->header( 'Content-type: ' . $contentType . '; charset=UTF-8' );
-		# Output may contain user-specific data;
-		# vary generated content for open sessions on private wikis
-		$privateCache = !User::isEveryoneAllowed( 'read' ) && ( $smaxage == 0 || session_id() != '' );
-		// Bug 53032 - make this private if user is logged in,
-		// so we don't accidentally cache cookies
-		$privateCache = $privateCache ?: $this->getUser()->isLoggedIn();
-		# allow the client to cache this for 24 hours
+		// Output may contain user-specific data;
+		// vary generated content for open sessions on private wikis
+		$privateCache = !User::isEveryoneAllowed( 'read' ) &&
+			( $smaxage == 0 || MediaWiki\Session\SessionManager::getGlobalSession()->isPersistent() );
+		// Don't accidentally cache cookies if user is logged in (T55032)
+		$privateCache = $privateCache || $this->getUser()->isLoggedIn();
 		$mode = $privateCache ? 'private' : 'public';
 		$response->header(
 			'Cache-Control: ' . $mode . ', s-maxage=' . $smaxage . ', max-age=' . $maxage
 		);
 
-		$text = $this->getRawText();
-
-		if ( $text === false && $contentType == 'text/x-wiki' ) {
-			# Don't return a 404 response for CSS or JavaScript;
-			# 404s aren't generally cached and it would create
-			# extra hits when user CSS/JS are on and the user doesn't
-			# have the pages.
-			$response->header( 'HTTP/1.x 404 Not Found' );
+		// In the event of user JS, don't allow loading a user JS/CSS/Json
+		// subpage that has no registered user associated with, as
+		// someone could register the account and take control of the
+		// JS/CSS/Json page.
+		$title = $this->getTitle();
+		if ( $title->isCssJsSubpage() && $contentType !== 'text/x-wiki' ) {
+			// not using getRootText() as we want this to work
+			// even if subpages are disabled.
+			$rootPage = strtok( $title->getText(), '/' );
+			$userFromTitle = User::newFromName( $rootPage, 'usable' );
+			if ( !$userFromTitle || $userFromTitle->getId() === 0 ) {
+				$log = LoggerFactory::getInstance( "security" );
+				$log->warning(
+					"Unsafe JS/CSS/Json load - {user} loaded {title} with {ctype}",
+					[
+						'user' => $this->getUser()->getName(),
+						'title' => $title->getPrefixedDBKey(),
+						'ctype' => $contentType,
+					]
+				);
+				$msg = wfMessage( 'unregistered-user-config' );
+				throw new HttpError( 403, $msg );
+			}
 		}
 
-		if ( !Hooks::run( 'RawPageViewBeforeOutput', array( &$this, &$text ) ) ) {
+		// Don't allow loading non-protected pages as javascript.
+		// In future we may further restrict this to only CONTENT_MODEL_JAVASCRIPT
+		// in NS_MEDIAWIKI or NS_USER, as well as including other config types,
+		// but for now be more permissive. Allowing protected pages outside of
+		// NS_USER and NS_MEDIAWIKI in particular should be considered a temporary
+		// allowance.
+		if (
+			$contentType === 'text/javascript' &&
+			!$title->isJsSubpage() &&
+			!$title->inNamespace( NS_MEDIAWIKI ) &&
+			!in_array( 'sysop', $title->getRestrictions( 'edit' ) ) &&
+			!in_array( 'editprotected', $title->getRestrictions( 'edit' ) )
+		) {
+
+			$log = LoggerFactory::getInstance( "security" );
+			$log->info( "Blocked loading unprotected JS {title} for {user}",
+				[
+					'user' => $this->getUser()->getName(),
+					'title' => $title->getPrefixedDBKey(),
+				]
+			);
+			throw new HttpError( 403, wfMessage( 'unprotected-js' ) );
+		}
+
+		$response->header( 'Content-type: ' . $contentType . '; charset=UTF-8' );
+
+		$text = $this->getRawText();
+
+		// Don't return a 404 response for CSS or JavaScript;
+		// 404s aren't generally cached and it would create
+		// extra hits when user CSS/JS are on and the user doesn't
+		// have the pages.
+		if ( $text === false && $contentType == 'text/x-wiki' ) {
+			$response->statusHeader( 404 );
+		}
+
+		// Avoid PHP 7.1 warning of passing $this by reference
+		$rawAction = $this;
+		if ( !Hooks::run( 'RawPageViewBeforeOutput', [ &$rawAction, &$text ] ) ) {
 			wfDebug( __METHOD__ . ": RawPageViewBeforeOutput hook broke raw page output.\n" );
 		}
 
@@ -139,11 +178,6 @@ class RawAction extends FormlessAction {
 	public function getRawText() {
 		global $wgParser;
 
-		# No longer used
-		if ( $this->gen ) {
-			return '';
-		}
-
 		$text = false;
 		$title = $this->getTitle();
 		$request = $this->getRequest();
@@ -254,7 +288,7 @@ class RawAction extends FormlessAction {
 			}
 		}
 
-		$allowedCTypes = array( 'text/x-wiki', 'text/javascript', 'text/css', 'application/x-zope-edit' );
+		$allowedCTypes = [ 'text/x-wiki', 'text/javascript', 'text/css', 'application/x-zope-edit' ];
 		if ( $ctype == '' || !in_array( $ctype, $allowedCTypes ) ) {
 			$ctype = 'text/x-wiki';
 		}
