@@ -41,36 +41,43 @@ class ApiQuery extends ApiBase {
 	 * List of Api Query prop modules
 	 * @var array
 	 */
-	private static $QueryPropModules = array(
+	private static $QueryPropModules = [
 		'categories' => 'ApiQueryCategories',
 		'categoryinfo' => 'ApiQueryCategoryInfo',
 		'contributors' => 'ApiQueryContributors',
+		'deletedrevisions' => 'ApiQueryDeletedRevisions',
 		'duplicatefiles' => 'ApiQueryDuplicateFiles',
 		'extlinks' => 'ApiQueryExternalLinks',
+		'fileusage' => 'ApiQueryBacklinksprop',
 		'images' => 'ApiQueryImages',
 		'imageinfo' => 'ApiQueryImageInfo',
 		'info' => 'ApiQueryInfo',
 		'links' => 'ApiQueryLinks',
+		'linkshere' => 'ApiQueryBacklinksprop',
 		'iwlinks' => 'ApiQueryIWLinks',
 		'langlinks' => 'ApiQueryLangLinks',
 		'pageprops' => 'ApiQueryPageProps',
-		'redirects' => 'ApiQueryRedirects',
+		'redirects' => 'ApiQueryBacklinksprop',
 		'revisions' => 'ApiQueryRevisions',
 		'stashimageinfo' => 'ApiQueryStashImageInfo',
 		'templates' => 'ApiQueryLinks',
-	);
+		'transcludedin' => 'ApiQueryBacklinksprop',
+	];
 
 	/**
 	 * List of Api Query list modules
 	 * @var array
 	 */
-	private static $QueryListModules = array(
+	private static $QueryListModules = [
 		'allcategories' => 'ApiQueryAllCategories',
+		'alldeletedrevisions' => 'ApiQueryAllDeletedRevisions',
 		'allfileusages' => 'ApiQueryAllLinks',
 		'allimages' => 'ApiQueryAllImages',
 		'alllinks' => 'ApiQueryAllLinks',
 		'allpages' => 'ApiQueryAllPages',
 		'allredirects' => 'ApiQueryAllLinks',
+		'allrevisions' => 'ApiQueryAllRevisions',
+		'mystashedfiles' => 'ApiQueryMyStashedFiles',
 		'alltransclusions' => 'ApiQueryAllLinks',
 		'allusers' => 'ApiQueryAllUsers',
 		'backlinks' => 'ApiQueryBacklinks',
@@ -97,18 +104,20 @@ class ApiQuery extends ApiBase {
 		'users' => 'ApiQueryUsers',
 		'watchlist' => 'ApiQueryWatchlist',
 		'watchlistraw' => 'ApiQueryWatchlistRaw',
-	);
+	];
 
 	/**
 	 * List of Api Query meta modules
 	 * @var array
 	 */
-	private static $QueryMetaModules = array(
+	private static $QueryMetaModules = [
 		'allmessages' => 'ApiQueryAllMessages',
+		'authmanagerinfo' => 'ApiQueryAuthManagerInfo',
 		'siteinfo' => 'ApiQuerySiteinfo',
 		'userinfo' => 'ApiQueryUserInfo',
 		'filerepoinfo' => 'ApiQueryFileRepoInfo',
-	);
+		'tokens' => 'ApiQueryTokens',
+	];
 
 	/**
 	 * @var ApiPageSet
@@ -116,28 +125,28 @@ class ApiQuery extends ApiBase {
 	private $mPageSet;
 
 	private $mParams;
-	private $mNamedDB = array();
+	private $mNamedDB = [];
 	private $mModuleMgr;
-	private $mGeneratorContinue;
-	private $mUseLegacyContinue;
 
 	/**
-	 * @param $main ApiMain
-	 * @param $action string
+	 * @param ApiMain $main
+	 * @param string $action
 	 */
-	public function __construct( $main, $action ) {
+	public function __construct( ApiMain $main, $action ) {
 		parent::__construct( $main, $action );
 
 		$this->mModuleMgr = new ApiModuleManager( $this );
 
 		// Allow custom modules to be added in LocalSettings.php
-		global $wgAPIPropModules, $wgAPIListModules, $wgAPIMetaModules;
+		$config = $this->getConfig();
 		$this->mModuleMgr->addModules( self::$QueryPropModules, 'prop' );
-		$this->mModuleMgr->addModules( $wgAPIPropModules, 'prop' );
+		$this->mModuleMgr->addModules( $config->get( 'APIPropModules' ), 'prop' );
 		$this->mModuleMgr->addModules( self::$QueryListModules, 'list' );
-		$this->mModuleMgr->addModules( $wgAPIListModules, 'list' );
+		$this->mModuleMgr->addModules( $config->get( 'APIListModules' ), 'list' );
 		$this->mModuleMgr->addModules( self::$QueryMetaModules, 'meta' );
-		$this->mModuleMgr->addModules( $wgAPIMetaModules, 'meta' );
+		$this->mModuleMgr->addModules( $config->get( 'APIMetaModules' ), 'meta' );
+
+		Hooks::run( 'ApiQuery::moduleManager', [ $this->mModuleMgr ] );
 
 		// Create PageSet that will process titles/pageids/revids/generator
 		$this->mPageSet = new ApiPageSet( $this );
@@ -163,9 +172,7 @@ class ApiQuery extends ApiBase {
 	 */
 	public function getNamedDB( $name, $db, $groups ) {
 		if ( !array_key_exists( $name, $this->mNamedDB ) ) {
-			$this->profileDBIn();
 			$this->mNamedDB[$name] = wfGetDB( $db, $groups );
-			$this->profileDBOut();
 		}
 
 		return $this->mNamedDB[$name];
@@ -179,44 +186,6 @@ class ApiQuery extends ApiBase {
 		return $this->mPageSet;
 	}
 
-	/**
-	 * Get the array mapping module names to class names
-	 * @deprecated since 1.21, use getModuleManager()'s methods instead
-	 * @return array array(modulename => classname)
-	 */
-	public function getModules() {
-		wfDeprecated( __METHOD__, '1.21' );
-
-		return $this->getModuleManager()->getNamesWithClasses();
-	}
-
-	/**
-	 * Get the generators array mapping module names to class names
-	 * @deprecated since 1.21, list of generators is maintained by ApiPageSet
-	 * @return array array(modulename => classname)
-	 */
-	public function getGenerators() {
-		wfDeprecated( __METHOD__, '1.21' );
-		$gens = array();
-		foreach ( $this->mModuleMgr->getNamesWithClasses() as $name => $class ) {
-			if ( is_subclass_of( $class, 'ApiQueryGeneratorBase' ) ) {
-				$gens[$name] = $class;
-			}
-		}
-
-		return $gens;
-	}
-
-	/**
-	 * Get whether the specified module is a prop, list or a meta query module
-	 * @deprecated since 1.21, use getModuleManager()->getModuleGroup()
-	 * @param string $moduleName Name of the module to find type for
-	 * @return mixed string or null
-	 */
-	function getModuleType( $moduleName ) {
-		return $this->getModuleManager()->getModuleGroup( $moduleName );
-	}
-
 	/**
 	 * @return ApiFormatRaw|null
 	 */
@@ -245,23 +214,24 @@ class ApiQuery extends ApiBase {
 	public function execute() {
 		$this->mParams = $this->extractRequestParams();
 
-		// $pagesetParams is a array of parameter names used by the pageset generator
-		//   or null if pageset has already finished and is no longer needed
-		// $completeModules is a set of complete modules with the name as key
-		$this->initContinue( $pagesetParams, $completeModules );
-
 		// Instantiate requested modules
-		$allModules = array();
+		$allModules = [];
 		$this->instantiateModules( $allModules, 'prop' );
-		$propModules = $allModules; // Keep a copy
+		$propModules = array_keys( $allModules );
 		$this->instantiateModules( $allModules, 'list' );
 		$this->instantiateModules( $allModules, 'meta' );
 
 		// Filter modules based on continue parameter
-		$modules = $this->initModules( $allModules, $completeModules, $pagesetParams !== null );
+		$continuationManager = new ApiContinuationManager( $this, $allModules, $propModules );
+		$this->setContinuationManager( $continuationManager );
+		$modules = $continuationManager->getRunModules();
 
-		// Execute pageset if in legacy mode or if pageset is not done
-		if ( $completeModules === null || $pagesetParams !== null ) {
+		if ( !$continuationManager->isGeneratorDone() ) {
+			// Query modules may optimize data requests through the $this->getPageSet()
+			// object by adding extra fields from the page table.
+			foreach ( $modules as $module ) {
+				$module->requestExtraData( $this->mPageSet );
+			}
 			// Populate page/revision information
 			$this->mPageSet->execute();
 			// Record page information (title, namespace, if exists, etc)
@@ -278,153 +248,33 @@ class ApiQuery extends ApiBase {
 			$params = $module->extractRequestParams();
 			$cacheMode = $this->mergeCacheMode(
 				$cacheMode, $module->getCacheMode( $params ) );
-			$module->profileIn();
 			$module->execute();
-			wfRunHooks( 'APIQueryAfterExecute', array( &$module ) );
-			$module->profileOut();
+			Hooks::run( 'APIQueryAfterExecute', [ &$module ] );
 		}
 
 		// Set the cache mode
 		$this->getMain()->setCacheMode( $cacheMode );
 
-		if ( $completeModules === null ) {
-			return; // Legacy continue, we are done
-		}
-
-		// Reformat query-continue result section
-		$result = $this->getResult();
-		$qc = $result->getData();
-		if ( isset( $qc['query-continue'] ) ) {
-			$qc = $qc['query-continue'];
-			$result->unsetValue( null, 'query-continue' );
-		} elseif ( $this->mGeneratorContinue !== null ) {
-			$qc = array();
-		} else {
-			// no more "continue"s, we are done!
-			return;
-		}
-
-		// we are done with all the modules that do not have result in query-continue
-		$completeModules = array_merge( $completeModules, array_diff_key( $modules, $qc ) );
-		if ( $pagesetParams !== null ) {
-			// The pageset is still in use, check if all props have finished
-			$incompleteProps = array_intersect_key( $propModules, $qc );
-			if ( count( $incompleteProps ) > 0 ) {
-				// Properties are not done, continue with the same pageset state - copy current parameters
-				$main = $this->getMain();
-				$contValues = array();
-				foreach ( $pagesetParams as $param ) {
-					// The param name is already prefix-encoded
-					$contValues[$param] = $main->getVal( $param );
-				}
-			} elseif ( $this->mGeneratorContinue !== null ) {
-				// Move to the next set of pages produced by pageset, properties need to be restarted
-				$contValues = $this->mGeneratorContinue;
-				$pagesetParams = array_keys( $contValues );
-				$completeModules = array_diff_key( $completeModules, $propModules );
-			} else {
-				// Done with the pageset, finish up with the the lists and meta modules
-				$pagesetParams = null;
-			}
-		}
-
-		$continue = '||' . implode( '|', array_keys( $completeModules ) );
-		if ( $pagesetParams !== null ) {
-			// list of all pageset parameters to use in the next request
-			$continue = implode( '|', $pagesetParams ) . $continue;
-		} else {
-			// we are done with the pageset
-			$contValues = array();
-			$continue = '-' . $continue;
-		}
-		$contValues['continue'] = $continue;
-		foreach ( $qc as $qcModule ) {
-			foreach ( $qcModule as $qcKey => $qcValue ) {
-				$contValues[$qcKey] = $qcValue;
-			}
-		}
-		$this->getResult()->addValue( null, 'continue', $contValues );
-	}
-
-	/**
-	 * Parse 'continue' parameter into the list of complete modules and a list of generator parameters
-	 * @param array|null $pagesetParams returns list of generator params or null if pageset is done
-	 * @param array|null $completeModules returns list of finished modules (as keys), or null if legacy
-	 */
-	private function initContinue( &$pagesetParams, &$completeModules ) {
-		$pagesetParams = array();
-		$continue = $this->mParams['continue'];
-		if ( $continue !== null ) {
-			$this->mUseLegacyContinue = false;
-			if ( $continue !== '' ) {
-				// Format: ' pagesetParam1 | pagesetParam2 || module1 | module2 | module3 | ...
-				// If pageset is done, use '-'
-				$continue = explode( '||', $continue );
-				$this->dieContinueUsageIf( count( $continue ) !== 2 );
-				if ( $continue[0] === '-' ) {
-					$pagesetParams = null; // No need to execute pageset
-				} elseif ( $continue[0] !== '' ) {
-					// list of pageset params that might need to be repeated
-					$pagesetParams = explode( '|', $continue[0] );
-				}
-				$continue = $continue[1];
-			}
-			if ( $continue !== '' ) {
-				$completeModules = array_flip( explode( '|', $continue ) );
-			} else {
-				$completeModules = array();
+		// Write the continuation data into the result
+		$this->setContinuationManager( null );
+		if ( $this->mParams['rawcontinue'] ) {
+			$data = $continuationManager->getRawContinuation();
+			if ( $data ) {
+				$this->getResult()->addValue( null, 'query-continue', $data,
+					ApiResult::ADD_ON_TOP | ApiResult::NO_SIZE_CHECK );
 			}
 		} else {
-			$this->mUseLegacyContinue = true;
-			$completeModules = null;
+			$continuationManager->setContinuationIntoResult( $this->getResult() );
 		}
 	}
 
-	/**
-	 * Validate sub-modules, filter out completed ones, and do requestExtraData()
-	 * @param array $allModules An dict of name=>instance of all modules requested by the client
-	 * @param array|null $completeModules list of finished modules, or null if legacy continue
-	 * @param bool $usePageset True if pageset will be executed
-	 * @return array of modules to be processed during this execution
-	 */
-	private function initModules( $allModules, $completeModules, $usePageset ) {
-		$modules = $allModules;
-		$tmp = $completeModules;
-		$wasPosted = $this->getRequest()->wasPosted();
-
-		/** @var $module ApiQueryBase */
-		foreach ( $allModules as $moduleName => $module ) {
-			if ( !$wasPosted && $module->mustBePosted() ) {
-				$this->dieUsageMsgOrDebug( array( 'mustbeposted', $moduleName ) );
-			}
-			if ( $completeModules !== null && array_key_exists( $moduleName, $completeModules ) ) {
-				// If this module is done, mark all its params as used
-				$module->extractRequestParams();
-				// Make sure this module is not used during execution
-				unset( $modules[$moduleName] );
-				unset( $tmp[$moduleName] );
-			} elseif ( $completeModules === null || $usePageset ) {
-				// Query modules may optimize data requests through the $this->getPageSet()
-				// object by adding extra fields from the page table.
-				// This function will gather all the extra request fields from the modules.
-				$module->requestExtraData( $this->mPageSet );
-			} else {
-				// Error - this prop module must have finished before generator is done
-				$this->dieContinueUsageIf( $this->mModuleMgr->getModuleGroup( $moduleName ) === 'prop' );
-			}
-		}
-		$this->dieContinueUsageIf( $completeModules !== null && count( $tmp ) !== 0 );
-
-		return $modules;
-	}
-
 	/**
 	 * Update a cache mode string, applying the cache mode of a new module to it.
 	 * The cache mode may increase in the level of privacy, but public modules
 	 * added to private data do not decrease the level of privacy.
 	 *
-	 * @param $cacheMode string
-	 * @param $modCacheMode string
+	 * @param string $cacheMode
+	 * @param string $modCacheMode
 	 * @return string
 	 */
 	protected function mergeCacheMode( $cacheMode, $modCacheMode ) {
@@ -443,16 +293,20 @@ class ApiQuery extends ApiBase {
 
 	/**
 	 * Create instances of all modules requested by the client
-	 * @param array $modules to append instantiated modules to
+	 * @param array $modules To append instantiated modules to
 	 * @param string $param Parameter name to read modules from
 	 */
 	private function instantiateModules( &$modules, $param ) {
+		$wasPosted = $this->getRequest()->wasPosted();
 		if ( isset( $this->mParams[$param] ) ) {
 			foreach ( $this->mParams[$param] as $moduleName ) {
 				$instance = $this->mModuleMgr->getModule( $moduleName, $param );
 				if ( $instance === null ) {
 					ApiBase::dieDebug( __METHOD__, 'Error instantiating module' );
 				}
+				if ( !$wasPosted && $instance->mustBePosted() ) {
+					$this->dieUsageMsgOrDebug( [ 'mustbeposted', $moduleName ] );
+				}
 				// Ignore duplicates. TODO 2.0: die()?
 				if ( !array_key_exists( $moduleName, $modules ) ) {
 					$modules[$moduleName] = $instance;
@@ -470,123 +324,113 @@ class ApiQuery extends ApiBase {
 		$pageSet = $this->getPageSet();
 		$result = $this->getResult();
 
-		// We don't check for a full result set here because we can't be adding
-		// more than 380K. The maximum revision size is in the megabyte range,
-		// and the maximum result size must be even higher than that.
+		// We can't really handle max-result-size failure here, but we need to
+		// check anyway in case someone set the limit stupidly low.
+		$fit = true;
 
 		$values = $pageSet->getNormalizedTitlesAsResult( $result );
 		if ( $values ) {
-			$result->addValue( 'query', 'normalized', $values );
+			$fit = $fit && $result->addValue( 'query', 'normalized', $values );
 		}
 		$values = $pageSet->getConvertedTitlesAsResult( $result );
 		if ( $values ) {
-			$result->addValue( 'query', 'converted', $values );
+			$fit = $fit && $result->addValue( 'query', 'converted', $values );
 		}
 		$values = $pageSet->getInterwikiTitlesAsResult( $result, $this->mParams['iwurl'] );
 		if ( $values ) {
-			$result->addValue( 'query', 'interwiki', $values );
+			$fit = $fit && $result->addValue( 'query', 'interwiki', $values );
 		}
 		$values = $pageSet->getRedirectTitlesAsResult( $result );
 		if ( $values ) {
-			$result->addValue( 'query', 'redirects', $values );
+			$fit = $fit && $result->addValue( 'query', 'redirects', $values );
 		}
 		$values = $pageSet->getMissingRevisionIDsAsResult( $result );
 		if ( $values ) {
-			$result->addValue( 'query', 'badrevids', $values );
+			$fit = $fit && $result->addValue( 'query', 'badrevids', $values );
 		}
 
 		// Page elements
-		$pages = array();
+		$pages = [];
 
 		// Report any missing titles
 		foreach ( $pageSet->getMissingTitles() as $fakeId => $title ) {
-			$vals = array();
+			$vals = [];
 			ApiQueryBase::addTitleInfo( $vals, $title );
-			$vals['missing'] = '';
+			$vals['missing'] = true;
 			$pages[$fakeId] = $vals;
 		}
 		// Report any invalid titles
-		foreach ( $pageSet->getInvalidTitles() as $fakeId => $title ) {
-			$pages[$fakeId] = array( 'title' => $title, 'invalid' => '' );
+		foreach ( $pageSet->getInvalidTitlesAndReasons() as $fakeId => $data ) {
+			$pages[$fakeId] = $data + [ 'invalid' => true ];
 		}
 		// Report any missing page ids
 		foreach ( $pageSet->getMissingPageIDs() as $pageid ) {
-			$pages[$pageid] = array(
+			$pages[$pageid] = [
 				'pageid' => $pageid,
-				'missing' => ''
-			);
+				'missing' => true
+			];
 		}
 		// Report special pages
 		/** @var $title Title */
 		foreach ( $pageSet->getSpecialTitles() as $fakeId => $title ) {
-			$vals = array();
+			$vals = [];
 			ApiQueryBase::addTitleInfo( $vals, $title );
-			$vals['special'] = '';
+			$vals['special'] = true;
 			if ( $title->isSpecialPage() &&
 				!SpecialPageFactory::exists( $title->getDBkey() )
 			) {
-				$vals['missing'] = '';
+				$vals['missing'] = true;
 			} elseif ( $title->getNamespace() == NS_MEDIA &&
 				!wfFindFile( $title )
 			) {
-				$vals['missing'] = '';
+				$vals['missing'] = true;
 			}
 			$pages[$fakeId] = $vals;
 		}
 
 		// Output general page information for found titles
 		foreach ( $pageSet->getGoodTitles() as $pageid => $title ) {
-			$vals = array();
+			$vals = [];
 			$vals['pageid'] = $pageid;
 			ApiQueryBase::addTitleInfo( $vals, $title );
 			$pages[$pageid] = $vals;
 		}
 
 		if ( count( $pages ) ) {
+			$pageSet->populateGeneratorData( $pages );
+			ApiResult::setArrayType( $pages, 'BCarray' );
+
 			if ( $this->mParams['indexpageids'] ) {
-				$pageIDs = array_keys( $pages );
+				$pageIDs = array_keys( ApiResult::stripMetadataNonRecursive( $pages ) );
 				// json treats all map keys as strings - converting to match
 				$pageIDs = array_map( 'strval', $pageIDs );
-				$result->setIndexedTagName( $pageIDs, 'id' );
-				$result->addValue( 'query', 'pageids', $pageIDs );
+				ApiResult::setIndexedTagName( $pageIDs, 'id' );
+				$fit = $fit && $result->addValue( 'query', 'pageids', $pageIDs );
 			}
 
-			$result->setIndexedTagName( $pages, 'page' );
-			$result->addValue( 'query', 'pages', $pages );
+			ApiResult::setIndexedTagName( $pages, 'page' );
+			$fit = $fit && $result->addValue( 'query', 'pages', $pages );
 		}
+
+		if ( !$fit ) {
+			$this->dieUsage(
+				'The value of $wgAPIMaxResultSize on this wiki is ' .
+					'too small to hold basic result information',
+				'badconfig'
+			);
+		}
+
 		if ( $this->mParams['export'] ) {
 			$this->doExport( $pageSet, $result );
 		}
 	}
 
 	/**
-	 * This method is called by the generator base when generator in the smart-continue
-	 * mode tries to set 'query-continue' value. ApiQuery stores those values separately
-	 * until the post-processing when it is known if the generation should continue or repeat.
-	 * @param ApiQueryGeneratorBase $module generator module
-	 * @param string $paramName
-	 * @param mixed $paramValue
-	 * @return bool true if processed, false if this is a legacy continue
-	 */
-	public function setGeneratorContinue( $module, $paramName, $paramValue ) {
-		if ( $this->mUseLegacyContinue ) {
-			return false;
-		}
-		$paramName = $module->encodeParamName( $paramName );
-		if ( $this->mGeneratorContinue === null ) {
-			$this->mGeneratorContinue = array();
-		}
-		$this->mGeneratorContinue[$paramName] = $paramValue;
-
-		return true;
-	}
-
-	/**
-	 * @param $pageSet ApiPageSet Pages to be exported
-	 * @param $result ApiResult Result to output to
+	 * @param ApiPageSet $pageSet Pages to be exported
+	 * @param ApiResult $result Result to output to
 	 */
 	private function doExport( $pageSet, $result ) {
-		$exportTitles = array();
+		$exportTitles = [];
 		$titles = $pageSet->getGoodTitles();
 		if ( count( $titles ) ) {
 			$user = $this->getUser();
@@ -613,40 +457,41 @@ class ApiQuery extends ApiBase {
 		// Don't check the size of exported stuff
 		// It's not continuable, so it would cause more
 		// problems than it'd solve
-		$result->disableSizeCheck();
 		if ( $this->mParams['exportnowrap'] ) {
 			$result->reset();
 			// Raw formatter will handle this
-			$result->addValue( null, 'text', $exportxml );
-			$result->addValue( null, 'mime', 'text/xml' );
+			$result->addValue( null, 'text', $exportxml, ApiResult::NO_SIZE_CHECK );
+			$result->addValue( null, 'mime', 'text/xml', ApiResult::NO_SIZE_CHECK );
+			$result->addValue( null, 'filename', 'export.xml', ApiResult::NO_SIZE_CHECK );
 		} else {
-			$r = array();
-			ApiResult::setContent( $r, $exportxml );
-			$result->addValue( 'query', 'export', $r );
+			$result->addValue( 'query', 'export', $exportxml, ApiResult::NO_SIZE_CHECK );
+			$result->addValue( 'query', ApiResult::META_BC_SUBELEMENTS, [ 'export' ] );
 		}
-		$result->enableSizeCheck();
 	}
 
 	public function getAllowedParams( $flags = 0 ) {
-		$result = array(
-			'prop' => array(
+		$result = [
+			'prop' => [
 				ApiBase::PARAM_ISMULTI => true,
-				ApiBase::PARAM_TYPE => $this->mModuleMgr->getNames( 'prop' )
-			),
-			'list' => array(
+				ApiBase::PARAM_TYPE => 'submodule',
+			],
+			'list' => [
 				ApiBase::PARAM_ISMULTI => true,
-				ApiBase::PARAM_TYPE => $this->mModuleMgr->getNames( 'list' )
-			),
-			'meta' => array(
+				ApiBase::PARAM_TYPE => 'submodule',
+			],
+			'meta' => [
 				ApiBase::PARAM_ISMULTI => true,
-				ApiBase::PARAM_TYPE => $this->mModuleMgr->getNames( 'meta' )
-			),
+				ApiBase::PARAM_TYPE => 'submodule',
+			],
 			'indexpageids' => false,
 			'export' => false,
 			'exportnowrap' => false,
 			'iwurl' => false,
-			'continue' => null,
-		);
+			'continue' => [
+				ApiBase::PARAM_HELP_MSG => 'api-help-param-continue',
+			],
+			'rawcontinue' => false,
+		];
 		if ( $flags ) {
 			$result += $this->getPageSet()->getFinalParams( $flags );
 		}
@@ -656,9 +501,11 @@ class ApiQuery extends ApiBase {
 
 	/**
 	 * Override the parent to generate help messages for all available query modules.
+	 * @deprecated since 1.25
 	 * @return string
 	 */
 	public function makeHelpMsg() {
+		wfDeprecated( __METHOD__, '1.25' );
 
 		// Use parent to make default message for the query module
 		$msg = parent::makeHelpMsg();
@@ -678,11 +525,12 @@ class ApiQuery extends ApiBase {
 
 	/**
 	 * For all modules of a given group, generate help messages and join them together
+	 * @deprecated since 1.25
 	 * @param string $group Module group
 	 * @return string
 	 */
 	private function makeHelpMsgHelper( $group ) {
-		$moduleDescriptions = array();
+		$moduleDescriptions = [];
 
 		$moduleNames = $this->mModuleMgr->getNames( $group );
 		sort( $moduleNames );
@@ -706,62 +554,53 @@ class ApiQuery extends ApiBase {
 		return implode( "\n", $moduleDescriptions );
 	}
 
-	public function shouldCheckMaxlag() {
-		return true;
+	public function isReadMode() {
+		// We need to make an exception for certain meta modules that should be
+		// accessible even without the 'read' right. Restrict the exception as
+		// much as possible: no other modules allowed, and no pageset
+		// parameters either. We do allow the 'rawcontinue' and 'indexpageids'
+		// parameters since frameworks might add these unconditionally and they
+		// can't expose anything here.
+		$this->mParams = $this->extractRequestParams();
+		$params = array_filter(
+			array_diff_key(
+				$this->mParams + $this->getPageSet()->extractRequestParams(),
+				[ 'rawcontinue' => 1, 'indexpageids' => 1 ]
+			)
+		);
+		if ( array_keys( $params ) !== [ 'meta' ] ) {
+			return true;
+		}
+
+		// Ask each module if it requires read mode. Any true => this returns
+		// true.
+		$modules = [];
+		$this->instantiateModules( $modules, 'meta' );
+		foreach ( $modules as $module ) {
+			if ( $module->isReadMode() ) {
+				return true;
+			}
+		}
+
+		return false;
 	}
 
-	public function getParamDescription() {
-		return $this->getPageSet()->getFinalParamDescription() + array(
-			'prop' => 'Which properties to get for the titles/revisions/pageids. ' .
-				'Module help is available below',
-			'list' => 'Which lists to get. Module help is available below',
-			'meta' => 'Which metadata to get about the site. Module help is available below',
-			'indexpageids' => 'Include an additional pageids section listing all returned page IDs',
-			'export' => 'Export the current revisions of all given or generated pages',
-			'exportnowrap' => 'Return the export XML without wrapping it in an ' .
-				'XML result (same format as Special:Export). Can only be used with export',
-			'iwurl' => 'Whether to get the full URL if the title is an interwiki link',
-			'continue' => array(
-				'When present, formats query-continue as key-value pairs that ' .
-					'should simply be merged into the original request.',
-				'This parameter must be set to an empty string in the initial query.',
-				'This parameter is recommended for all new development, and ' .
-					'will be made default in the next API version.'
-			),
-		);
-	}
-
-	public function getDescription() {
-		return array(
-			'Query API module allows applications to get needed pieces of data ' .
-				'from the MediaWiki databases,',
-			'and is loosely based on the old query.php interface.',
-			'All data modifications will first have to use query to acquire a ' .
-				'token to prevent abuse from malicious sites.'
-		);
-	}
-
-	public function getPossibleErrors() {
-		return array_merge(
-			parent::getPossibleErrors(),
-			$this->getPageSet()->getFinalPossibleErrors()
-		);
-	}
-
-	public function getExamples() {
-		return array(
-			'api.php?action=query&prop=revisions&meta=siteinfo&' .
-				'titles=Main%20Page&rvprop=user|comment&continue=',
-			'api.php?action=query&generator=allpages&gapprefix=API/&prop=revisions&continue=',
-		);
+	protected function getExamplesMessages() {
+		return [
+			'action=query&prop=revisions&meta=siteinfo&' .
+				'titles=Main%20Page&rvprop=user|comment&continue='
+				=> 'apihelp-query-example-revisions',
+			'action=query&generator=allpages&gapprefix=API/&prop=revisions&continue='
+				=> 'apihelp-query-example-allpages',
+		];
 	}
 
 	public function getHelpUrls() {
-		return array(
+		return [
 			'https://www.mediawiki.org/wiki/API:Query',
 			'https://www.mediawiki.org/wiki/API:Meta',
 			'https://www.mediawiki.org/wiki/API:Properties',
 			'https://www.mediawiki.org/wiki/API:Lists',
-		);
+		];
 	}
 }
