@@ -37,7 +37,7 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 
 	public function execute() {
 		$params = $this->extractRequestParams();
-		$done = array();
+		$done = [];
 		$fit = false;
 		foreach ( $params['prop'] as $p ) {
 			switch ( $p ) {
@@ -108,6 +108,9 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 				case 'defaultoptions':
 					$fit = $this->appendDefaultOptions( $p );
 					break;
+				case 'uploaddialog':
+					$fit = $this->appendUploadDialog( $p );
+					break;
 				default:
 					ApiBase::dieDebug( __METHOD__, "Unknown prop=$p" );
 			}
@@ -127,7 +130,7 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 
 		$config = $this->getConfig();
 
-		$data = array();
+		$data = [];
 		$mainPage = Title::newMainPage();
 		$data['mainpage'] = $mainPage->getPrefixedText();
 		$data['base'] = wfExpandUrl( $mainPage->getFullURL(), PROTO_CURRENT );
@@ -147,7 +150,7 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 		$data['dbtype'] = $config->get( 'DBtype' );
 		$data['dbversion'] = $this->getDB()->getServerVersion();
 
-		$allowFrom = array( '' );
+		$allowFrom = [ '' ];
 		$allowException = true;
 		if ( !$config->get( 'AllowExternalImages' ) ) {
 			$data['imagewhitelistenabled'] = (bool)$config->get( 'EnableImageWhitelist' );
@@ -176,6 +179,7 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 		$data['linktrail'] = $linktrail ?: '';
 
 		$data['legaltitlechars'] = Title::legalChars();
+		$data['invalidusernamechars'] = $config->get( 'InvalidUsernameCharacters' );
 
 		global $IP;
 		$git = SpecialVersion::getGitHeadSha1( $IP );
@@ -183,31 +187,26 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 			$data['git-hash'] = $git;
 			$data['git-branch'] =
 				SpecialVersion::getGitCurrentBranch( $GLOBALS['IP'] );
-		} else {
-			$svn = SpecialVersion::getSvnRevision( $IP );
-			if ( $svn ) {
-				$data['rev'] = $svn;
-			}
 		}
 
 		// 'case-insensitive' option is reserved for future
 		$data['case'] = $config->get( 'CapitalLinks' ) ? 'first-letter' : 'case-sensitive';
 		$data['lang'] = $config->get( 'LanguageCode' );
 
-		$fallbacks = array();
+		$fallbacks = [];
 		foreach ( $wgContLang->getFallbackLanguages() as $code ) {
-			$fallbacks[] = array( 'code' => $code );
+			$fallbacks[] = [ 'code' => $code ];
 		}
 		$data['fallback'] = $fallbacks;
 		ApiResult::setIndexedTagName( $data['fallback'], 'lang' );
 
 		if ( $wgContLang->hasVariants() ) {
-			$variants = array();
+			$variants = [];
 			foreach ( $wgContLang->getVariants() as $code ) {
-				$variants[] = array(
+				$variants[] = [
 					'code' => $code,
 					'name' => $wgContLang->getVariantname( $code ),
-				);
+				];
 			}
 			$data['variants'] = $variants;
 			ApiResult::setIndexedTagName( $data['variants'], 'lang' );
@@ -244,16 +243,18 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 
 		$data['misermode'] = (bool)$config->get( 'MiserMode' );
 
+		$data['uploadsenabled'] = UploadBase::isEnabled();
 		$data['maxuploadsize'] = UploadBase::getMaxUploadSize();
+		$data['minuploadchunksize'] = (int)$this->getConfig()->get( 'MinUploadChunkSize' );
 
 		$data['thumblimits'] = $config->get( 'ThumbLimits' );
 		ApiResult::setArrayType( $data['thumblimits'], 'BCassoc' );
 		ApiResult::setIndexedTagName( $data['thumblimits'], 'limit' );
-		$data['imagelimits'] = array();
+		$data['imagelimits'] = [];
 		ApiResult::setArrayType( $data['imagelimits'], 'BCassoc' );
 		ApiResult::setIndexedTagName( $data['imagelimits'], 'limit' );
 		foreach ( $config->get( 'ImageLimits' ) as $k => $limit ) {
-			$data['imagelimits'][$k] = array( 'width' => $limit[0], 'height' => $limit[1] );
+			$data['imagelimits'][$k] = [ 'width' => $limit[0], 'height' => $limit[1] ];
 		}
 
 		$favicon = $config->get( 'Favicon' );
@@ -263,21 +264,25 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 			$data['favicon'] = wfExpandUrl( $favicon, PROTO_RELATIVE );
 		}
 
-		Hooks::run( 'APIQuerySiteInfoGeneralInfo', array( $this, &$data ) );
+		$data['centralidlookupprovider'] = $this->getConfig()->get( 'CentralIdLookupProvider' );
+		$providerIds = array_keys( $this->getConfig()->get( 'CentralIdLookupProviders' ) );
+		$data['allcentralidlookupproviders'] = $providerIds;
+
+		Hooks::run( 'APIQuerySiteInfoGeneralInfo', [ $this, &$data ] );
 
 		return $this->getResult()->addValue( 'query', $property, $data );
 	}
 
 	protected function appendNamespaces( $property ) {
 		global $wgContLang;
-		$data = array(
+		$data = [
 			ApiResult::META_TYPE => 'assoc',
-		);
+		];
 		foreach ( $wgContLang->getFormattedNamespaces() as $ns => $title ) {
-			$data[$ns] = array(
+			$data[$ns] = [
 				'id' => intval( $ns ),
 				'case' => MWNamespace::isCapitalized( $ns ) ? 'first-letter' : 'case-sensitive',
-			);
+			];
 			ApiResult::setContentValue( $data[$ns], 'name', $title );
 			$canonical = MWNamespace::getCanonicalName( $ns );
 
@@ -296,6 +301,7 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 			}
 		}
 
+		ApiResult::setArrayType( $data, 'assoc' );
 		ApiResult::setIndexedTagName( $data, 'ns' );
 
 		return $this->getResult()->addValue( 'query', $property, $data );
@@ -306,15 +312,15 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 		$aliases = array_merge( $this->getConfig()->get( 'NamespaceAliases' ),
 			$wgContLang->getNamespaceAliases() );
 		$namespaces = $wgContLang->getNamespaces();
-		$data = array();
+		$data = [];
 		foreach ( $aliases as $title => $ns ) {
 			if ( $namespaces[$ns] == $title ) {
 				// Don't list duplicates
 				continue;
 			}
-			$item = array(
+			$item = [
 				'id' => intval( $ns )
-			);
+			];
 			ApiResult::setContentValue( $item, 'alias', strtr( $title, '_', ' ' ) );
 			$data[] = $item;
 		}
@@ -328,11 +334,11 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 
 	protected function appendSpecialPageAliases( $property ) {
 		global $wgContLang;
-		$data = array();
+		$data = [];
 		$aliases = $wgContLang->getSpecialPageAliases();
 		foreach ( SpecialPageFactory::getNames() as $specialpage ) {
 			if ( isset( $aliases[$specialpage] ) ) {
-				$arr = array( 'realname' => $specialpage, 'aliases' => $aliases[$specialpage] );
+				$arr = [ 'realname' => $specialpage, 'aliases' => $aliases[$specialpage] ];
 				ApiResult::setIndexedTagName( $arr['aliases'], 'alias' );
 				$data[] = $arr;
 			}
@@ -344,10 +350,10 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 
 	protected function appendMagicWords( $property ) {
 		global $wgContLang;
-		$data = array();
+		$data = [];
 		foreach ( $wgContLang->getMagicWords() as $magicword => $aliases ) {
 			$caseSensitive = array_shift( $aliases );
-			$arr = array( 'name' => $magicword, 'aliases' => $aliases );
+			$arr = [ 'name' => $magicword, 'aliases' => $aliases ];
 			$arr['case-sensitive'] = (bool)$caseSensitive;
 			ApiResult::setIndexedTagName( $arr['aliases'], 'alias' );
 			$data[] = $arr;
@@ -374,11 +380,11 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 		$getPrefixes = Interwiki::getAllPrefixes( $local );
 		$extraLangPrefixes = $this->getConfig()->get( 'ExtraInterlanguageLinkPrefixes' );
 		$localInterwikis = $this->getConfig()->get( 'LocalInterwikis' );
-		$data = array();
+		$data = [];
 
 		foreach ( $getPrefixes as $row ) {
 			$prefix = $row['iw_prefix'];
-			$val = array();
+			$val = [];
 			$val['prefix'] = $prefix;
 			if ( isset( $row['iw_local'] ) && $row['iw_local'] == '1' ) {
 				$val['local'] = true;
@@ -425,7 +431,7 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 	}
 
 	protected function appendDbReplLagInfo( $property, $includeAll ) {
-		$data = array();
+		$data = [];
 		$lb = wfGetLB();
 		$showHostnames = $this->getConfig()->get( 'ShowHostnames' );
 		if ( $includeAll ) {
@@ -438,29 +444,28 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 
 			$lags = $lb->getLagTimes();
 			foreach ( $lags as $i => $lag ) {
-				$data[] = array(
+				$data[] = [
 					'host' => $lb->getServerName( $i ),
 					'lag' => $lag
-				);
+				];
 			}
 		} else {
 			list( , $lag, $index ) = $lb->getMaxLag();
-			$data[] = array(
+			$data[] = [
 				'host' => $showHostnames
 						? $lb->getServerName( $index )
 						: '',
 				'lag' => intval( $lag )
-			);
+			];
 		}
 
-		$result = $this->getResult();
 		ApiResult::setIndexedTagName( $data, 'db' );
 
 		return $this->getResult()->addValue( 'query', $property, $data );
 	}
 
 	protected function appendStatistics( $property ) {
-		$data = array();
+		$data = [];
 		$data['pages'] = intval( SiteStats::pages() );
 		$data['articles'] = intval( SiteStats::articles() );
 		$data['edits'] = intval( SiteStats::edits() );
@@ -470,7 +475,7 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 		$data['admins'] = intval( SiteStats::numberingroup( 'sysop' ) );
 		$data['jobs'] = intval( SiteStats::jobs() );
 
-		Hooks::run( 'APIQuerySiteInfoStatisticsInfo', array( &$data ) );
+		Hooks::run( 'APIQuerySiteInfoStatisticsInfo', [ &$data ] );
 
 		return $this->getResult()->addValue( 'query', $property, $data );
 	}
@@ -478,14 +483,14 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 	protected function appendUserGroups( $property, $numberInGroup ) {
 		$config = $this->getConfig();
 
-		$data = array();
+		$data = [];
 		$result = $this->getResult();
 		$allGroups = User::getAllGroups();
 		foreach ( $config->get( 'GroupPermissions' ) as $group => $permissions ) {
-			$arr = array(
+			$arr = [
 				'name' => $group,
 				'rights' => array_keys( $permissions, true ),
-			);
+			];
 
 			if ( $numberInGroup ) {
 				$autopromote = $config->get( 'Autopromote' );
@@ -494,22 +499,23 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 					$arr['number'] = SiteStats::users();
 				// '*' and autopromote groups have no size
 				} elseif ( $group !== '*' && !isset( $autopromote[$group] ) ) {
-					$arr['number'] = SiteStats::numberInGroup( $group );
+					$arr['number'] = SiteStats::numberingroup( $group );
 				}
 			}
 
-			$groupArr = array(
+			$groupArr = [
 				'add' => $config->get( 'AddGroups' ),
 				'remove' => $config->get( 'RemoveGroups' ),
 				'add-self' => $config->get( 'GroupsAddToSelf' ),
 				'remove-self' => $config->get( 'GroupsRemoveFromSelf' )
-			);
+			];
 
 			foreach ( $groupArr as $type => $rights ) {
 				if ( isset( $rights[$group] ) ) {
 					$groups = array_intersect( $rights[$group], $allGroups );
 					if ( $groups ) {
 						$arr[$type] = $groups;
+						ApiResult::setArrayType( $arr[$type], 'BCarray' );
 						ApiResult::setIndexedTagName( $arr[$type], 'group' );
 					}
 				}
@@ -525,9 +531,9 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 	}
 
 	protected function appendFileExtensions( $property ) {
-		$data = array();
+		$data = [];
 		foreach ( array_unique( $this->getConfig()->get( 'FileExtensions' ) ) as $ext ) {
-			$data[] = array( 'ext' => $ext );
+			$data[] = [ 'ext' => $ext ];
 		}
 		ApiResult::setIndexedTagName( $data, 'fe' );
 
@@ -536,27 +542,23 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 
 	protected function appendInstalledLibraries( $property ) {
 		global $IP;
-		$path = "$IP/composer.lock";
+		$path = "$IP/vendor/composer/installed.json";
 		if ( !file_exists( $path ) ) {
-			// Maybe they're using mediawiki/vendor?
-			$path = "$IP/vendor/composer.lock";
-			if ( !file_exists( $path ) ) {
-				return true;
-			}
+			return true;
 		}
 
-		$data = array();
-		$lock = new ComposerLock( $path );
-		foreach ( $lock->getInstalledDependencies() as $name => $info ) {
+		$data = [];
+		$installed = new ComposerInstalled( $path );
+		foreach ( $installed->getInstalledDependencies() as $name => $info ) {
 			if ( strpos( $info['type'], 'mediawiki-' ) === 0 ) {
 				// Skip any extensions or skins since they'll be listed
 				// in their proper section
 				continue;
 			}
-			$data[] = array(
+			$data[] = [
 				'name' => $name,
 				'version' => $info['version'],
-			);
+			];
 		}
 		ApiResult::setIndexedTagName( $data, 'library' );
 
@@ -565,10 +567,10 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 	}
 
 	protected function appendExtensions( $property ) {
-		$data = array();
+		$data = [];
 		foreach ( $this->getConfig()->get( 'ExtensionCredits' ) as $type => $extensions ) {
 			foreach ( $extensions as $ext ) {
-				$ret = array();
+				$ret = [];
 				$ret['type'] = $type;
 				if ( isset( $ext['name'] ) ) {
 					$ret['name'] = $ext['name'];
@@ -598,11 +600,6 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 				}
 				if ( isset( $ext['version'] ) ) {
 					$ret['version'] = $ext['version'];
-				} elseif ( isset( $ext['svn-revision'] ) &&
-					preg_match( '/\$(?:Rev|LastChangedRevision|Revision): *(\d+)/',
-						$ext['svn-revision'], $m )
-				) {
-					$ret['version'] = 'r' . $m[1];
 				}
 				if ( isset( $ext['path'] ) ) {
 					$extensionPath = dirname( $ext['path'] );
@@ -616,13 +613,6 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 						if ( $vcsDate !== false ) {
 							$ret['vcs-date'] = wfTimestamp( TS_ISO_8601, $vcsDate );
 						}
-					} else {
-						$svnInfo = SpecialVersion::getSvnInfo( $extensionPath );
-						if ( $svnInfo !== false ) {
-							$ret['vcs-system'] = 'svn';
-							$ret['vcs-version'] = $svnInfo['checkout-rev'];
-							$ret['vcs-url'] = isset( $svnInfo['viewvc-url'] ) ? $svnInfo['viewvc-url'] : '';
-						}
 					}
 
 					if ( SpecialVersion::getExtLicenseFileName( $extensionPath ) ) {
@@ -664,22 +654,27 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 			$text = $title->getPrefixedText();
 		}
 
-		$data = array(
-			'url' => $url ? $url : '',
-			'text' => $text ? $text : ''
-		);
+		$data = [
+			'url' => $url ?: '',
+			'text' => $text ?: ''
+		];
 
 		return $this->getResult()->addValue( 'query', $property, $data );
 	}
 
 	protected function appendRestrictions( $property ) {
 		$config = $this->getConfig();
-		$data = array(
+		$data = [
 			'types' => $config->get( 'RestrictionTypes' ),
 			'levels' => $config->get( 'RestrictionLevels' ),
 			'cascadinglevels' => $config->get( 'CascadingRestrictionLevels' ),
 			'semiprotectedlevels' => $config->get( 'SemiprotectedRestrictionLevels' ),
-		);
+		];
+
+		ApiResult::setArrayType( $data['types'], 'BCarray' );
+		ApiResult::setArrayType( $data['levels'], 'BCarray' );
+		ApiResult::setArrayType( $data['cascadinglevels'], 'BCarray' );
+		ApiResult::setArrayType( $data['semiprotectedlevels'], 'BCarray' );
 
 		ApiResult::setIndexedTagName( $data['types'], 'type' );
 		ApiResult::setIndexedTagName( $data['levels'], 'level' );
@@ -694,10 +689,10 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 		$langCode = isset( $params['inlanguagecode'] ) ? $params['inlanguagecode'] : '';
 		$langNames = Language::fetchLanguageNames( $langCode );
 
-		$data = array();
+		$data = [];
 
 		foreach ( $langNames as $code => $name ) {
-			$lang = array( 'code' => $code );
+			$lang = [ 'code' => $code ];
 			ApiResult::setContentValue( $lang, 'name', $name );
 			$data[] = $lang;
 		}
@@ -707,7 +702,7 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 	}
 
 	public function appendSkins( $property ) {
-		$data = array();
+		$data = [];
 		$allowed = Skin::getAllowedSkins();
 		$default = Skin::normalizeKey( 'default' );
 		foreach ( Skin::getSkinNames() as $name => $displayName ) {
@@ -721,7 +716,7 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 			if ( $msg->exists() ) {
 				$displayName = $msg->text();
 			}
-			$skin = array( 'code' => $name );
+			$skin = [ 'code' => $name ];
 			ApiResult::setContentValue( $skin, 'name', $displayName );
 			if ( !isset( $allowed[$name] ) ) {
 				$skin['unusable'] = true;
@@ -739,7 +734,8 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 	public function appendExtensionTags( $property ) {
 		global $wgParser;
 		$wgParser->firstCallInit();
-		$tags = array_map( array( $this, 'formatParserTags' ), $wgParser->getTags() );
+		$tags = array_map( [ $this, 'formatParserTags' ], $wgParser->getTags() );
+		ApiResult::setArrayType( $tags, 'BCarray' );
 		ApiResult::setIndexedTagName( $tags, 't' );
 
 		return $this->getResult()->addValue( 'query', $property, $tags );
@@ -749,6 +745,7 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 		global $wgParser;
 		$wgParser->firstCallInit();
 		$hooks = $wgParser->getFunctionHooks();
+		ApiResult::setArrayType( $hooks, 'BCarray' );
 		ApiResult::setIndexedTagName( $hooks, 'h' );
 
 		return $this->getResult()->addValue( 'query', $property, $hooks );
@@ -756,6 +753,7 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 
 	public function appendVariables( $property ) {
 		$variables = MagicWord::getVariableIDs();
+		ApiResult::setArrayType( $variables, 'BCarray' );
 		ApiResult::setIndexedTagName( $variables, 'v' );
 
 		return $this->getResult()->addValue( 'query', $property, $variables );
@@ -764,6 +762,7 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 	public function appendProtocols( $property ) {
 		// Make a copy of the global so we don't try to set the _element key of it - bug 45130
 		$protocols = array_values( $this->getConfig()->get( 'UrlProtocols' ) );
+		ApiResult::setArrayType( $protocols, 'BCarray' );
 		ApiResult::setIndexedTagName( $protocols, 'p' );
 
 		return $this->getResult()->addValue( 'query', $property, $protocols );
@@ -775,6 +774,11 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 		return $this->getResult()->addValue( 'query', $property, $options );
 	}
 
+	public function appendUploadDialog( $property ) {
+		$config = $this->getConfig()->get( 'UploadDialog' );
+		return $this->getResult()->addValue( 'query', $property, $config );
+	}
+
 	private function formatParserTags( $item ) {
 		return "<{$item}>";
 	}
@@ -784,13 +788,14 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 		$myWgHooks = $hooks;
 		ksort( $myWgHooks );
 
-		$data = array();
+		$data = [];
 		foreach ( $myWgHooks as $name => $subscribers ) {
-			$arr = array(
+			$arr = [
 				'name' => $name,
-				'subscribers' => array_map( array( 'SpecialVersion', 'arrayToString' ), $subscribers ),
-			);
+				'subscribers' => array_map( [ 'SpecialVersion', 'arrayToString' ], $subscribers ),
+			];
 
+			ApiResult::setArrayType( $arr['subscribers'], 'array' );
 			ApiResult::setIndexedTagName( $arr['subscribers'], 's' );
 			$data[] = $arr;
 		}
@@ -814,11 +819,11 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 	}
 
 	public function getAllowedParams() {
-		return array(
-			'prop' => array(
+		return [
+			'prop' => [
 				ApiBase::PARAM_DFLT => 'general',
 				ApiBase::PARAM_ISMULTI => true,
-				ApiBase::PARAM_TYPE => array(
+				ApiBase::PARAM_TYPE => [
 					'general',
 					'namespaces',
 					'namespacealiases',
@@ -841,32 +846,34 @@ class ApiQuerySiteinfo extends ApiQueryBase {
 					'variables',
 					'protocols',
 					'defaultoptions',
-				)
-			),
-			'filteriw' => array(
-				ApiBase::PARAM_TYPE => array(
+					'uploaddialog',
+				],
+				ApiBase::PARAM_HELP_MSG_PER_VALUE => [],
+			],
+			'filteriw' => [
+				ApiBase::PARAM_TYPE => [
 					'local',
 					'!local',
-				)
-			),
+				]
+			],
 			'showalldb' => false,
 			'numberingroup' => false,
 			'inlanguagecode' => null,
-		);
+		];
 	}
 
 	protected function getExamplesMessages() {
-		return array(
+		return [
 			'action=query&meta=siteinfo&siprop=general|namespaces|namespacealiases|statistics'
 				=> 'apihelp-query+siteinfo-example-simple',
 			'action=query&meta=siteinfo&siprop=interwikimap&sifilteriw=local'
 				=> 'apihelp-query+siteinfo-example-interwiki',
 			'action=query&meta=siteinfo&siprop=dbrepllag&sishowalldb='
 				=> 'apihelp-query+siteinfo-example-replag',
-		);
+		];
 	}
 
 	public function getHelpUrls() {
-		return 'https://www.mediawiki.org/wiki/API:Meta#siteinfo_.2F_si';
+		return 'https://www.mediawiki.org/wiki/API:Siteinfo';
 	}
 }
