@@ -108,12 +108,24 @@ class ApiResult implements ApiSerializable {
 	const META_TYPE = '_type';
 
 	/**
-	 * Key (rather than "name" or other default) for when META_TYPE is 'kvp' or
-	 * 'BCkvp'. Value is string.
+	 * Key for the metadata item whose value specifies the name used for the
+	 * kvp key in the alternative output format with META_TYPE 'kvp' or
+	 * 'BCkvp', i.e. the "name" in <container><item name="key">value</item></container>.
+	 * Value is string.
 	 * @since 1.25
 	 */
 	const META_KVP_KEY_NAME = '_kvpkeyname';
 
+	/**
+	 * Key for the metadata item that indicates that the KVP key should be
+	 * added into an assoc value, i.e. {"key":{"val1":"a","val2":"b"}}
+	 * transforms to {"name":"key","val1":"a","val2":"b"} rather than
+	 * {"name":"key","value":{"val1":"a","val2":"b"}}.
+	 * Value is boolean.
+	 * @since 1.26
+	 */
+	const META_KVP_MERGE = '_kvpmerge';
+
 	/**
 	 * Key for the 'BC bools' metadata item. Value is string[].
 	 * Note no setter is provided.
@@ -132,7 +144,7 @@ class ApiResult implements ApiSerializable {
 	private $errorFormatter;
 
 	// Deprecated fields
-	private $isRawMode, $checkingSize, $mainForContinuation;
+	private $checkingSize, $mainForContinuation;
 
 	/**
 	 * @param int|bool $maxSize Maximum result "size", or false for no limit
@@ -147,7 +159,6 @@ class ApiResult implements ApiSerializable {
 		}
 
 		$this->maxSize = $maxSize;
-		$this->isRawMode = false;
 		$this->checkingSize = true;
 		$this->reset();
 	}
@@ -179,9 +190,9 @@ class ApiResult implements ApiSerializable {
 	 * Clear the current result data.
 	 */
 	public function reset() {
-		$this->data = array(
+		$this->data = [
 			self::META_TYPE => 'assoc', // Usually what's desired
-		);
+		];
 		$this->size = 0;
 	}
 
@@ -238,7 +249,7 @@ class ApiResult implements ApiSerializable {
 	 * @param array $transforms See above
 	 * @return mixed Result data, or null if not found
 	 */
-	public function getResultData( $path = array(), $transforms = array() ) {
+	public function getResultData( $path = [], $transforms = [] ) {
 		$path = (array)$path;
 		if ( !$path ) {
 			return self::applyTransformations( $this->data, $transforms );
@@ -276,7 +287,7 @@ class ApiResult implements ApiSerializable {
 	 * @param int $flags Zero or more OR-ed flags like OVERRIDE | ADD_ON_TOP.
 	 */
 	public static function setValue( array &$arr, $name, $value, $flags = 0 ) {
-		if ( !( $flags & ApiResult::NO_VALIDATE ) ) {
+		if ( ( $flags & ApiResult::NO_VALIDATE ) !== ApiResult::NO_VALIDATE ) {
 			$value = self::validateValue( $value );
 		}
 
@@ -292,7 +303,7 @@ class ApiResult implements ApiSerializable {
 		$exists = isset( $arr[$name] );
 		if ( !$exists || ( $flags & ApiResult::OVERRIDE ) ) {
 			if ( !$exists && ( $flags & ApiResult::ADD_ON_TOP ) ) {
-				$arr = array( $name => $value ) + $arr;
+				$arr = [ $name => $value ] + $arr;
 			} else {
 				$arr[$name] = $value;
 			}
@@ -301,17 +312,22 @@ class ApiResult implements ApiSerializable {
 			if ( !$conflicts ) {
 				$arr[$name] += $value;
 			} else {
-				$keys = join( ', ', array_keys( $conflicts ) );
-				throw new RuntimeException( "Conflicting keys ($keys) when attempting to merge element $name" );
+				$keys = implode( ', ', array_keys( $conflicts ) );
+				throw new RuntimeException(
+					"Conflicting keys ($keys) when attempting to merge element $name"
+				);
 			}
 		} else {
-			throw new RuntimeException( "Attempting to add element $name=$value, existing value is {$arr[$name]}" );
+			throw new RuntimeException(
+				"Attempting to add element $name=$value, existing value is {$arr[$name]}"
+			);
 		}
 	}
 
 	/**
 	 * Validate a value for addition to the result
 	 * @param mixed $value
+	 * @return array|mixed|string
 	 */
 	private static function validateValue( $value ) {
 		global $wgContLang;
@@ -319,12 +335,12 @@ class ApiResult implements ApiSerializable {
 		if ( is_object( $value ) ) {
 			// Note we use is_callable() here instead of instanceof because
 			// ApiSerializable is an informal protocol (see docs there for details).
-			if ( is_callable( array( $value, 'serializeForApiResult' ) ) ) {
+			if ( is_callable( [ $value, 'serializeForApiResult' ] ) ) {
 				$oldValue = $value;
 				$value = $value->serializeForApiResult();
 				if ( is_object( $value ) ) {
 					throw new UnexpectedValueException(
-						get_class( $oldValue ) . "::serializeForApiResult() returned an object of class " .
+						get_class( $oldValue ) . '::serializeForApiResult() returned an object of class ' .
 							get_class( $value )
 					);
 				}
@@ -335,24 +351,28 @@ class ApiResult implements ApiSerializable {
 					return self::validateValue( $value );
 				} catch ( Exception $ex ) {
 					throw new UnexpectedValueException(
-						get_class( $oldValue ) . "::serializeForApiResult() returned an invalid value: " .
+						get_class( $oldValue ) . '::serializeForApiResult() returned an invalid value: ' .
 							$ex->getMessage(),
 						0,
 						$ex
 					);
 				}
-			} elseif ( is_callable( array( $value, '__toString' ) ) ) {
+			} elseif ( is_callable( [ $value, '__toString' ] ) ) {
 				$value = (string)$value;
 			} else {
-				$value = (array)$value + array( self::META_TYPE => 'assoc' );
+				$value = (array)$value + [ self::META_TYPE => 'assoc' ];
 			}
 		}
 		if ( is_array( $value ) ) {
+			// Work around PHP bug 45959 by copying to a temporary
+			// (in this case, foreach gets $k === "1" but $tmp[$k] assigns as if $k === 1)
+			$tmp = [];
 			foreach ( $value as $k => $v ) {
-				$value[$k] = self::validateValue( $v );
+				$tmp[$k] = self::validateValue( $v );
 			}
+			$value = $tmp;
 		} elseif ( is_float( $value ) && !is_finite( $value ) ) {
-			throw new InvalidArgumentException( "Cannot add non-finite floats to ApiResult" );
+			throw new InvalidArgumentException( 'Cannot add non-finite floats to ApiResult' );
 		} elseif ( is_string( $value ) ) {
 			$value = $wgContLang->normalize( $value );
 		} elseif ( $value !== null && !is_scalar( $value ) ) {
@@ -386,6 +406,11 @@ class ApiResult implements ApiSerializable {
 		$arr = &$this->path( $path, ( $flags & ApiResult::ADD_ON_TOP ) ? 'prepend' : 'append' );
 
 		if ( $this->checkingSize && !( $flags & ApiResult::NO_SIZE_CHECK ) ) {
+			// self::valueSize needs the validated value. Then flag
+			// to not re-validate later.
+			$value = self::validateValue( $value );
+			$flags |= ApiResult::NO_VALIDATE;
+
 			$newsize = $this->size + self::valueSize( $value );
 			if ( $this->maxSize !== false && $newsize > $this->maxSize ) {
 				/// @todo Add i18n message when replacing calls to ->setWarning()
@@ -513,7 +538,7 @@ class ApiResult implements ApiSerializable {
 		) {
 			throw new RuntimeException(
 				"Attempting to set content element as $name when " . $arr[self::META_CONTENT] .
-					" is already set as the content element"
+					' is already set as the content element'
 			);
 		}
 		$arr[self::META_CONTENT] = $name;
@@ -703,7 +728,9 @@ class ApiResult implements ApiSerializable {
 	 * @param string $kvpKeyName See ApiResult::META_KVP_KEY_NAME
 	 */
 	public static function setArrayType( array &$arr, $type, $kvpKeyName = null ) {
-		if ( !in_array( $type, array( 'default', 'array', 'assoc', 'kvp', 'BCarray', 'BCassoc', 'BCkvp' ), true ) ) {
+		if ( !in_array( $type, [
+				'default', 'array', 'assoc', 'kvp', 'BCarray', 'BCassoc', 'BCkvp'
+				], true ) ) {
 			throw new InvalidArgumentException( 'Bad type' );
 		}
 		$arr[self::META_TYPE] = $type;
@@ -716,7 +743,7 @@ class ApiResult implements ApiSerializable {
 	 * Set the array data type for a path
 	 * @since 1.25
 	 * @param array|string|null $path See ApiResult::addValue()
-	 * @param string $type See ApiResult::META_TYPE
+	 * @param string $tag See ApiResult::META_TYPE
 	 * @param string $kvpKeyName See ApiResult::META_KVP_KEY_NAME
 	 */
 	public function addArrayType( $path, $tag, $kvpKeyName = null ) {
@@ -744,7 +771,7 @@ class ApiResult implements ApiSerializable {
 	 * Set the array data type for a path recursively
 	 * @since 1.25
 	 * @param array|string|null $path See ApiResult::addValue()
-	 * @param string $type See ApiResult::META_TYPE
+	 * @param string $tag See ApiResult::META_TYPE
 	 * @param string $kvpKeyName See ApiResult::META_KVP_KEY_NAME
 	 */
 	public function addArrayTypeRecursive( $path, $tag, $kvpKeyName = null ) {
@@ -774,7 +801,7 @@ class ApiResult implements ApiSerializable {
 	 *
 	 * @see ApiResult::getResultData()
 	 * @since 1.25
-	 * @param array $data
+	 * @param array $dataIn
 	 * @param array $transforms
 	 * @return array|object
 	 */
@@ -788,14 +815,14 @@ class ApiResult implements ApiSerializable {
 			throw new InvalidArgumentException( __METHOD__ . ':Value for "Types" must be an array' );
 		}
 
-		$metadata = array();
+		$metadata = [];
 		$data = self::stripMetadataNonRecursive( $dataIn, $metadata );
 
 		if ( isset( $transforms['Custom'] ) ) {
 			if ( !is_callable( $transforms['Custom'] ) ) {
 				throw new InvalidArgumentException( __METHOD__ . ': Value for "Custom" must be callable' );
 			}
-			call_user_func_array( $transforms['Custom'], array( &$data, &$metadata ) );
+			call_user_func_array( $transforms['Custom'], [ &$data, &$metadata ] );
 		}
 
 		if ( ( isset( $transforms['BC'] ) || $transformTypes !== null ) &&
@@ -808,7 +835,6 @@ class ApiResult implements ApiSerializable {
 
 		// BC transformations
 		$boolKeys = null;
-		$forceKVP = false;
 		if ( isset( $transforms['BC'] ) ) {
 			if ( !is_array( $transforms['BC'] ) ) {
 				throw new InvalidArgumentException( __METHOD__ . ':Value for "BC" must be an array' );
@@ -816,7 +842,7 @@ class ApiResult implements ApiSerializable {
 			if ( !in_array( 'nobool', $transforms['BC'], true ) ) {
 				$boolKeys = isset( $metadata[self::META_BC_BOOLS] )
 					? array_flip( $metadata[self::META_BC_BOOLS] )
-					: array();
+					: [];
 			}
 
 			if ( !in_array( 'no*', $transforms['BC'], true ) &&
@@ -833,11 +859,11 @@ class ApiResult implements ApiSerializable {
 			) {
 				foreach ( $metadata[self::META_BC_SUBELEMENTS] as $k ) {
 					if ( isset( $data[$k] ) ) {
-						$data[$k] = array(
+						$data[$k] = [
 							'*' => $data[$k],
 							self::META_CONTENT => '*',
 							self::META_TYPE => 'assoc',
-						);
+						];
 					}
 				}
 			}
@@ -879,16 +905,16 @@ class ApiResult implements ApiSerializable {
 		switch ( $strip ) {
 			case 'all':
 			case 'base':
-				$keepMetadata = array();
+				$keepMetadata = [];
 				break;
 			case 'none':
 				$keepMetadata = &$metadata;
 				break;
 			case 'bc':
-				$keepMetadata = array_intersect_key( $metadata, array(
+				$keepMetadata = array_intersect_key( $metadata, [
 					self::META_INDEXED_TAG_NAME => 1,
 					self::META_SUBELEMENTS => 1,
-				) );
+				] );
 				break;
 			default:
 				throw new InvalidArgumentException( __METHOD__ . ': Unknown value for "Strip"' );
@@ -935,19 +961,43 @@ class ApiResult implements ApiSerializable {
 						: $transformTypes['ArmorKVP'];
 					$valKey = isset( $transforms['BC'] ) ? '*' : 'value';
 					$assocAsObject = !empty( $transformTypes['AssocAsObject'] );
+					$merge = !empty( $metadata[self::META_KVP_MERGE] );
 
-					$ret = array();
+					$ret = [];
 					foreach ( $data as $k => $v ) {
-						$item = array(
-							$key => $k,
-							$valKey => $v,
-						);
-						if ( $strip === 'none' ) {
-							$item += array(
-								self::META_PRESERVE_KEYS => array( $key ),
-								self::META_CONTENT => $valKey,
-								self::META_TYPE => 'assoc',
-							);
+						if ( $merge && ( is_array( $v ) || is_object( $v ) ) ) {
+							$vArr = (array)$v;
+							if ( isset( $vArr[self::META_TYPE] ) ) {
+								$mergeType = $vArr[self::META_TYPE];
+							} elseif ( is_object( $v ) ) {
+								$mergeType = 'assoc';
+							} else {
+								$keys = array_keys( $vArr );
+								sort( $keys, SORT_NUMERIC );
+								$mergeType = ( $keys === array_keys( $keys ) ) ? 'array' : 'assoc';
+							}
+						} else {
+							$mergeType = 'n/a';
+						}
+						if ( $mergeType === 'assoc' ) {
+							$item = $vArr + [
+								$key => $k,
+							];
+							if ( $strip === 'none' ) {
+								self::setPreserveKeysList( $item, [ $key ] );
+							}
+						} else {
+							$item = [
+								$key => $k,
+								$valKey => $v,
+							];
+							if ( $strip === 'none' ) {
+								$item += [
+									self::META_PRESERVE_KEYS => [ $key ],
+									self::META_CONTENT => $valKey,
+									self::META_TYPE => 'assoc',
+								];
+							}
 						}
 						$ret[] = $assocAsObject ? (object)$item : $item;
 					}
@@ -981,7 +1031,7 @@ class ApiResult implements ApiSerializable {
 			}
 			$preserveKeys = isset( $data[self::META_PRESERVE_KEYS] )
 				? (array)$data[self::META_PRESERVE_KEYS]
-				: array();
+				: [];
 			foreach ( $data as $k => $v ) {
 				if ( self::isMetadataKey( $k ) && !in_array( $k, $preserveKeys, true ) ) {
 					unset( $data[$k] );
@@ -1009,7 +1059,7 @@ class ApiResult implements ApiSerializable {
 	 */
 	public static function stripMetadataNonRecursive( $data, &$metadata = null ) {
 		if ( !is_array( $metadata ) ) {
-			$metadata = array();
+			$metadata = [];
 		}
 		if ( is_array( $data ) || is_object( $data ) ) {
 			$isObj = is_object( $data );
@@ -1018,7 +1068,7 @@ class ApiResult implements ApiSerializable {
 			}
 			$preserveKeys = isset( $data[self::META_PRESERVE_KEYS] )
 				? (array)$data[self::META_PRESERVE_KEYS]
-				: array();
+				: [];
 			foreach ( $data as $k => $v ) {
 				if ( self::isMetadataKey( $k ) && !in_array( $k, $preserveKeys, true ) ) {
 					$metadata[$k] = $v;
@@ -1035,17 +1085,16 @@ class ApiResult implements ApiSerializable {
 	/**
 	 * Get the 'real' size of a result item. This means the strlen() of the item,
 	 * or the sum of the strlen()s of the elements if the item is an array.
-	 * @note Once the deprecated public self::size is removed, we can rename this back to a less awkward name.
-	 * @param mixed $value
+	 * @note Once the deprecated public self::size is removed, we can rename
+	 *       this back to a less awkward name.
+	 * @param mixed $value Validated value (see self::validateValue())
 	 * @return int
 	 */
 	private static function valueSize( $value ) {
 		$s = 0;
-		if ( is_array( $value ) ||
-			is_object( $value ) && !is_callable( array( $value, '__toString' ) )
-		) {
+		if ( is_array( $value ) ) {
 			foreach ( $value as $k => $v ) {
-				if ( !self::isMetadataKey( $s ) ) {
+				if ( !self::isMetadataKey( $k ) ) {
 					$s += self::valueSize( $v );
 				}
 			}
@@ -1074,21 +1123,21 @@ class ApiResult implements ApiSerializable {
 			if ( !isset( $ret[$k] ) ) {
 				switch ( $create ) {
 					case 'append':
-						$ret[$k] = array();
+						$ret[$k] = [];
 						break;
 					case 'prepend':
-						$ret = array( $k => array() ) + $ret;
+						$ret = [ $k => [] ] + $ret;
 						break;
 					case 'dummy':
-						$tmp = array();
+						$tmp = [];
 						return $tmp;
 					default:
-						$fail = join( '.', array_slice( $path, 0, $i + 1 ) );
+						$fail = implode( '.', array_slice( $path, 0, $i + 1 ) );
 						throw new InvalidArgumentException( "Path $fail does not exist" );
 				}
 			}
 			if ( !is_array( $ret[$k] ) ) {
-				$fail = join( '.', array_slice( $path, 0, $i + 1 ) );
+				$fail = implode( '.', array_slice( $path, 0, $i + 1 ) );
 				throw new InvalidArgumentException( "Path $fail is not an array" );
 			}
 			$ret = &$ret[$k];
@@ -1096,6 +1145,61 @@ class ApiResult implements ApiSerializable {
 		return $ret;
 	}
 
+	/**
+	 * Add the correct metadata to an array of vars we want to export through
+	 * the API.
+	 *
+	 * @param array $vars
+	 * @param bool $forceHash
+	 * @return array
+	 */
+	public static function addMetadataToResultVars( $vars, $forceHash = true ) {
+		// Process subarrays and determine if this is a JS [] or {}
+		$hash = $forceHash;
+		$maxKey = -1;
+		$bools = [];
+		foreach ( $vars as $k => $v ) {
+			if ( is_array( $v ) || is_object( $v ) ) {
+				$vars[$k] = ApiResult::addMetadataToResultVars( (array)$v, is_object( $v ) );
+			} elseif ( is_bool( $v ) ) {
+				// Better here to use real bools even in BC formats
+				$bools[] = $k;
+			}
+			if ( is_string( $k ) ) {
+				$hash = true;
+			} elseif ( $k > $maxKey ) {
+				$maxKey = $k;
+			}
+		}
+		if ( !$hash && $maxKey !== count( $vars ) - 1 ) {
+			$hash = true;
+		}
+
+		// Set metadata appropriately
+		if ( $hash ) {
+			// Get the list of keys we actually care about. Unfortunately, we can't support
+			// certain keys that conflict with ApiResult metadata.
+			$keys = array_diff( array_keys( $vars ), [
+				ApiResult::META_TYPE, ApiResult::META_PRESERVE_KEYS, ApiResult::META_KVP_KEY_NAME,
+				ApiResult::META_INDEXED_TAG_NAME, ApiResult::META_BC_BOOLS
+			] );
+
+			return [
+				ApiResult::META_TYPE => 'kvp',
+				ApiResult::META_KVP_KEY_NAME => 'key',
+				ApiResult::META_PRESERVE_KEYS => $keys,
+				ApiResult::META_BC_BOOLS => $bools,
+				ApiResult::META_INDEXED_TAG_NAME => 'var',
+			] + $vars;
+		} else {
+			return [
+				ApiResult::META_TYPE => 'array',
+				ApiResult::META_BC_BOOLS => $bools,
+				ApiResult::META_INDEXED_TAG_NAME => 'value',
+			] + $vars;
+		}
+	}
+
 	/**@}*/
 
 	/************************************************************************//**
@@ -1104,27 +1208,23 @@ class ApiResult implements ApiSerializable {
 	 */
 
 	/**
-	 * Call this function when special elements such as '_element'
-	 * are needed by the formatter, for example in XML printing.
+	 * Formerly used to enable/disable "raw mode".
 	 * @deprecated since 1.25, you shouldn't have been using it in the first place
 	 * @since 1.23 $flag parameter added
 	 * @param bool $flag Set the raw mode flag to this state
 	 */
 	public function setRawMode( $flag = true ) {
-		// Can't wfDeprecated() here, since we need to set this flag from
-		// ApiMain for BC with stuff using self::getIsRawMode as
-		// "self::getIsXMLMode".
-		$this->isRawMode = $flag;
+		wfDeprecated( __METHOD__, '1.25' );
 	}
 
 	/**
-	 * Returns true whether the formatter requested raw data.
+	 * Returns true, the equivalent of "raw mode" is always enabled now
 	 * @deprecated since 1.25, you shouldn't have been using it in the first place
 	 * @return bool
 	 */
 	public function getIsRawMode() {
-		/// @todo: After Wikibase stops calling this, warn
-		return $this->isRawMode;
+		wfDeprecated( __METHOD__, '1.25' );
+		return true;
 	}
 
 	/**
@@ -1134,11 +1234,11 @@ class ApiResult implements ApiSerializable {
 	 */
 	public function getData() {
 		wfDeprecated( __METHOD__, '1.25' );
-		return $this->getResultData( null, array(
-			'BC' => array(),
-			'Types' => array(),
-			'Strip' => $this->isRawMode ? 'bc' : 'all',
-		) );
+		return $this->getResultData( null, [
+			'BC' => [],
+			'Types' => [],
+			'Strip' => 'all',
+		] );
 	}
 
 	/**
@@ -1176,7 +1276,7 @@ class ApiResult implements ApiSerializable {
 	 */
 	public static function setElement( &$arr, $name, $value, $flags = 0 ) {
 		wfDeprecated( __METHOD__, '1.25' );
-		return self::setValue( $arr, $name, $value, $flags );
+		self::setValue( $arr, $name, $value, $flags );
 	}
 
 	/**
@@ -1198,7 +1298,7 @@ class ApiResult implements ApiSerializable {
 			self::setContentValue( $arr, 'content', $value );
 		} else {
 			if ( !isset( $arr[$subElemName] ) ) {
-				$arr[$subElemName] = array();
+				$arr[$subElemName] = [];
 			}
 			self::setContentValue( $arr[$subElemName], 'content', $value );
 		}
@@ -1274,7 +1374,7 @@ class ApiResult implements ApiSerializable {
 	 * @return array
 	 */
 	public function beginContinuation(
-		$continue, array $allModules = array(), array $generatedModules = array()
+		$continue, array $allModules = [], array $generatedModules = []
 	) {
 		wfDeprecated( __METHOD__, '1.25' );
 		if ( $this->mainForContinuation->getContinuationManager() ) {
@@ -1294,13 +1394,13 @@ class ApiResult implements ApiSerializable {
 			$newCtx = new DerivativeContext( $oldCtx );
 			$newCtx->setRequest( new DerivativeRequest(
 				$oldCtx->getRequest(),
-				array( 'continue' => $continue ) + $oldCtx->getRequest()->getValues(),
+				[ 'continue' => $continue ] + $oldCtx->getRequest()->getValues(),
 				$oldCtx->getRequest()->wasPosted()
 			) );
 			$this->mainForContinuation->setContext( $newCtx );
 			$reset = new ScopedCallback(
-				array( $this->mainForContinuation, 'setContext' ),
-				array( $oldCtx )
+				[ $this->mainForContinuation, 'setContext' ],
+				[ $oldCtx ]
 			);
 		}
 		$manager = new ApiContinuationManager(
@@ -1310,10 +1410,10 @@ class ApiResult implements ApiSerializable {
 
 		$this->mainForContinuation->setContinuationManager( $manager );
 
-		return array(
+		return [
 			$manager->isGeneratorDone(),
 			$manager->getRunModules(),
-		);
+		];
 	}
 
 	/**
@@ -1390,7 +1490,7 @@ class ApiResult implements ApiSerializable {
 	 */
 	public static function size( $value ) {
 		wfDeprecated( __METHOD__, '1.25' );
-		return self::valueSize( $value );
+		return self::valueSize( self::validateValue( $value ) );
 	}
 
 	/**
