@@ -29,30 +29,31 @@ class ChangeTagsRevisionList extends ChangeTagsList {
 	}
 
 	/**
-	 * @param DatabaseBase $db
+	 * @param IDatabase $db
 	 * @return mixed
 	 */
 	public function doQuery( $db ) {
 		$ids = array_map( 'intval', $this->ids );
-		$queryInfo = array(
-			'tables' => array( 'revision', 'user' ),
+		$queryInfo = [
+			'tables' => [ 'revision', 'user' ],
 			'fields' => array_merge( Revision::selectFields(), Revision::selectUserFields() ),
-			'conds' => array(
+			'conds' => [
 				'rev_page' => $this->title->getArticleID(),
 				'rev_id' => $ids,
-			),
-			'options' => array( 'ORDER BY' => 'rev_id DESC' ),
-			'join_conds' => array(
+			],
+			'options' => [ 'ORDER BY' => 'rev_id DESC' ],
+			'join_conds' => [
 				'page' => Revision::pageJoinCond(),
 				'user' => Revision::userJoinCond(),
-			),
-		);
+			],
+		];
 		ChangeTags::modifyDisplayQuery(
 			$queryInfo['tables'],
 			$queryInfo['fields'],
 			$queryInfo['conds'],
 			$queryInfo['join_conds'],
-			$queryInfo['options']
+			$queryInfo['options'],
+			''
 		);
 		return $db->select(
 			$queryInfo['tables'],
