@@ -1,7 +1,5 @@
 <?php
 /**
- * Request-dependant objects containers.
- *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -17,19 +15,20 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  * http://www.gnu.org/copyleft/gpl.html
  *
- * @since 1.19
- *
  * @author Daniel Friesen
  * @file
  */
+use Liuggio\StatsdClient\Factory\StatsdDataFactory;
+use MediaWiki\MediaWikiServices;
 
 /**
  * An IContextSource implementation which will inherit context from another source
  * but allow individual pieces of context to be changed locally
  * eg: A ContextSource that can inherit from the main RequestContext but have
  *     a different Title instance set on it.
+ * @since 1.19
  */
-class DerivativeContext extends ContextSource {
+class DerivativeContext extends ContextSource implements MutableContext {
 	/**
 	 * @var WebRequest
 	 */
@@ -70,6 +69,11 @@ class DerivativeContext extends ContextSource {
 	 */
 	private $config;
 
+	/**
+	 * @var Timing
+	 */
+	private $timing;
+
 	/**
 	 * Constructor
 	 * @param IContextSource $context Context to inherit from
@@ -100,6 +104,30 @@ class DerivativeContext extends ContextSource {
 		}
 	}
 
+	/**
+	 * Get the stats object
+	 *
+	 * @deprecated since 1.27 use a StatsdDataFactory from MediaWikiServices (preferably injected)
+	 *
+	 * @return StatsdDataFactory
+	 */
+	public function getStats() {
+		return MediaWikiServices::getInstance()->getStatsdDataFactory();
+	}
+
+	/**
+	 * Get the timing object
+	 *
+	 * @return Timing
+	 */
+	public function getTiming() {
+		if ( !is_null( $this->timing ) ) {
+			return $this->timing;
+		} else {
+			return $this->getContext()->getTiming();
+		}
+	}
+
 	/**
 	 * Set the WebRequest object
 	 *
@@ -126,19 +154,15 @@ class DerivativeContext extends ContextSource {
 	 * Set the Title object
 	 *
 	 * @param Title $t
-	 * @throws MWException
 	 */
-	public function setTitle( $t ) {
-		if ( $t !== null && !$t instanceof Title ) {
-			throw new MWException( __METHOD__ . " expects an instance of Title" );
-		}
+	public function setTitle( Title $t ) {
 		$this->title = $t;
 	}
 
 	/**
 	 * Get the Title object
 	 *
-	 * @return Title
+	 * @return Title|null
 	 */
 	public function getTitle() {
 		if ( !is_null( $this->title ) ) {
@@ -237,17 +261,6 @@ class DerivativeContext extends ContextSource {
 		}
 	}
 
-	/**
-	 * Set the Language object
-	 *
-	 * @deprecated since 1.19 Use setLanguage instead
-	 * @param Language|string $l Language instance or language code
-	 */
-	public function setLang( $l ) {
-		wfDeprecated( __METHOD__, '1.19' );
-		$this->setLanguage( $l );
-	}
-
 	/**
 	 * Set the Language object
 	 *
@@ -267,15 +280,6 @@ class DerivativeContext extends ContextSource {
 		}
 	}
 
-	/**
-	 * @deprecated since 1.19 Use getLanguage instead
-	 * @return Language
-	 */
-	public function getLang() {
-		wfDeprecated( __METHOD__, '1.19' );
-		$this->getLanguage();
-	}
-
 	/**
 	 * Get the Language object
 	 *
@@ -320,8 +324,7 @@ class DerivativeContext extends ContextSource {
 	 * it would set only the original context, and not take
 	 * into account any changes.
 	 *
-	 * @param String Message name
-	 * @param Variable number of message arguments
+	 * @param mixed $args,... Arguments to wfMessage
 	 * @return Message
 	 */
 	public function msg() {
