@@ -78,6 +78,10 @@ class DBExpectedError extends DBError {
 		return $s;
 	}
 
+	function getPageTitle() {
+		return $this->msg( 'databaseerror', 'Database error' );
+	}
+
 	/**
 	 * @return string
 	 */
@@ -133,21 +137,17 @@ class DBConnectionError extends DBExpectedError {
 	 * @return string Unprocessed plain error text with parameters replaced
 	 */
 	function msg( $key, $fallback /*[, params...] */ ) {
-		global $wgLang;
-
 		$args = array_slice( func_get_args(), 2 );
 
 		if ( $this->useMessageCache() ) {
-			$message = $wgLang->getMessage( $key );
+			return wfMessage( $key, $args )->useDatabase( false )->text();
 		} else {
-			$message = $fallback;
+			return wfMsgReplaceArgs( $fallback, $args );
 		}
-
-		return wfMsgReplaceArgs( $message, $args );
 	}
 
 	/**
-	 * @return boolean
+	 * @return bool
 	 */
 	function isLoggable() {
 		// Don't send to the exception log, already in dberror log
@@ -171,13 +171,13 @@ class DBConnectionError extends DBExpectedError {
 
 		if ( $wgShowHostnames || $wgShowSQLErrors ) {
 			$info = str_replace(
-				'$1', Html::element( 'span', array( 'dir' => 'ltr' ), $this->error ),
-				htmlspecialchars( $this->msg( 'dberr-info', '(Cannot contact the database server: $1)' ) )
+				'$1', Html::element( 'span', [ 'dir' => 'ltr' ], $this->error ),
+				htmlspecialchars( $this->msg( 'dberr-info', '(Cannot access the database: $1)' ) )
 			);
 		} else {
 			$info = htmlspecialchars( $this->msg(
 				'dberr-info-hidden',
-				'(Cannot contact the database server)'
+				'(Cannot access the database)'
 			) );
 		}
 
@@ -191,7 +191,6 @@ class DBConnectionError extends DBExpectedError {
 		}
 
 		$html .= '<hr />';
-		$html .= $this->searchForm();
 
 		return $html;
 	}
@@ -221,7 +220,7 @@ class DBConnectionError extends DBExpectedError {
 				// Cached version on file system?
 				if ( $cache !== null ) {
 					// Hack: extend the body for error messages
-					$cache = str_replace( array( '</html>', '</body>' ), '', $cache );
+					$cache = str_replace( [ '</html>', '</body>' ], '', $cache );
 					// Add cache notice...
 					$cache .= '<div style="border:1px solid #ffd0d0;padding:1em;">' .
 						htmlspecialchars( $this->msg( 'dberr-cachederror',
@@ -233,7 +232,7 @@ class DBConnectionError extends DBExpectedError {
 
 					return;
 				}
-			} catch ( MWException $e ) {
+			} catch ( Exception $e ) {
 				// Do nothing, just use the default page
 			}
 		}
@@ -242,49 +241,6 @@ class DBConnectionError extends DBExpectedError {
 		parent::reportHTML();
 	}
 
-	/**
-	 * @return string
-	 */
-	function searchForm() {
-		global $wgSitename, $wgCanonicalServer, $wgRequest;
-
-		$usegoogle = htmlspecialchars( $this->msg(
-			'dberr-usegoogle',
-			'You can try searching via Google in the meantime.'
-		) );
-		$outofdate = htmlspecialchars( $this->msg(
-			'dberr-outofdate',
-			'Note that their indexes of our content may be out of date.'
-		) );
-		$googlesearch = htmlspecialchars( $this->msg( 'searchbutton', 'Search' ) );
-
-		$search = htmlspecialchars( $wgRequest->getVal( 'search' ) );
-
-		$server = htmlspecialchars( $wgCanonicalServer );
-		$sitename = htmlspecialchars( $wgSitename );
-
-		$trygoogle = <<<EOT
-<div style="margin: 1.5em">$usegoogle<br />
-<small>$outofdate</small>
-</div>
-<form method="get" action="//www.google.com/search" id="googlesearch">
-	<input type="hidden" name="domains" value="$server" />
-	<input type="hidden" name="num" value="50" />
-	<input type="hidden" name="ie" value="UTF-8" />
-	<input type="hidden" name="oe" value="UTF-8" />
-
-	<input type="text" name="q" size="31" maxlength="255" value="$search" />
-	<input type="submit" name="btnG" value="$googlesearch" />
-	<p>
-		<label><input type="radio" name="sitesearch" value="$server" checked="checked" />$sitename</label>
-		<label><input type="radio" name="sitesearch" value="" />WWW</label>
-	</p>
-</form>
-EOT;
-
-		return $trygoogle;
-	}
-
 	/**
 	 * @return string
 	 */
@@ -310,7 +266,7 @@ EOT;
 			}
 		}
 
-		$cache = HTMLFileCache::newFromTitle( $t, 'view' );
+		$cache = new HTMLFileCache( $t, 'view' );
 		if ( $cache->isCached() ) {
 			return $cache->fetchText();
 		} else {
@@ -333,12 +289,19 @@ class DBQueryError extends DBExpectedError {
 	 * @param string $fname
 	 */
 	function __construct( DatabaseBase $db, $error, $errno, $sql, $fname ) {
-		$message = "A database error has occurred. Did you forget to run " .
-			"maintenance/update.php after upgrading?  See: " .
-			"https://www.mediawiki.org/wiki/Manual:Upgrading#Run_the_update_script\n" .
-			"Query: $sql\n" .
-			"Function: $fname\n" .
-			"Error: $errno $error\n";
+		if ( $db->wasConnectionError( $errno ) ) {
+			$message = "A connection error occured. \n" .
+				"Query: $sql\n" .
+				"Function: $fname\n" .
+				"Error: $errno $error\n";
+		} else {
+			$message = "A database error has occurred. Did you forget to run " .
+				"maintenance/update.php after upgrading?  See: " .
+				"https://www.mediawiki.org/wiki/Manual:Upgrading#Run_the_update_script\n" .
+				"Query: $sql\n" .
+				"Function: $fname\n" .
+				"Error: $errno $error\n";
+		}
 		parent::__construct( $db, $message );
 
 		$this->error = $error;
@@ -347,14 +310,6 @@ class DBQueryError extends DBExpectedError {
 		$this->fname = $fname;
 	}
 
-	/**
-	 * @return bool
-	 */
-	function isLoggable() {
-		// Don't send to the exception log, already in dberror log
-		return false;
-	}
-
 	/**
 	 * @return string
 	 */
@@ -367,7 +322,7 @@ class DBQueryError extends DBExpectedError {
 	 */
 	protected function getHTMLContent() {
 		$key = 'databaseerror-text';
-		$s = Html::element( 'p', array(), $this->msg( $key, $this->getFallbackMessage( $key ) ) );
+		$s = Html::element( 'p', [], $this->msg( $key, $this->getFallbackMessage( $key ) ) );
 
 		$details = $this->getTechnicalDetails();
 		if ( $details ) {
@@ -375,7 +330,7 @@ class DBQueryError extends DBExpectedError {
 			foreach ( $details as $key => $detail ) {
 				$s .= str_replace(
 					'$1', call_user_func_array( 'Html::element', $detail ),
-					Html::element( 'li', array(),
+					Html::element( 'li', [],
 						$this->msg( $key, $this->getFallbackMessage( $key ) )
 					)
 				);
@@ -416,18 +371,18 @@ class DBQueryError extends DBExpectedError {
 	protected function getTechnicalDetails() {
 		global $wgShowHostnames, $wgShowSQLErrors;
 
-		$attribs = array( 'dir' => 'ltr' );
-		$details = array();
+		$attribs = [ 'dir' => 'ltr' ];
+		$details = [];
 
 		if ( $wgShowSQLErrors ) {
-			$details['databaseerror-query'] = array(
-				'div', array( 'class' => 'mw-code' ) + $attribs, $this->sql );
+			$details['databaseerror-query'] = [
+				'div', [ 'class' => 'mw-code' ] + $attribs, $this->sql ];
 		}
 
 		if ( $wgShowHostnames || $wgShowSQLErrors ) {
 			$errorMessage = $this->errno . ' ' . $this->error;
-			$details['databaseerror-function'] = array( 'code', $attribs, $this->fname );
-			$details['databaseerror-error'] = array( 'samp', $attribs, $errorMessage );
+			$details['databaseerror-function'] = [ 'code', $attribs, $this->fname ];
+			$details['databaseerror-error'] = [ 'samp', $attribs, $errorMessage ];
 		}
 
 		return $details;
@@ -438,14 +393,14 @@ class DBQueryError extends DBExpectedError {
 	 * @return string English message text
 	 */
 	private function getFallbackMessage( $key ) {
-		$messages = array(
+		$messages = [
 			'databaseerror-text' => 'A database query error has occurred.
 This may indicate a bug in the software.',
 			'databaseerror-textcl' => 'A database query error has occurred.',
 			'databaseerror-query' => 'Query: $1',
 			'databaseerror-function' => 'Function: $1',
 			'databaseerror-error' => 'Error: $1',
-		);
+		];
 
 		return $messages[$key];
 	}
@@ -456,3 +411,18 @@ This may indicate a bug in the software.',
  */
 class DBUnexpectedError extends DBError {
 }
+
+/**
+ * @ingroup Database
+ */
+class DBReadOnlyError extends DBExpectedError {
+	function getPageTitle() {
+		return $this->msg( 'readonly', 'Database is locked' );
+	}
+}
+
+/**
+ * @ingroup Database
+ */
+class DBTransactionError extends DBExpectedError {
+}
