@@ -25,13 +25,13 @@
  *
  * @todo document (e.g. one-sentence top-level class description).
  */
-class LinksUpdate extends SqlDataUpdate {
+class LinksUpdate extends SqlDataUpdate implements EnqueueableDataUpdate {
 	// @todo make members protected, but make sure extensions don't break
 
 	/** @var int Page ID of the article linked from */
 	public $mId;
 
-	/** @var Title object of the article linked from */
+	/** @var Title Title object of the article linked from */
 	public $mTitle;
 
 	/** @var ParserOutput */
@@ -52,21 +52,21 @@ class LinksUpdate extends SqlDataUpdate {
 	/** @var array Map of category names to sort keys */
 	public $mCategories;
 
-	/** @var array ap of language codes to titles */
+	/** @var array Map of language codes to titles */
 	public $mInterlangs;
 
+	/** @var array 2-D map of (prefix => DBK => 1) */
+	public $mInterwikis;
+
 	/** @var array Map of arbitrary name to value */
 	public $mProperties;
 
-	/** @var DatabaseBase Database connection reference */
-	public $mDb;
-
-	/** @var array SELECT options to be used */
-	public $mOptions;
-
 	/** @var bool Whether to queue jobs for recursive updates */
 	public $mRecursive;
 
+	/** @var Revision Revision for which this update has been triggered */
+	private $mRevision;
+
 	/**
 	 * @var null|array Added links if calculated.
 	 */
@@ -77,6 +77,11 @@ class LinksUpdate extends SqlDataUpdate {
 	 */
 	private $linkDeletions = null;
 
+	/**
+	 * @var User|null
+	 */
+	private $user;
+
 	/**
 	 * Constructor
 	 *
@@ -85,25 +90,16 @@ class LinksUpdate extends SqlDataUpdate {
 	 * @param bool $recursive Queue jobs for recursive updates?
 	 * @throws MWException
 	 */
-	function __construct( $title, $parserOutput, $recursive = true ) {
+	function __construct( Title $title, ParserOutput $parserOutput, $recursive = true ) {
 		parent::__construct( false ); // no implicit transaction
 
-		if ( !( $title instanceof Title ) ) {
-			throw new MWException( "The calling convention to LinksUpdate::LinksUpdate() has changed. " .
-				"Please see Article::editUpdates() for an invocation example.\n" );
-		}
-
-		if ( !( $parserOutput instanceof ParserOutput ) ) {
-			throw new MWException( "The calling convention to LinksUpdate::__construct() has changed. " .
-				"Please see WikiPage::doEditUpdates() for an invocation example.\n" );
-		}
-
 		$this->mTitle = $title;
-		$this->mId = $title->getArticleID();
+		$this->mId = $title->getArticleID( Title::GAID_FOR_UPDATE );
 
 		if ( !$this->mId ) {
-			throw new MWException( "The Title object did not provide an article " .
-				"ID. Perhaps the page doesn't exist?" );
+			throw new InvalidArgumentException(
+				"The Title object yields no ID. Perhaps the page doesn't exist?"
+			);
 		}
 
 		$this->mParserOutput = $parserOutput;
@@ -121,7 +117,7 @@ class LinksUpdate extends SqlDataUpdate {
 		# the way back to the skin, so either a skin API break would be required, or an
 		# inefficient back-conversion.
 		$ill = $parserOutput->getLanguageLinks();
-		$this->mInterlangs = array();
+		$this->mInterlangs = [];
 		foreach ( $ill as $link ) {
 			list( $key, $title ) = explode( ':', $link, 2 );
 			$this->mInterlangs[$key] = $title;
@@ -140,21 +136,28 @@ class LinksUpdate extends SqlDataUpdate {
 
 		$this->mRecursive = $recursive;
 
-		wfRunHooks( 'LinksUpdateConstructed', array( &$this ) );
+		// Avoid PHP 7.1 warning from passing $this by reference
+		$linksUpdate = $this;
+		Hooks::run( 'LinksUpdateConstructed', [ &$linksUpdate ] );
 	}
 
 	/**
 	 * Update link tables with outgoing links from an updated article
 	 */
 	public function doUpdate() {
-		wfRunHooks( 'LinksUpdate', array( &$this ) );
+		// Avoid PHP 7.1 warning from passing $this by reference
+		$linksUpdate = $this;
+		Hooks::run( 'LinksUpdate', [ &$linksUpdate ] );
 		$this->doIncrementalUpdate();
-		wfRunHooks( 'LinksUpdateComplete', array( &$this ) );
+
+		$this->mDb->onTransactionIdle( function() {
+			// Avoid PHP 7.1 warning from passing $this by reference
+			$linksUpdate = $this;
+			Hooks::run( 'LinksUpdateComplete', [ &$linksUpdate ] );
+		} );
 	}
 
 	protected function doIncrementalUpdate() {
-		wfProfileIn( __METHOD__ );
-
 		# Page links
 		$existing = $this->getExistingLinks();
 		$this->linkDeletions = $this->getLinkDeletions( $existing );
@@ -227,7 +230,6 @@ class LinksUpdate extends SqlDataUpdate {
 			$this->queueRecursiveJobs();
 		}
 
-		wfProfileOut( __METHOD__ );
 	}
 
 	/**
@@ -236,12 +238,24 @@ class LinksUpdate extends SqlDataUpdate {
 	 * Which means do LinksUpdate on all pages that include the current page,
 	 * using the job queue.
 	 */
-	function queueRecursiveJobs() {
+	protected function queueRecursiveJobs() {
 		self::queueRecursiveJobsForTable( $this->mTitle, 'templatelinks' );
 		if ( $this->mTitle->getNamespace() == NS_FILE ) {
 			// Process imagelinks in case the title is or was a redirect
 			self::queueRecursiveJobsForTable( $this->mTitle, 'imagelinks' );
 		}
+
+		$bc = $this->mTitle->getBacklinkCache();
+		// Get jobs for cascade-protected backlinks for a high priority queue.
+		// If meta-templates change to using a new template, the new template
+		// should be implicitly protected as soon as possible, if applicable.
+		// These jobs duplicate a subset of the above ones, but can run sooner.
+		// Which ever runs first generally no-ops the other one.
+		$jobs = [];
+		foreach ( $bc->getCascadeProtectedLinks() as $title ) {
+			$jobs[] = RefreshLinksJob::newPrioritized( $title, [] );
+		}
+		JobQueueGroup::singleton()->push( $jobs );
 	}
 
 	/**
@@ -251,25 +265,23 @@ class LinksUpdate extends SqlDataUpdate {
 	 * @param string $table Table to use (e.g. 'templatelinks')
 	 */
 	public static function queueRecursiveJobsForTable( Title $title, $table ) {
-		wfProfileIn( __METHOD__ );
 		if ( $title->getBacklinkCache()->hasLinks( $table ) ) {
 			$job = new RefreshLinksJob(
 				$title,
-				array(
+				[
 					'table' => $table,
 					'recursive' => true,
-				) + Job::newRootJobParams( // "overall" refresh links job info
+				] + Job::newRootJobParams( // "overall" refresh links job info
 					"refreshlinks:{$table}:{$title->getPrefixedText()}"
 				)
 			);
+
 			JobQueueGroup::singleton()->push( $job );
-			JobQueueGroup::singleton()->deduplicateRootJob( $job );
 		}
-		wfProfileOut( __METHOD__ );
 	}
 
 	/**
-	 * @param $cats
+	 * @param array $cats
 	 */
 	function invalidateCategories( $cats ) {
 		$this->invalidatePages( NS_CATEGORY, array_keys( $cats ) );
@@ -288,7 +300,7 @@ class LinksUpdate extends SqlDataUpdate {
 	}
 
 	/**
-	 * @param $images
+	 * @param array $images
 	 */
 	function invalidateImageDescriptions( $images ) {
 		$this->invalidatePages( NS_FILE, array_keys( $images ) );
@@ -307,7 +319,7 @@ class LinksUpdate extends SqlDataUpdate {
 		} else {
 			$fromField = "{$prefix}_from";
 		}
-		$where = array( $fromField => $this->mId );
+		$where = [ $fromField => $this->mId ];
 		if ( $table == 'pagelinks' || $table == 'templatelinks' || $table == 'iwlinks' ) {
 			if ( $table == 'iwlinks' ) {
 				$baseKey = 'iwl_prefix';
@@ -339,7 +351,7 @@ class LinksUpdate extends SqlDataUpdate {
 		}
 		if ( count( $insertions ) ) {
 			$this->mDb->insert( $table, $insertions, __METHOD__, 'IGNORE' );
-			wfRunHooks( 'LinksUpdateAfterInsert', array( $this, $table, $insertions ) );
+			Hooks::run( 'LinksUpdateAfterInsert', [ $this, $table, $insertions ] );
 		}
 	}
 
@@ -349,18 +361,19 @@ class LinksUpdate extends SqlDataUpdate {
 	 * @param array $existing
 	 * @return array
 	 */
-	private function getLinkInsertions( $existing = array() ) {
-		$arr = array();
+	private function getLinkInsertions( $existing = [] ) {
+		$arr = [];
 		foreach ( $this->mLinks as $ns => $dbkeys ) {
 			$diffs = isset( $existing[$ns] )
 				? array_diff_key( $dbkeys, $existing[$ns] )
 				: $dbkeys;
 			foreach ( $diffs as $dbk => $id ) {
-				$arr[] = array(
+				$arr[] = [
 					'pl_from' => $this->mId,
+					'pl_from_namespace' => $this->mTitle->getNamespace(),
 					'pl_namespace' => $ns,
 					'pl_title' => $dbk
-				);
+				];
 			}
 		}
 
@@ -372,16 +385,17 @@ class LinksUpdate extends SqlDataUpdate {
 	 * @param array $existing
 	 * @return array
 	 */
-	private function getTemplateInsertions( $existing = array() ) {
-		$arr = array();
+	private function getTemplateInsertions( $existing = [] ) {
+		$arr = [];
 		foreach ( $this->mTemplates as $ns => $dbkeys ) {
 			$diffs = isset( $existing[$ns] ) ? array_diff_key( $dbkeys, $existing[$ns] ) : $dbkeys;
 			foreach ( $diffs as $dbk => $id ) {
-				$arr[] = array(
+				$arr[] = [
 					'tl_from' => $this->mId,
+					'tl_from_namespace' => $this->mTitle->getNamespace(),
 					'tl_namespace' => $ns,
 					'tl_title' => $dbk
-				);
+				];
 			}
 		}
 
@@ -394,14 +408,15 @@ class LinksUpdate extends SqlDataUpdate {
 	 * @param array $existing
 	 * @return array
 	 */
-	private function getImageInsertions( $existing = array() ) {
-		$arr = array();
+	private function getImageInsertions( $existing = [] ) {
+		$arr = [];
 		$diffs = array_diff_key( $this->mImages, $existing );
 		foreach ( $diffs as $iname => $dummy ) {
-			$arr[] = array(
+			$arr[] = [
 				'il_from' => $this->mId,
+				'il_from_namespace' => $this->mTitle->getNamespace(),
 				'il_to' => $iname
-			);
+			];
 		}
 
 		return $arr;
@@ -412,17 +427,17 @@ class LinksUpdate extends SqlDataUpdate {
 	 * @param array $existing
 	 * @return array
 	 */
-	private function getExternalInsertions( $existing = array() ) {
-		$arr = array();
+	private function getExternalInsertions( $existing = [] ) {
+		$arr = [];
 		$diffs = array_diff_key( $this->mExternals, $existing );
 		foreach ( $diffs as $url => $dummy ) {
 			foreach ( wfMakeUrlIndexes( $url ) as $index ) {
-				$arr[] = array(
+				$arr[] = [
 					'el_id' => $this->mDb->nextSequenceValue( 'externallinks_el_id_seq' ),
 					'el_from' => $this->mId,
 					'el_to' => $url,
 					'el_index' => $index,
-				);
+				];
 			}
 		}
 
@@ -432,15 +447,15 @@ class LinksUpdate extends SqlDataUpdate {
 	/**
 	 * Get an array of category insertions
 	 *
-	 * @param array $existing mapping existing category names to sort keys. If both
+	 * @param array $existing Mapping existing category names to sort keys. If both
 	 * match a link in $this, the link will be omitted from the output
 	 *
 	 * @return array
 	 */
-	private function getCategoryInsertions( $existing = array() ) {
+	private function getCategoryInsertions( $existing = [] ) {
 		global $wgContLang, $wgCategoryCollation;
 		$diffs = array_diff_assoc( $this->mCategories, $existing );
-		$arr = array();
+		$arr = [];
 		foreach ( $diffs as $name => $prefix ) {
 			$nt = Title::makeTitleSafe( NS_CATEGORY, $name );
 			$wgContLang->findVariantLink( $name, $nt, true );
@@ -460,7 +475,7 @@ class LinksUpdate extends SqlDataUpdate {
 			$sortkey = Collation::singleton()->getSortKey(
 				$this->mTitle->getCategorySortkey( $prefix ) );
 
-			$arr[] = array(
+			$arr[] = [
 				'cl_from' => $this->mId,
 				'cl_to' => $name,
 				'cl_sortkey' => $sortkey,
@@ -468,7 +483,7 @@ class LinksUpdate extends SqlDataUpdate {
 				'cl_sortkey_prefix' => $prefix,
 				'cl_collation' => $wgCategoryCollation,
 				'cl_type' => $type,
-			);
+			];
 		}
 
 		return $arr;
@@ -477,19 +492,19 @@ class LinksUpdate extends SqlDataUpdate {
 	/**
 	 * Get an array of interlanguage link insertions
 	 *
-	 * @param array $existing mapping existing language codes to titles
+	 * @param array $existing Mapping existing language codes to titles
 	 *
 	 * @return array
 	 */
-	private function getInterlangInsertions( $existing = array() ) {
+	private function getInterlangInsertions( $existing = [] ) {
 		$diffs = array_diff_assoc( $this->mInterlangs, $existing );
-		$arr = array();
+		$arr = [];
 		foreach ( $diffs as $lang => $title ) {
-			$arr[] = array(
+			$arr[] = [
 				'll_from' => $this->mId,
 				'll_lang' => $lang,
 				'll_title' => $title
-			);
+			];
 		}
 
 		return $arr;
@@ -500,39 +515,90 @@ class LinksUpdate extends SqlDataUpdate {
 	 * @param array $existing
 	 * @return array
 	 */
-	function getPropertyInsertions( $existing = array() ) {
+	function getPropertyInsertions( $existing = [] ) {
 		$diffs = array_diff_assoc( $this->mProperties, $existing );
-		$arr = array();
-		foreach ( $diffs as $name => $value ) {
-			$arr[] = array(
-				'pp_page' => $this->mId,
-				'pp_propname' => $name,
-				'pp_value' => $value,
-			);
+
+		$arr = [];
+		foreach ( array_keys( $diffs ) as $name ) {
+			$arr[] = $this->getPagePropRowData( $name );
 		}
 
 		return $arr;
 	}
 
+	/**
+	 * Returns an associative array to be used for inserting a row into
+	 * the page_props table. Besides the given property name, this will
+	 * include the page id from $this->mId and any property value from
+	 * $this->mProperties.
+	 *
+	 * The array returned will include the pp_sortkey field if this
+	 * is present in the database (as indicated by $wgPagePropsHaveSortkey).
+	 * The sortkey value is currently determined by getPropertySortKeyValue().
+	 *
+	 * @note this assumes that $this->mProperties[$prop] is defined.
+	 *
+	 * @param string $prop The name of the property.
+	 *
+	 * @return array
+	 */
+	private function getPagePropRowData( $prop ) {
+		global $wgPagePropsHaveSortkey;
+
+		$value = $this->mProperties[$prop];
+
+		$row = [
+			'pp_page' => $this->mId,
+			'pp_propname' => $prop,
+			'pp_value' => $value,
+		];
+
+		if ( $wgPagePropsHaveSortkey ) {
+			$row['pp_sortkey'] = $this->getPropertySortKeyValue( $value );
+		}
+
+		return $row;
+	}
+
+	/**
+	 * Determines the sort key for the given property value.
+	 * This will return $value if it is a float or int,
+	 * 1 or resp. 0 if it is a bool, and null otherwise.
+	 *
+	 * @note In the future, we may allow the sortkey to be specified explicitly
+	 *       in ParserOutput::setProperty.
+	 *
+	 * @param mixed $value
+	 *
+	 * @return float|null
+	 */
+	private function getPropertySortKeyValue( $value ) {
+		if ( is_int( $value ) || is_float( $value ) || is_bool( $value ) ) {
+			return floatval( $value );
+		}
+
+		return null;
+	}
+
 	/**
 	 * Get an array of interwiki insertions for passing to the DB
 	 * Skips the titles specified by the 2-D array $existing
 	 * @param array $existing
 	 * @return array
 	 */
-	private function getInterwikiInsertions( $existing = array() ) {
-		$arr = array();
+	private function getInterwikiInsertions( $existing = [] ) {
+		$arr = [];
 		foreach ( $this->mInterwikis as $prefix => $dbkeys ) {
 			$diffs = isset( $existing[$prefix] )
 				? array_diff_key( $dbkeys, $existing[$prefix] )
 				: $dbkeys;
 
 			foreach ( $diffs as $dbk => $id ) {
-				$arr[] = array(
+				$arr[] = [
 					'iwl_from' => $this->mId,
 					'iwl_prefix' => $prefix,
 					'iwl_title' => $dbk
-				);
+				];
 			}
 		}
 
@@ -546,7 +612,7 @@ class LinksUpdate extends SqlDataUpdate {
 	 * @return array
 	 */
 	private function getLinkDeletions( $existing ) {
-		$del = array();
+		$del = [];
 		foreach ( $existing as $ns => $dbkeys ) {
 			if ( isset( $this->mLinks[$ns] ) ) {
 				$del[$ns] = array_diff_key( $existing[$ns], $this->mLinks[$ns] );
@@ -565,7 +631,7 @@ class LinksUpdate extends SqlDataUpdate {
 	 * @return array
 	 */
 	private function getTemplateDeletions( $existing ) {
-		$del = array();
+		$del = [];
 		foreach ( $existing as $ns => $dbkeys ) {
 			if ( isset( $this->mTemplates[$ns] ) ) {
 				$del[$ns] = array_diff_key( $existing[$ns], $this->mTemplates[$ns] );
@@ -633,7 +699,7 @@ class LinksUpdate extends SqlDataUpdate {
 	 * @return array
 	 */
 	private function getInterwikiDeletions( $existing ) {
-		$del = array();
+		$del = [];
 		foreach ( $existing as $prefix => $dbkeys ) {
 			if ( isset( $this->mInterwikis[$prefix] ) ) {
 				$del[$prefix] = array_diff_key( $existing[$prefix], $this->mInterwikis[$prefix] );
@@ -651,12 +717,12 @@ class LinksUpdate extends SqlDataUpdate {
 	 * @return array
 	 */
 	private function getExistingLinks() {
-		$res = $this->mDb->select( 'pagelinks', array( 'pl_namespace', 'pl_title' ),
-			array( 'pl_from' => $this->mId ), __METHOD__, $this->mOptions );
-		$arr = array();
+		$res = $this->mDb->select( 'pagelinks', [ 'pl_namespace', 'pl_title' ],
+			[ 'pl_from' => $this->mId ], __METHOD__, $this->mOptions );
+		$arr = [];
 		foreach ( $res as $row ) {
 			if ( !isset( $arr[$row->pl_namespace] ) ) {
-				$arr[$row->pl_namespace] = array();
+				$arr[$row->pl_namespace] = [];
 			}
 			$arr[$row->pl_namespace][$row->pl_title] = 1;
 		}
@@ -670,12 +736,12 @@ class LinksUpdate extends SqlDataUpdate {
 	 * @return array
 	 */
 	private function getExistingTemplates() {
-		$res = $this->mDb->select( 'templatelinks', array( 'tl_namespace', 'tl_title' ),
-			array( 'tl_from' => $this->mId ), __METHOD__, $this->mOptions );
-		$arr = array();
+		$res = $this->mDb->select( 'templatelinks', [ 'tl_namespace', 'tl_title' ],
+			[ 'tl_from' => $this->mId ], __METHOD__, $this->mOptions );
+		$arr = [];
 		foreach ( $res as $row ) {
 			if ( !isset( $arr[$row->tl_namespace] ) ) {
-				$arr[$row->tl_namespace] = array();
+				$arr[$row->tl_namespace] = [];
 			}
 			$arr[$row->tl_namespace][$row->tl_title] = 1;
 		}
@@ -689,9 +755,9 @@ class LinksUpdate extends SqlDataUpdate {
 	 * @return array
 	 */
 	private function getExistingImages() {
-		$res = $this->mDb->select( 'imagelinks', array( 'il_to' ),
-			array( 'il_from' => $this->mId ), __METHOD__, $this->mOptions );
-		$arr = array();
+		$res = $this->mDb->select( 'imagelinks', [ 'il_to' ],
+			[ 'il_from' => $this->mId ], __METHOD__, $this->mOptions );
+		$arr = [];
 		foreach ( $res as $row ) {
 			$arr[$row->il_to] = 1;
 		}
@@ -705,9 +771,9 @@ class LinksUpdate extends SqlDataUpdate {
 	 * @return array
 	 */
 	private function getExistingExternals() {
-		$res = $this->mDb->select( 'externallinks', array( 'el_to' ),
-			array( 'el_from' => $this->mId ), __METHOD__, $this->mOptions );
-		$arr = array();
+		$res = $this->mDb->select( 'externallinks', [ 'el_to' ],
+			[ 'el_from' => $this->mId ], __METHOD__, $this->mOptions );
+		$arr = [];
 		foreach ( $res as $row ) {
 			$arr[$row->el_to] = 1;
 		}
@@ -721,9 +787,9 @@ class LinksUpdate extends SqlDataUpdate {
 	 * @return array
 	 */
 	private function getExistingCategories() {
-		$res = $this->mDb->select( 'categorylinks', array( 'cl_to', 'cl_sortkey_prefix' ),
-			array( 'cl_from' => $this->mId ), __METHOD__, $this->mOptions );
-		$arr = array();
+		$res = $this->mDb->select( 'categorylinks', [ 'cl_to', 'cl_sortkey_prefix' ],
+			[ 'cl_from' => $this->mId ], __METHOD__, $this->mOptions );
+		$arr = [];
 		foreach ( $res as $row ) {
 			$arr[$row->cl_to] = $row->cl_sortkey_prefix;
 		}
@@ -738,9 +804,9 @@ class LinksUpdate extends SqlDataUpdate {
 	 * @return array
 	 */
 	private function getExistingInterlangs() {
-		$res = $this->mDb->select( 'langlinks', array( 'll_lang', 'll_title' ),
-			array( 'll_from' => $this->mId ), __METHOD__, $this->mOptions );
-		$arr = array();
+		$res = $this->mDb->select( 'langlinks', [ 'll_lang', 'll_title' ],
+			[ 'll_from' => $this->mId ], __METHOD__, $this->mOptions );
+		$arr = [];
 		foreach ( $res as $row ) {
 			$arr[$row->ll_lang] = $row->ll_title;
 		}
@@ -753,12 +819,12 @@ class LinksUpdate extends SqlDataUpdate {
 	 * @return array (prefix => array(dbkey => 1))
 	 */
 	protected function getExistingInterwikis() {
-		$res = $this->mDb->select( 'iwlinks', array( 'iwl_prefix', 'iwl_title' ),
-			array( 'iwl_from' => $this->mId ), __METHOD__, $this->mOptions );
-		$arr = array();
+		$res = $this->mDb->select( 'iwlinks', [ 'iwl_prefix', 'iwl_title' ],
+			[ 'iwl_from' => $this->mId ], __METHOD__, $this->mOptions );
+		$arr = [];
 		foreach ( $res as $row ) {
 			if ( !isset( $arr[$row->iwl_prefix] ) ) {
-				$arr[$row->iwl_prefix] = array();
+				$arr[$row->iwl_prefix] = [];
 			}
 			$arr[$row->iwl_prefix][$row->iwl_title] = 1;
 		}
@@ -769,12 +835,12 @@ class LinksUpdate extends SqlDataUpdate {
 	/**
 	 * Get an array of existing categories, with the name in the key and sort key in the value.
 	 *
-	 * @return array of property names and values
+	 * @return array Array of property names and values
 	 */
 	private function getExistingProperties() {
-		$res = $this->mDb->select( 'page_props', array( 'pp_propname', 'pp_value' ),
-			array( 'pp_page' => $this->mId ), __METHOD__, $this->mOptions );
-		$arr = array();
+		$res = $this->mDb->select( 'page_props', [ 'pp_propname', 'pp_value' ],
+			[ 'pp_page' => $this->mId ], __METHOD__, $this->mOptions );
+		$arr = [];
 		foreach ( $res as $row ) {
 			$arr[$row->pp_propname] = $row->pp_value;
 		}
@@ -807,6 +873,35 @@ class LinksUpdate extends SqlDataUpdate {
 		return $this->mImages;
 	}
 
+	/**
+	 * Set the revision corresponding to this LinksUpdate
+	 *
+	 * @since 1.27
+	 *
+	 * @param Revision $revision
+	 */
+	public function setRevision( Revision $revision ) {
+		$this->mRevision = $revision;
+	}
+
+	/**
+	 * Set the User who triggered this LinksUpdate
+	 *
+	 * @since 1.27
+	 * @param User $user
+	 */
+	public function setTriggeringUser( User $user ) {
+		$this->user = $user;
+	}
+
+	/**
+	 * @since 1.27
+	 * @return null|User
+	 */
+	public function getTriggeringUser() {
+		return $this->user;
+	}
+
 	/**
 	 * Invalidate any necessary link lists related to page property changes
 	 * @param array $changed
@@ -818,11 +913,10 @@ class LinksUpdate extends SqlDataUpdate {
 			if ( isset( $wgPagePropLinkInvalidations[$name] ) ) {
 				$inv = $wgPagePropLinkInvalidations[$name];
 				if ( !is_array( $inv ) ) {
-					$inv = array( $inv );
+					$inv = [ $inv ];
 				}
 				foreach ( $inv as $table ) {
-					$update = new HTMLCacheUpdate( $this->mTitle, $table );
-					$update->doUpdate();
+					DeferredUpdates::addUpdate( new HTMLCacheUpdate( $this->mTitle, $table ) );
 				}
 			}
 		}
@@ -831,13 +925,13 @@ class LinksUpdate extends SqlDataUpdate {
 	/**
 	 * Fetch page links added by this LinksUpdate.  Only available after the update is complete.
 	 * @since 1.22
-	 * @return null|array of Titles
+	 * @return null|array Array of Titles
 	 */
 	public function getAddedLinks() {
 		if ( $this->linkInsertions === null ) {
 			return null;
 		}
-		$result = array();
+		$result = [];
 		foreach ( $this->linkInsertions as $insertion ) {
 			$result[] = Title::makeTitle( $insertion['pl_namespace'], $insertion['pl_title'] );
 		}
@@ -848,13 +942,13 @@ class LinksUpdate extends SqlDataUpdate {
 	/**
 	 * Fetch page links removed by this LinksUpdate.  Only available after the update is complete.
 	 * @since 1.22
-	 * @return null|array of Titles
+	 * @return null|array Array of Titles
 	 */
 	public function getRemovedLinks() {
 		if ( $this->linkDeletions === null ) {
 			return null;
 		}
-		$result = array();
+		$result = [];
 		foreach ( $this->linkDeletions as $ns => $titles ) {
 			foreach ( $titles as $title => $unused ) {
 				$result[] = Title::makeTitle( $ns, $title );
@@ -872,94 +966,43 @@ class LinksUpdate extends SqlDataUpdate {
 			// The link updates made here only reflect the freshness of the parser output
 			$timestamp = $this->mParserOutput->getCacheTime();
 			$this->mDb->update( 'page',
-				array( 'page_links_updated' => $this->mDb->timestamp( $timestamp ) ),
-				array( 'page_id' => $this->mId ),
+				[ 'page_links_updated' => $this->mDb->timestamp( $timestamp ) ],
+				[ 'page_id' => $this->mId ],
 				__METHOD__
 			);
 		}
 	}
-}
 
-/**
- * Update object handling the cleanup of links tables after a page was deleted.
- **/
-class LinksDeletionUpdate extends SqlDataUpdate {
-	/** @var WikiPage The WikiPage that was deleted */
-	protected $mPage;
-
-	/**
-	 * Constructor
-	 *
-	 * @param WikiPage $page Page we are updating
-	 * @throws MWException
-	 */
-	function __construct( WikiPage $page ) {
-		parent::__construct( false ); // no implicit transaction
-
-		$this->mPage = $page;
-
-		if ( !$page->exists() ) {
-			throw new MWException( "Page ID not known, perhaps the page doesn't exist?" );
-		}
-	}
-
-	/**
-	 * Do some database updates after deletion
-	 */
-	public function doUpdate() {
-		$title = $this->mPage->getTitle();
-		$id = $this->mPage->getId();
-
-		# Delete restrictions for it
-		$this->mDb->delete( 'page_restrictions', array( 'pr_page' => $id ), __METHOD__ );
-
-		# Fix category table counts
-		$cats = array();
-		$res = $this->mDb->select( 'categorylinks', 'cl_to', array( 'cl_from' => $id ), __METHOD__ );
-
-		foreach ( $res as $row ) {
-			$cats[] = $row->cl_to;
+	public function getAsJobSpecification() {
+		if ( $this->user ) {
+			$userInfo = [
+				'userId' => $this->user->getId(),
+				'userName' => $this->user->getName(),
+			];
+		} else {
+			$userInfo = false;
 		}
 
-		$this->mPage->updateCategoryCounts( array(), $cats );
-
-		# If using cascading deletes, we can skip some explicit deletes
-		if ( !$this->mDb->cascadingDeletes() ) {
-			# Delete outgoing links
-			$this->mDb->delete( 'pagelinks', array( 'pl_from' => $id ), __METHOD__ );
-			$this->mDb->delete( 'imagelinks', array( 'il_from' => $id ), __METHOD__ );
-			$this->mDb->delete( 'categorylinks', array( 'cl_from' => $id ), __METHOD__ );
-			$this->mDb->delete( 'templatelinks', array( 'tl_from' => $id ), __METHOD__ );
-			$this->mDb->delete( 'externallinks', array( 'el_from' => $id ), __METHOD__ );
-			$this->mDb->delete( 'langlinks', array( 'll_from' => $id ), __METHOD__ );
-			$this->mDb->delete( 'iwlinks', array( 'iwl_from' => $id ), __METHOD__ );
-			$this->mDb->delete( 'redirect', array( 'rd_from' => $id ), __METHOD__ );
-			$this->mDb->delete( 'page_props', array( 'pp_page' => $id ), __METHOD__ );
+		if ( $this->mRevision ) {
+			$triggeringRevisionId = $this->mRevision->getId();
+		} else {
+			$triggeringRevisionId = false;
 		}
 
-		# If using cleanup triggers, we can skip some manual deletes
-		if ( !$this->mDb->cleanupTriggers() ) {
-			# Clean up recentchanges entries...
-			$this->mDb->delete( 'recentchanges',
-				array( 'rc_type != ' . RC_LOG,
-					'rc_namespace' => $title->getNamespace(),
-					'rc_title' => $title->getDBkey() ),
-				__METHOD__ );
-			$this->mDb->delete( 'recentchanges',
-				array( 'rc_type != ' . RC_LOG, 'rc_cur_id' => $id ),
-				__METHOD__ );
-		}
-	}
-
-	/**
-	 * Update all the appropriate counts in the category table.
-	 * @param array $added Associative array of category name => sort key
-	 * @param array $deleted Associative array of category name => sort key
-	 */
-	function updateCategoryCounts( $added, $deleted ) {
-		$a = WikiPage::factory( $this->mTitle );
-		$a->updateCategoryCounts(
-			array_keys( $added ), array_keys( $deleted )
-		);
+		return [
+			'wiki' => $this->mDb->getWikiID(),
+			'job'  => new JobSpecification(
+				'refreshLinksPrioritized',
+				[
+					// Reuse the parser cache if it was saved
+					'rootJobTimestamp' => $this->mParserOutput->getCacheTime(),
+					'useRecursiveLinksUpdate' => $this->mRecursive,
+					'triggeringUser' => $userInfo,
+					'triggeringRevisionId' => $triggeringRevisionId,
+				],
+				[ 'removeDuplicates' => true ],
+				$this->getTitle()
+			)
+		];
 	}
 }
