@@ -27,17 +27,37 @@
  * @ingroup FileRepo
  */
 class ForeignDBRepo extends LocalRepo {
-	# Settings
-	var $dbType, $dbServer, $dbUser, $dbPassword, $dbName, $dbFlags,
-		$tablePrefix, $hasSharedCache;
+	/** @var string */
+	protected $dbType;
+
+	/** @var string */
+	protected $dbServer;
+
+	/** @var string */
+	protected $dbUser;
+
+	/** @var string */
+	protected $dbPassword;
+
+	/** @var string */
+	protected $dbName;
+
+	/** @var string */
+	protected $dbFlags;
+
+	/** @var string */
+	protected $tablePrefix;
+
+	/** @var bool */
+	protected $hasSharedCache;
 
 	# Other stuff
-	var $dbConn;
-	var $fileFactory = array( 'ForeignDBFile', 'newFromTitle' );
-	var $fileFromRowFactory = array( 'ForeignDBFile', 'newFromRow' );
+	protected $dbConn;
+	protected $fileFactory = [ 'ForeignDBFile', 'newFromTitle' ];
+	protected $fileFromRowFactory = [ 'ForeignDBFile', 'newFromRow' ];
 
 	/**
-	 * @param $info array|null
+	 * @param array|null $info
 	 */
 	function __construct( $info ) {
 		parent::__construct( $info );
@@ -52,32 +72,44 @@ class ForeignDBRepo extends LocalRepo {
 	}
 
 	/**
-	 * @return DatabaseBase
+	 * @return IDatabase
 	 */
 	function getMasterDB() {
 		if ( !isset( $this->dbConn ) ) {
-			$this->dbConn = DatabaseBase::factory( $this->dbType,
-				array(
-					'host' => $this->dbServer,
-					'user' => $this->dbUser,
-					'password' => $this->dbPassword,
-					'dbname' => $this->dbName,
-					'flags' => $this->dbFlags,
-					'tablePrefix' => $this->tablePrefix,
-					'foreign' => true,
-				)
-			);
+			$func = $this->getDBFactory();
+			$this->dbConn = $func( DB_MASTER );
 		}
+
 		return $this->dbConn;
 	}
 
 	/**
-	 * @return DatabaseBase
+	 * @return IDatabase
 	 */
 	function getSlaveDB() {
 		return $this->getMasterDB();
 	}
 
+	/**
+	 * @return Closure
+	 */
+	protected function getDBFactory() {
+		$type = $this->dbType;
+		$params = [
+			'host' => $this->dbServer,
+			'user' => $this->dbUser,
+			'password' => $this->dbPassword,
+			'dbname' => $this->dbName,
+			'flags' => $this->dbFlags,
+			'tablePrefix' => $this->tablePrefix,
+			'foreign' => true,
+		];
+
+		return function ( $index ) use ( $type, $params ) {
+			return DatabaseBase::factory( $type, $params );
+		};
+	}
+
 	/**
 	 * @return bool
 	 */
@@ -95,6 +127,7 @@ class ForeignDBRepo extends LocalRepo {
 		if ( $this->hasSharedCache() ) {
 			$args = func_get_args();
 			array_unshift( $args, $this->dbName, $this->tablePrefix );
+
 			return call_user_func_array( 'wfForeignMemcKey', $args );
 		} else {
 			return false;
@@ -104,4 +137,14 @@ class ForeignDBRepo extends LocalRepo {
 	protected function assertWritableRepo() {
 		throw new MWException( get_class( $this ) . ': write operations are not supported.' );
 	}
+
+	/**
+	 * Return information about the repository.
+	 *
+	 * @return array
+	 * @since 1.22
+	 */
+	function getInfo() {
+		return FileRepo::getInfo();
+	}
 }
