@@ -14,6 +14,16 @@ class HTMLInfoField extends HTMLFormField {
 		return !empty( $this->mParams['raw'] ) ? $value : htmlspecialchars( $value );
 	}
 
+	public function getInputOOUI( $value ) {
+		if ( !empty( $this->mParams['raw'] ) ) {
+			$value = new OOUI\HtmlSnippet( $value );
+		}
+
+		return new OOUI\LabelWidget( [
+			'label' => $value,
+		] );
+	}
+
 	public function getTableRow( $value ) {
 		if ( !empty( $this->mParams['rawrow'] ) ) {
 			return $value;
@@ -23,6 +33,8 @@ class HTMLInfoField extends HTMLFormField {
 	}
 
 	/**
+	 * @param string $value
+	 * @return string
 	 * @since 1.20
 	 */
 	public function getDiv( $value ) {
@@ -34,6 +46,8 @@ class HTMLInfoField extends HTMLFormField {
 	}
 
 	/**
+	 * @param string $value
+	 * @return string
 	 * @since 1.20
 	 */
 	public function getRaw( $value ) {
