@@ -31,26 +31,27 @@ require_once __DIR__ . '/../../maintenance/Maintenance.php';
  * @since 1.17
  */
 abstract class DatabaseUpdater {
+	protected static $updateCounter = 0;
 
 	/**
 	 * Array of updates to perform on the database
 	 *
 	 * @var array
 	 */
-	protected $updates = array();
+	protected $updates = [];
 
 	/**
 	 * Array of updates that were skipped
 	 *
 	 * @var array
 	 */
-	protected $updatesSkipped = array();
+	protected $updatesSkipped = [];
 
 	/**
 	 * List of extension-provided database updates
 	 * @var array
 	 */
-	protected $extensionUpdates = array();
+	protected $extensionUpdates = [];
 
 	/**
 	 * Handle to the database subclass
@@ -62,17 +63,19 @@ abstract class DatabaseUpdater {
 	protected $shared = false;
 
 	/**
-	 * Scripts to run after database update
+	 * @var string[] Scripts to run after database update
 	 * Should be a subclass of LoggedUpdateMaintenance
 	 */
-	protected $postDatabaseUpdateMaintenance = array(
-		'DeleteDefaultMessages',
-		'PopulateRevisionLength',
-		'PopulateRevisionSha1',
-		'PopulateImageSha1',
-		'FixExtLinksProtocolRelative',
-		'PopulateFilearchiveSha1',
-	);
+	protected $postDatabaseUpdateMaintenance = [
+		DeleteDefaultMessages::class,
+		PopulateRevisionLength::class,
+		PopulateRevisionSha1::class,
+		PopulateImageSha1::class,
+		FixExtLinksProtocolRelative::class,
+		PopulateFilearchiveSha1::class,
+		PopulateBacklinkNamespace::class,
+		FixDefaultJsonContentPages::class,
+	];
 
 	/**
 	 * File handle for SQL output.
@@ -113,7 +116,7 @@ abstract class DatabaseUpdater {
 		$this->maintenance->setDB( $db );
 		$this->initOldGlobals();
 		$this->loadExtensions();
-		wfRunHooks( 'LoadExtensionSchemaUpdates', array( $this ) );
+		Hooks::run( 'LoadExtensionSchemaUpdates', [ $this ] );
 	}
 
 	/**
@@ -126,12 +129,12 @@ abstract class DatabaseUpdater {
 
 		# For extensions only, should be populated via hooks
 		# $wgDBtype should be checked to specifiy the proper file
-		$wgExtNewTables = array(); // table, dir
-		$wgExtNewFields = array(); // table, column, dir
-		$wgExtPGNewFields = array(); // table, column, column attributes; for PostgreSQL
-		$wgExtPGAlteredFields = array(); // table, column, new type, conversion method; for PostgreSQL
-		$wgExtNewIndexes = array(); // table, index, dir
-		$wgExtModifiedFields = array(); // table, index, dir
+		$wgExtNewTables = []; // table, dir
+		$wgExtNewFields = []; // table, column, dir
+		$wgExtPGNewFields = []; // table, column, column attributes; for PostgreSQL
+		$wgExtPGAlteredFields = []; // table, column, new type, conversion method; for PostgreSQL
+		$wgExtNewIndexes = []; // table, index, dir
+		$wgExtModifiedFields = []; // table, index, dir
 	}
 
 	/**
@@ -143,15 +146,26 @@ abstract class DatabaseUpdater {
 			return; // already loaded
 		}
 		$vars = Installer::getExistingLocalSettings();
-		if ( !$vars ) {
-			return; // no LocalSettings found
+
+		$registry = ExtensionRegistry::getInstance();
+		$queue = $registry->getQueue();
+		// Don't accidentally load extensions in the future
+		$registry->clearQueue();
+
+		// This will automatically add "AutoloadClasses" to $wgAutoloadClasses
+		$data = $registry->readFromQueue( $queue );
+		$hooks = [ 'wgHooks' => [ 'LoadExtensionSchemaUpdates' => [] ] ];
+		if ( isset( $data['globals']['wgHooks']['LoadExtensionSchemaUpdates'] ) ) {
+			$hooks = $data['globals']['wgHooks']['LoadExtensionSchemaUpdates'];
 		}
-		if ( !isset( $vars['wgHooks'] ) || !isset( $vars['wgHooks']['LoadExtensionSchemaUpdates'] ) ) {
-			return;
+		if ( $vars && isset( $vars['wgHooks']['LoadExtensionSchemaUpdates'] ) ) {
+			$hooks = array_merge_recursive( $hooks, $vars['wgHooks']['LoadExtensionSchemaUpdates'] );
 		}
 		global $wgHooks, $wgAutoloadClasses;
-		$wgHooks['LoadExtensionSchemaUpdates'] = $vars['wgHooks']['LoadExtensionSchemaUpdates'];
-		$wgAutoloadClasses = $wgAutoloadClasses + $vars['wgAutoloadClasses'];
+		$wgHooks['LoadExtensionSchemaUpdates'] = $hooks;
+		if ( $vars && isset( $vars['wgAutoloadClasses'] ) ) {
+			$wgAutoloadClasses += $vars['wgAutoloadClasses'];
+		}
 	}
 
 	/**
@@ -226,42 +240,42 @@ abstract class DatabaseUpdater {
 	 * @param string $sqlPath Full path to the schema file
 	 */
 	public function addExtensionTable( $tableName, $sqlPath ) {
-		$this->extensionUpdates[] = array( 'addTable', $tableName, $sqlPath, true );
+		$this->extensionUpdates[] = [ 'addTable', $tableName, $sqlPath, true ];
 	}
 
 	/**
 	 * @since 1.19
 	 *
-	 * @param $tableName string
-	 * @param $indexName string
-	 * @param $sqlPath string
+	 * @param string $tableName
+	 * @param string $indexName
+	 * @param string $sqlPath
 	 */
 	public function addExtensionIndex( $tableName, $indexName, $sqlPath ) {
-		$this->extensionUpdates[] = array( 'addIndex', $tableName, $indexName, $sqlPath, true );
+		$this->extensionUpdates[] = [ 'addIndex', $tableName, $indexName, $sqlPath, true ];
 	}
 
 	/**
 	 *
 	 * @since 1.19
 	 *
-	 * @param $tableName string
-	 * @param $columnName string
-	 * @param $sqlPath string
+	 * @param string $tableName
+	 * @param string $columnName
+	 * @param string $sqlPath
 	 */
 	public function addExtensionField( $tableName, $columnName, $sqlPath ) {
-		$this->extensionUpdates[] = array( 'addField', $tableName, $columnName, $sqlPath, true );
+		$this->extensionUpdates[] = [ 'addField', $tableName, $columnName, $sqlPath, true ];
 	}
 
 	/**
 	 *
 	 * @since 1.20
 	 *
-	 * @param $tableName string
-	 * @param $columnName string
-	 * @param $sqlPath string
+	 * @param string $tableName
+	 * @param string $columnName
+	 * @param string $sqlPath
 	 */
 	public function dropExtensionField( $tableName, $columnName, $sqlPath ) {
-		$this->extensionUpdates[] = array( 'dropField', $tableName, $columnName, $sqlPath, true );
+		$this->extensionUpdates[] = [ 'dropField', $tableName, $columnName, $sqlPath, true ];
 	}
 
 	/**
@@ -274,18 +288,18 @@ abstract class DatabaseUpdater {
 	 * @param string $sqlPath The path to the SQL change path
 	 */
 	public function dropExtensionIndex( $tableName, $indexName, $sqlPath ) {
-		$this->extensionUpdates[] = array( 'dropIndex', $tableName, $indexName, $sqlPath, true );
+		$this->extensionUpdates[] = [ 'dropIndex', $tableName, $indexName, $sqlPath, true ];
 	}
 
 	/**
 	 *
 	 * @since 1.20
 	 *
-	 * @param $tableName string
-	 * @param $sqlPath string
+	 * @param string $tableName
+	 * @param string $sqlPath
 	 */
 	public function dropExtensionTable( $tableName, $sqlPath ) {
-		$this->extensionUpdates[] = array( 'dropTable', $tableName, $sqlPath, true );
+		$this->extensionUpdates[] = [ 'dropTable', $tableName, $sqlPath, true ];
 	}
 
 	/**
@@ -296,14 +310,14 @@ abstract class DatabaseUpdater {
 	 * @param string $tableName The table name
 	 * @param string $oldIndexName The old index name
 	 * @param string $newIndexName The new index name
-	 * @param $skipBothIndexExistWarning Boolean: Whether to warn if both the old
-	 * and the new indexes exist. [facultative; by default, false]
 	 * @param string $sqlPath The path to the SQL change path
+	 * @param bool $skipBothIndexExistWarning Whether to warn if both the old
+	 * and the new indexes exist. [facultative; by default, false]
 	 */
 	public function renameExtensionIndex( $tableName, $oldIndexName, $newIndexName,
 		$sqlPath, $skipBothIndexExistWarning = false
 	) {
-		$this->extensionUpdates[] = array(
+		$this->extensionUpdates[] = [
 			'renameIndex',
 			$tableName,
 			$oldIndexName,
@@ -311,7 +325,7 @@ abstract class DatabaseUpdater {
 			$skipBothIndexExistWarning,
 			$sqlPath,
 			true
-		);
+		];
 	}
 
 	/**
@@ -322,14 +336,14 @@ abstract class DatabaseUpdater {
 	 * @param string $sqlPath The path to the SQL change path
 	 */
 	public function modifyExtensionField( $tableName, $fieldName, $sqlPath ) {
-		$this->extensionUpdates[] = array( 'modifyField', $tableName, $fieldName, $sqlPath, true );
+		$this->extensionUpdates[] = [ 'modifyField', $tableName, $fieldName, $sqlPath, true ];
 	}
 
 	/**
 	 *
 	 * @since 1.20
 	 *
-	 * @param $tableName string
+	 * @param string $tableName
 	 * @return bool
 	 */
 	public function tableExists( $tableName ) {
@@ -352,7 +366,7 @@ abstract class DatabaseUpdater {
 	/**
 	 * Get the list of extension-defined updates
 	 *
-	 * @return Array
+	 * @return array
 	 */
 	protected function getExtensionUpdates() {
 		return $this->extensionUpdates;
@@ -361,7 +375,7 @@ abstract class DatabaseUpdater {
 	/**
 	 * @since 1.17
 	 *
-	 * @return array
+	 * @return string[]
 	 */
 	public function getPostDatabaseUpdateMaintenance() {
 		return $this->postDatabaseUpdateMaintenance;
@@ -371,10 +385,11 @@ abstract class DatabaseUpdater {
 	 * @since 1.21
 	 *
 	 * Writes the schema updates desired to a file for the DB Admin to run.
+	 * @param array $schemaUpdate
 	 */
-	private function writeSchemaUpdateFile( $schemaUpdate = array() ) {
+	private function writeSchemaUpdateFile( $schemaUpdate = [] ) {
 		$updates = $this->updatesSkipped;
-		$this->updatesSkipped = array();
+		$this->updatesSkipped = [];
 
 		foreach ( $updates as $funcList ) {
 			$func = $funcList[0];
@@ -391,7 +406,7 @@ abstract class DatabaseUpdater {
 	 *
 	 * @param array $what What updates to perform
 	 */
-	public function doUpdates( $what = array( 'core', 'extensions', 'stats' ) ) {
+	public function doUpdates( $what = [ 'core', 'extensions', 'stats' ] ) {
 		global $wgVersion;
 
 		$this->db->begin( __METHOD__ );
@@ -423,18 +438,17 @@ abstract class DatabaseUpdater {
 	/**
 	 * Helper function for doUpdates()
 	 *
-	 * @param array $updates of updates to run
-	 * @param bool $passSelf Whether to pass this object we calling external
-	 *                  functions
+	 * @param array $updates Array of updates to run
+	 * @param bool $passSelf Whether to pass this object we calling external functions
 	 */
 	private function runUpdates( array $updates, $passSelf ) {
-		$updatesDone = array();
-		$updatesSkipped = array();
+		$updatesDone = [];
+		$updatesSkipped = [];
 		foreach ( $updates as $params ) {
 			$origParams = $params;
 			$func = array_shift( $params );
 			if ( !is_array( $func ) && method_exists( $this, $func ) ) {
-				$func = array( $this, $func );
+				$func = [ $this, $func ];
 			} elseif ( $passSelf ) {
 				array_unshift( $params, $this );
 			}
@@ -442,8 +456,9 @@ abstract class DatabaseUpdater {
 			flush();
 			if ( $ret !== false ) {
 				$updatesDone[] = $origParams;
+				wfGetLBFactory()->waitForReplication();
 			} else {
-				$updatesSkipped[] = array( $func, $params, $origParams );
+				$updatesSkipped[] = [ $func, $params, $origParams ];
 			}
 		}
 		$this->updatesSkipped = array_merge( $this->updatesSkipped, $updatesSkipped );
@@ -454,14 +469,15 @@ abstract class DatabaseUpdater {
 	 * @param string $version
 	 * @param array $updates
 	 */
-	protected function setAppliedUpdates( $version, $updates = array() ) {
+	protected function setAppliedUpdates( $version, $updates = [] ) {
 		$this->db->clearFlag( DBO_DDLMODE );
 		if ( !$this->canUseNewUpdatelog() ) {
 			return;
 		}
-		$key = "updatelist-$version-" . time();
+		$key = "updatelist-$version-" . microtime( true ) . self::$updateCounter;
+		self::$updateCounter++;
 		$this->db->insert( 'updatelog',
-			array( 'ul_key' => $key, 'ul_value' => serialize( $updates ) ),
+			[ 'ul_key' => $key, 'ul_value' => serialize( $updates ) ],
 			__METHOD__ );
 		$this->db->setFlag( DBO_DDLMODE );
 	}
@@ -476,8 +492,9 @@ abstract class DatabaseUpdater {
 	public function updateRowExists( $key ) {
 		$row = $this->db->selectRow(
 			'updatelog',
-			'1',
-			array( 'ul_key' => $key ),
+			# Bug 65813
+			'1 AS X',
+			[ 'ul_key' => $key ],
 			__METHOD__
 		);
 
@@ -493,7 +510,7 @@ abstract class DatabaseUpdater {
 	 */
 	public function insertUpdateRow( $key, $val = null ) {
 		$this->db->clearFlag( DBO_DDLMODE );
-		$values = array( 'ul_key' => $key );
+		$values = [ 'ul_key' => $key ];
 		if ( $val && $this->canUseNewUpdatelog() ) {
 			$values['ul_value'] = $val;
 		}
@@ -507,7 +524,7 @@ abstract class DatabaseUpdater {
 	 * class does). Pre-1.17 wikis won't have this column, and really old wikis
 	 * might not even have updatelog at all
 	 *
-	 * @return boolean
+	 * @return bool
 	 */
 	protected function canUseNewUpdatelog() {
 		return $this->db->tableExists( 'updatelog', __METHOD__ ) &&
@@ -531,7 +548,12 @@ abstract class DatabaseUpdater {
 			return true;
 		}
 
-		return !in_array( $name, $wgSharedTables );
+		if ( in_array( $name, $wgSharedTables ) ) {
+			$this->output( "...skipping update to shared table $name.\n" );
+			return false;
+		} else {
+			return true;
+		}
 	}
 
 	/**
@@ -546,33 +568,33 @@ abstract class DatabaseUpdater {
 		global $wgExtNewFields, $wgExtNewTables, $wgExtModifiedFields,
 			$wgExtNewIndexes;
 
-		$updates = array();
+		$updates = [];
 
 		foreach ( $wgExtNewTables as $tableRecord ) {
-			$updates[] = array(
+			$updates[] = [
 				'addTable', $tableRecord[0], $tableRecord[1], true
-			);
+			];
 		}
 
 		foreach ( $wgExtNewFields as $fieldRecord ) {
-			$updates[] = array(
+			$updates[] = [
 				'addField', $fieldRecord[0], $fieldRecord[1],
 				$fieldRecord[2], true
-			);
+			];
 		}
 
 		foreach ( $wgExtNewIndexes as $fieldRecord ) {
-			$updates[] = array(
+			$updates[] = [
 				'addIndex', $fieldRecord[0], $fieldRecord[1],
 				$fieldRecord[2], true
-			);
+			];
 		}
 
 		foreach ( $wgExtModifiedFields as $fieldRecord ) {
-			$updates[] = array(
+			$updates[] = [
 				'modifyField', $fieldRecord[0], $fieldRecord[1],
 				$fieldRecord[2], true
-			);
+			];
 		}
 
 		return $updates;
@@ -595,7 +617,7 @@ abstract class DatabaseUpdater {
 	 */
 	public function copyFile( $filename ) {
 		$this->db->sourceFile( $filename, false, false, false,
-			array( $this, 'appendLine' )
+			[ $this, 'appendLine' ]
 		);
 	}
 
@@ -603,7 +625,7 @@ abstract class DatabaseUpdater {
 	 * Append a line to the open filehandle.  The line is assumed to
 	 * be a complete SQL statement.
 	 *
-	 * This is used as a callback for for sourceLine().
+	 * This is used as a callback for sourceLine().
 	 *
 	 * @param string $line Text to append to the file
 	 * @return bool False to skip actually executing the file
@@ -622,7 +644,7 @@ abstract class DatabaseUpdater {
 	 * Applies a SQL patch
 	 *
 	 * @param string $path Path to the patch file
-	 * @param $isFullPath Boolean Whether to treat $path as a relative or not
+	 * @param bool $isFullPath Whether to treat $path as a relative or not
 	 * @param string $msg Description of the patch
 	 * @return bool False if patch is skipped.
 	 */
@@ -775,7 +797,7 @@ abstract class DatabaseUpdater {
 	 * @param string $table Name of the table to modify
 	 * @param string $oldIndex Old name of the index
 	 * @param string $newIndex New name of the index
-	 * @param $skipBothIndexExistWarning Boolean: Whether to warn if both the
+	 * @param bool $skipBothIndexExistWarning Whether to warn if both the
 	 * old and the new indexes exist.
 	 * @param string $patch Path to the patch file
 	 * @param bool $fullpath Whether to treat $patch path as a relative or not
@@ -888,6 +910,29 @@ abstract class DatabaseUpdater {
 		return true;
 	}
 
+	/**
+	 * Set any .htaccess files or equivilent for storage repos
+	 *
+	 * Some zones (e.g. "temp") used to be public and may have been initialized as such
+	 */
+	public function setFileAccess() {
+		$repo = RepoGroup::singleton()->getLocalRepo();
+		$zonePath = $repo->getZonePath( 'temp' );
+		if ( $repo->getBackend()->directoryExists( [ 'dir' => $zonePath ] ) ) {
+			// If the directory was never made, then it will have the right ACLs when it is made
+			$status = $repo->getBackend()->secure( [
+				'dir' => $zonePath,
+				'noAccess' => true,
+				'noListing' => true
+			] );
+			if ( $status->isOK() ) {
+				$this->output( "Set the local repo temp zone container to be private.\n" );
+			} else {
+				$this->output( "Failed to set the local repo temp zone container to be private.\n" );
+			}
+		}
+	}
+
 	/**
 	 * Purge the objectcache table
 	 */
@@ -900,7 +945,9 @@ abstract class DatabaseUpdater {
 		if ( $wgLocalisationCacheConf['manualRecache'] ) {
 			$this->rebuildLocalisationCache();
 		}
-		MessageBlobStore::clear();
+		$blobStore = new MessageBlobStore();
+		$blobStore->clear();
+		$this->db->delete( 'module_deps', '*', __METHOD__ );
 		$this->output( "done.\n" );
 	}
 
@@ -909,7 +956,7 @@ abstract class DatabaseUpdater {
 	 */
 	protected function checkStats() {
 		$this->output( "...site_stats is populated..." );
-		$row = $this->db->selectRow( 'site_stats', '*', array( 'ss_row_id' => 1 ), __METHOD__ );
+		$row = $this->db->selectRow( 'site_stats', '*', [ 'ss_row_id' => 1 ], __METHOD__ );
 		if ( $row === false ) {
 			$this->output( "data is missing! rebuilding...\n" );
 		} elseif ( isset( $row->site_stats ) && $row->ss_total_pages == -1 ) {
@@ -932,11 +979,11 @@ abstract class DatabaseUpdater {
 		if ( $activeUsers == -1 ) {
 			$activeUsers = $this->db->selectField( 'recentchanges',
 				'COUNT( DISTINCT rc_user_text )',
-				array( 'rc_user != 0', 'rc_bot' => 0, "rc_log_type != 'newusers'" ), __METHOD__
+				[ 'rc_user != 0', 'rc_bot' => 0, "rc_log_type != 'newusers'" ], __METHOD__
 			);
 			$this->db->update( 'site_stats',
-				array( 'ss_active_users' => intval( $activeUsers ) ),
-				array( 'ss_row_id' => 1 ), __METHOD__, array( 'LIMIT' => 1 )
+				[ 'ss_active_users' => intval( $activeUsers ) ],
+				[ 'ss_row_id' => 1 ], __METHOD__, [ 'LIMIT' => 1 ]
 			);
 		}
 		$this->output( "...ss_active_users user count set...\n" );
@@ -977,6 +1024,7 @@ abstract class DatabaseUpdater {
 
 	/**
 	 * Updates the timestamps in the transcache table
+	 * @return bool
 	 */
 	protected function doUpdateTranscacheField() {
 		if ( $this->updateRowExists( 'convert transcache field' ) ) {
@@ -1025,6 +1073,31 @@ abstract class DatabaseUpdater {
 		}
 	}
 
+	/**
+	 * Enable profiling table when it's turned on
+	 */
+	protected function doEnableProfiling() {
+		global $wgProfiler;
+
+		if ( !$this->doTable( 'profiling' ) ) {
+			return true;
+		}
+
+		$profileToDb = false;
+		if ( isset( $wgProfiler['output'] ) ) {
+			$out = $wgProfiler['output'];
+			if ( $out === 'db' ) {
+				$profileToDb = true;
+			} elseif ( is_array( $out ) && in_array( 'db', $out ) ) {
+				$profileToDb = true;
+			}
+		}
+
+		if ( $profileToDb && !$this->db->tableExists( 'profiling', __METHOD__ ) ) {
+			$this->applyPatch( 'patch-profiling.sql', false, 'Add profiling table' );
+		}
+	}
+
 	/**
 	 * Rebuilds the localisation cache
 	 */
