@@ -106,7 +106,7 @@ class FormatMetadata extends ContextSource {
 			// This seems ugly to wrap non-array's in an array just to unwrap again,
 			// especially when most of the time it is not an array
 			if ( !is_array( $tags[$tag] ) ) {
-				$vals = array( $vals );
+				$vals = [ $vals ];
 			}
 
 			// _type is a special value to say what array type
@@ -117,9 +117,9 @@ class FormatMetadata extends ContextSource {
 				$type = 'ul'; // default unordered list.
 			}
 
-			//This is done differently as the tag is an array.
+			// This is done differently as the tag is an array.
 			if ( $tag == 'GPSTimeStamp' && count( $vals ) === 3 ) {
-				//hour min sec array
+				// hour min sec array
 
 				$h = explode( '/', $vals[0] );
 				$m = explode( '/', $vals[1] );
@@ -190,8 +190,18 @@ class FormatMetadata extends ContextSource {
 
 					case 'PhotometricInterpretation':
 						switch ( $val ) {
+							case 0:
+							case 1:
 							case 2:
+							case 3:
+							case 4:
+							case 5:
 							case 6:
+							case 8:
+							case 9:
+							case 10:
+							case 32803:
+							case 34892:
 								$val = $this->exifMsg( $tag, $val );
 								break;
 							default:
@@ -407,15 +417,15 @@ class FormatMetadata extends ContextSource {
 						break;
 
 					case 'Flash':
-						$flashDecode = array(
+						$flashDecode = [
 							'fired' => $val & bindec( '00000001' ),
 							'return' => ( $val & bindec( '00000110' ) ) >> 1,
 							'mode' => ( $val & bindec( '00011000' ) ) >> 3,
 							'function' => ( $val & bindec( '00100000' ) ) >> 5,
 							'redeye' => ( $val & bindec( '01000000' ) ) >> 6,
-//						'reserved' => ($val & bindec( '10000000' )) >> 7,
-						);
-						$flashMsgs = array();
+// 						'reserved' => ($val & bindec( '10000000' )) >> 7,
+						];
+						$flashMsgs = [];
 						# We do not need to handle unknown values since all are used.
 						foreach ( $flashDecode as $subTag => $subValue ) {
 							# We do not need any message for zeroed values.
@@ -597,7 +607,7 @@ class FormatMetadata extends ContextSource {
 						}
 						break;
 
-					//The GPS...Ref values are kept for compatibility, probably won't be reached.
+					// The GPS...Ref values are kept for compatibility, probably won't be reached.
 					case 'GPSLatitudeRef':
 					case 'GPSDestLatitudeRef':
 						switch ( $val ) {
@@ -706,7 +716,7 @@ class FormatMetadata extends ContextSource {
 						break;
 
 					case 'GPSDOP':
-						// See http://en.wikipedia.org/wiki/Dilution_of_precision_(GPS)
+						// See https://en.wikipedia.org/wiki/Dilution_of_precision_(GPS)
 						if ( $val <= 2 ) {
 							$val = $this->exifMsg( $tag, 'excellent', $this->formatNum( $val ) );
 						} elseif ( $val <= 5 ) {
@@ -730,7 +740,7 @@ class FormatMetadata extends ContextSource {
 
 					case 'Software':
 						if ( is_array( $val ) ) {
-							//if its a software, version array.
+							// if its a software, version array.
 							$val = $this->msg( 'exif-software-version-value', $val[0], $val[1] )->text();
 						} else {
 							$val = $this->exifMsg( $tag, '', $val );
@@ -1003,28 +1013,6 @@ class FormatMetadata extends ContextSource {
 		return $obj->flattenArrayReal( $vals, $type, $noHtml );
 	}
 
-	/**
-	 * Flatten an array, using the user language for any messages.
-	 *
-	 * @param array $vals Array of values
-	 * @param string $type Type of array (either lang, ul, ol).
-	 *   lang = language assoc array with keys being the lang code
-	 *   ul = unordered list, ol = ordered list
-	 *   type can also come from the '_type' member of $vals.
-	 * @param bool $noHtml If to avoid returning anything resembling HTML.
-	 *   (Ugly hack for backwards compatibility with old MediaWiki).
-	 * @param bool|IContextSource $context
-	 * @return string Single value (in wiki-syntax).
-	 */
-	public static function flattenArray( $vals, $type = 'ul', $noHtml = false, $context = false ) {
-		$obj = new FormatMetadata;
-		if ( $context ) {
-			$obj->setContext( $context );
-		}
-
-		return $obj->flattenArrayReal( $vals, $type, $noHtml );
-	}
-
 	/**
 	 * A function to collapse multivalued tags into a single value.
 	 * This turns an array of (for example) authors into a bulleted list.
@@ -1053,7 +1041,7 @@ class FormatMetadata extends ContextSource {
 
 		if ( !is_array( $vals ) ) {
 			return $vals; // do nothing if not an array;
-		} elseif ( count( $vals ) === 1 && $type !== 'lang' ) {
+		} elseif ( count( $vals ) === 1 && $type !== 'lang' && isset( $vals[0] ) ) {
 			return $vals[0];
 		} elseif ( count( $vals ) === 0 ) {
 			wfDebug( __METHOD__ . " metadata array with 0 elements!\n" );
@@ -1104,7 +1092,7 @@ class FormatMetadata extends ContextSource {
 
 							if ( $this->singleLang ) {
 								return Html::rawElement( 'span',
-									array( 'lang' => $pLang ), $vals[$pLang] );
+									[ 'lang' => $pLang ], $vals[$pLang] );
 							}
 						}
 					}
@@ -1119,7 +1107,7 @@ class FormatMetadata extends ContextSource {
 							$lang, false, $noHtml );
 						if ( $this->singleLang ) {
 							return Html::rawElement( 'span',
-								array( 'lang' => $lang ), $item );
+								[ 'lang' => $lang ], $item );
 						}
 					}
 					if ( $defaultItem !== false ) {
@@ -1191,7 +1179,7 @@ class FormatMetadata extends ContextSource {
 		$lowLang = strtolower( $lang );
 		$langName = Language::fetchLanguageName( $lowLang );
 		if ( $langName === '' ) {
-			//try just the base language name. (aka en-US -> en ).
+			// try just the base language name. (aka en-US -> en ).
 			list( $langPrefix ) = explode( '-', $lowLang, 2 );
 			$langName = Language::fetchLanguageName( $langPrefix );
 			if ( $langName === '' ) {
@@ -1246,9 +1234,9 @@ class FormatMetadata extends ContextSource {
 	 * @return mixed A floating point number or whatever we were fed
 	 */
 	private function formatNum( $num, $round = false ) {
-		$m = array();
+		$m = [];
 		if ( is_array( $num ) ) {
-			$out = array();
+			$out = [];
 			foreach ( $num as $number ) {
 				$out[] = $this->formatNum( $number );
 			}
@@ -1282,7 +1270,7 @@ class FormatMetadata extends ContextSource {
 	 * @return mixed A floating point number or whatever we were fed
 	 */
 	private function formatFraction( $num ) {
-		$m = array();
+		$m = [];
 		if ( preg_match( '/^(-?\d+)\/(\d+)$/', $num, $m ) ) {
 			$numerator = intval( $m[1] );
 			$denominator = intval( $m[2] );
@@ -1305,7 +1293,7 @@ class FormatMetadata extends ContextSource {
 	 */
 	private function gcd( $a, $b ) {
 		/*
-			// http://en.wikipedia.org/wiki/Euclidean_algorithm
+			// https://en.wikipedia.org/wiki/Euclidean_algorithm
 			// Recursive form would be:
 			if( $b == 0 )
 				return $a;
@@ -1510,7 +1498,7 @@ class FormatMetadata extends ContextSource {
 					. '</span>';
 			}
 			if ( isset( $vals['CiEmailWork'] ) ) {
-				$emails = array();
+				$emails = [];
 				// Have to split multiple emails at commas/new lines.
 				$splitEmails = explode( "\n", $vals['CiEmailWork'] );
 				foreach ( $splitEmails as $e1 ) {
@@ -1572,10 +1560,10 @@ class FormatMetadata extends ContextSource {
 	 * @since 1.23
 	 */
 	public static function getVisibleFields() {
-		$fields = array();
+		$fields = [];
 		$lines = explode( "\n", wfMessage( 'metadata-fields' )->inContentLanguage()->text() );
 		foreach ( $lines as $line ) {
-			$matches = array();
+			$matches = [];
 			if ( preg_match( '/^\\*\s*(.*?)\s*$/', $line, $matches ) ) {
 				$fields[] = $matches[1];
 			}
@@ -1593,12 +1581,11 @@ class FormatMetadata extends ContextSource {
 	 * @since 1.23
 	 */
 	public function fetchExtendedMetadata( File $file ) {
-		global $wgMemc;
+		$cache = ObjectCache::getMainWANInstance();
 
 		// If revision deleted, exit immediately
 		if ( $file->isDeleted( File::DELETED_FILE ) ) {
-
-			return array();
+			return [];
 		}
 
 		$cacheKey = wfMemcKey(
@@ -1608,10 +1595,10 @@ class FormatMetadata extends ContextSource {
 			$file->getSha1()
 		);
 
-		$cachedValue = $wgMemc->get( $cacheKey );
+		$cachedValue = $cache->get( $cacheKey );
 		if (
 			$cachedValue
-			&& Hooks::run( 'ValidateExtendedMetadataCache', array( $cachedValue['timestamp'], $file ) )
+			&& Hooks::run( 'ValidateExtendedMetadataCache', [ $cachedValue['timestamp'], $file ] )
 		) {
 			$extendedMetadata = $cachedValue['data'];
 		} else {
@@ -1627,8 +1614,8 @@ class FormatMetadata extends ContextSource {
 			// outside fetchExtendedMetadata, but this way we don't need to redo the
 			// computation on a cache hit.
 			$this->sanitizeArrayForAPI( $extendedMetadata );
-			$valueToCache = array( 'data' => $extendedMetadata, 'timestamp' => wfTimestampNow() );
-			$wgMemc->set( $cacheKey, $valueToCache, $maxCacheTime );
+			$valueToCache = [ 'data' => $extendedMetadata, 'timestamp' => wfTimestampNow() ];
+			$cache->set( $cacheKey, $valueToCache, $maxCacheTime );
 		}
 
 		return $extendedMetadata;
@@ -1649,18 +1636,18 @@ class FormatMetadata extends ContextSource {
 		if ( $file instanceof ForeignAPIFile ) {
 			// In case of error we pretend no metadata - this will get cached.
 			// Might or might not be a good idea.
-			return $file->getExtendedMetadata() ?: array();
+			return $file->getExtendedMetadata() ?: [];
 		}
 
 		$uploadDate = wfTimestamp( TS_ISO_8601, $file->getTimestamp() );
 
-		$fileMetadata = array(
+		$fileMetadata = [
 			// This is modification time, which is close to "upload" time.
-			'DateTime' => array(
+			'DateTime' => [
 				'value' => $uploadDate,
 				'source' => 'mediawiki-metadata',
-			),
-		);
+			],
+		];
 
 		$title = $file->getTitle();
 		if ( $title ) {
@@ -1673,10 +1660,10 @@ class FormatMetadata extends ContextSource {
 				$name = $text;
 			}
 
-			$fileMetadata['ObjectName'] = array(
+			$fileMetadata['ObjectName'] = [
 				'value' => $name,
 				'source' => 'mediawiki-metadata',
-			);
+			];
 		}
 
 		return $fileMetadata;
@@ -1696,13 +1683,13 @@ class FormatMetadata extends ContextSource {
 		&$maxCacheTime
 	) {
 
-		Hooks::run( 'GetExtendedMetadata', array(
+		Hooks::run( 'GetExtendedMetadata', [
 			&$extendedMetadata,
 			$file,
 			$this->getContext(),
 			$this->singleLang,
 			&$maxCacheTime
-		) );
+		] );
 
 		$visible = array_flip( self::getVisibleFields() );
 		foreach ( $extendedMetadata as $key => $value ) {
@@ -1755,8 +1742,9 @@ class FormatMetadata extends ContextSource {
 	}
 
 	/**
-	 * Turns an XMP-style multivalue array into a single value by dropping all but the first value.
-	 * If the value is not a multivalue array (or a multivalue array inside a multilang array), it is returned unchanged.
+	 * Turns an XMP-style multivalue array into a single value by dropping all but the first
+	 * value. If the value is not a multivalue array (or a multivalue array inside a multilang
+	 * array), it is returned unchanged.
 	 * See mediawiki.org/wiki/Manual:File_metadata_handling#Multi-language_array_format
 	 * @param mixed $value
 	 * @return mixed The value, or the first value if there were multiple ones
@@ -1765,8 +1753,9 @@ class FormatMetadata extends ContextSource {
 	protected function resolveMultivalueValue( $value ) {
 		if ( !is_array( $value ) ) {
 			return $value;
-		} elseif ( isset( $value['_type'] ) && $value['_type'] === 'lang' ) { // if this is a multilang array, process fields separately
-			$newValue = array();
+		} elseif ( isset( $value['_type'] ) && $value['_type'] === 'lang' ) {
+			// if this is a multilang array, process fields separately
+			$newValue = [];
 			foreach ( $value as $k => $v ) {
 				$newValue[$k] = $this->resolveMultivalueValue( $v );
 			}
