@@ -28,19 +28,33 @@
  */
 class SearchDatabase extends SearchEngine {
 	/**
-	 * @var DatabaseBase Slave database for reading from for results
+	 * @var IDatabase Slave database for reading from for results
 	 */
 	protected $db;
 
 	/**
 	 * Constructor
-	 * @param DatabaseBase $db The database to search from
+	 * @param IDatabase $db The database to search from
 	 */
-	public function __construct( DatabaseBase $db = null ) {
+	public function __construct( IDatabase $db = null ) {
 		if ( $db ) {
 			$this->db = $db;
 		} else {
 			$this->db = wfGetDB( DB_SLAVE );
 		}
 	}
+
+	/**
+	 * Return a 'cleaned up' search string
+	 *
+	 * @param string $text
+	 * @return string
+	 */
+	protected function filter( $text ) {
+		// List of chars allowed in the search query.
+		// This must include chars used in the search syntax.
+		// Usually " (phrase) or * (wildcards) if supported by the engine
+		$lc = $this->legalSearchChars( self::CHARS_ALL );
+		return trim( preg_replace( "/[^{$lc}]/", " ", $text ) );
+	}
 }
