@@ -33,13 +33,13 @@ class DeadendPagesPage extends PageQueryPage {
 	}
 
 	function getPageHeader() {
-		return wfMsgExt( 'deadendpagestext', array( 'parse' ) );
+		return $this->msg( 'deadendpagestext' )->parseAsBlock();
 	}
 
 	/**
 	 * LEFT JOIN is expensive
 	 *
-	 * @return true
+	 * @return bool
 	 */
 	function isExpensive() {
 		return true;
@@ -50,36 +50,45 @@ class DeadendPagesPage extends PageQueryPage {
 	}
 
 	/**
-	 * @return false
+	 * @return bool
 	 */
 	function sortDescending() {
 		return false;
 	}
 
 	function getQueryInfo() {
-		return array(
-			'tables' => array( 'page', 'pagelinks' ),
-			'fields' => array( 'page_namespace AS namespace',
-					'page_title AS title',
-					'page_title AS value'
-			),
-			'conds' => array( 'pl_from IS NULL',
-					'page_namespace' => MWNamespace::getContentNamespaces(),
-					'page_is_redirect' => 0
-			),
-			'join_conds' => array( 'pagelinks' => array( 'LEFT JOIN', array(
-					'page_id=pl_from'
-			) ) )
-		);
+		return [
+			'tables' => [ 'page', 'pagelinks' ],
+			'fields' => [
+				'namespace' => 'page_namespace',
+				'title' => 'page_title',
+				'value' => 'page_title'
+			],
+			'conds' => [
+				'pl_from IS NULL',
+				'page_namespace' => MWNamespace::getContentNamespaces(),
+				'page_is_redirect' => 0
+			],
+			'join_conds' => [
+				'pagelinks' => [
+					'LEFT JOIN',
+					[ 'page_id=pl_from' ]
+				]
+			]
+		];
 	}
 
 	function getOrderFields() {
 		// For some crazy reason ordering by a constant
 		// causes a filesort
-		if( count( MWNamespace::getContentNamespaces() ) > 1 ) {
-			return array( 'page_namespace', 'page_title' );
+		if ( count( MWNamespace::getContentNamespaces() ) > 1 ) {
+			return [ 'page_namespace', 'page_title' ];
 		} else {
-			return array( 'page_title' );
+			return [ 'page_title' ];
 		}
 	}
+
+	protected function getGroupName() {
+		return 'maintenance';
+	}
 }
