@@ -26,7 +26,7 @@
  *
  * @ingroup SpecialPage
  */
-class SpecialPrefixindex extends SpecialAllpages {
+class SpecialPrefixindex extends SpecialAllPages {
 
 	/**
 	 * Whether to remove the searched prefix from the displayed link. Useful
@@ -36,9 +36,6 @@ class SpecialPrefixindex extends SpecialAllpages {
 
 	protected $hideRedirects = false;
 
-	// number of columns in output table
-	protected $columns = 3;
-
 	// Inherit $maxPerPage
 
 	function __construct() {
@@ -47,7 +44,7 @@ class SpecialPrefixindex extends SpecialAllpages {
 
 	/**
 	 * Entry point : initialise variables and call subfunctions.
-	 * @param string $par becomes "FOO" when called like Special:Prefixindex/FOO (default null)
+	 * @param string $par Becomes "FOO" when called like Special:Prefixindex/FOO (default null)
 	 */
 	function execute( $par ) {
 		global $wgContLang;
@@ -66,7 +63,6 @@ class SpecialPrefixindex extends SpecialAllpages {
 		$namespace = (int)$ns; // if no namespace given, use 0 (NS_MAIN).
 		$this->hideRedirects = $request->getBool( 'hideredirects', $this->hideRedirects );
 		$this->stripPrefix = $request->getBool( 'stripprefix', $this->stripPrefix );
-		$this->columns = $request->getInt( 'columns', $this->columns );
 
 		$namespaces = $wgContLang->getNamespaces();
 		$out->setPageTitle(
@@ -76,7 +72,7 @@ class SpecialPrefixindex extends SpecialAllpages {
 		);
 
 		$showme = '';
-		if ( isset( $par ) ) {
+		if ( $par !== null ) {
 			$showme = $par;
 		} elseif ( $prefix != '' ) {
 			$showme = $prefix;
@@ -96,25 +92,26 @@ class SpecialPrefixindex extends SpecialAllpages {
 
 	/**
 	 * HTML for the top form
-	 * @param $namespace Integer: a namespace constant (default NS_MAIN).
-	 * @param string $from dbKey we are starting listing at.
+	 * @param int $namespace A namespace constant (default NS_MAIN).
+	 * @param string $from DbKey we are starting listing at.
 	 * @return string
 	 */
 	protected function namespacePrefixForm( $namespace = NS_MAIN, $from = '' ) {
-		global $wgScript;
-
-		$out = Xml::openElement( 'div', array( 'class' => 'namespaceoptions' ) );
-		$out .= Xml::openElement( 'form', array( 'method' => 'get', 'action' => $wgScript ) );
+		$out = Xml::openElement( 'div', [ 'class' => 'namespaceoptions' ] );
+		$out .= Xml::openElement(
+			'form',
+			[ 'method' => 'get', 'action' => $this->getConfig()->get( 'Script' ) ]
+		);
 		$out .= Html::hidden( 'title', $this->getPageTitle()->getPrefixedText() );
 		$out .= Xml::openElement( 'fieldset' );
 		$out .= Xml::element( 'legend', null, $this->msg( 'allpages' )->text() );
-		$out .= Xml::openElement( 'table', array( 'id' => 'nsselect', 'class' => 'allpages' ) );
+		$out .= Xml::openElement( 'table', [ 'id' => 'nsselect', 'class' => 'allpages' ] );
 		$out .= "<tr>
 				<td class='mw-label'>" .
 			Xml::label( $this->msg( 'allpagesprefix' )->text(), 'nsfrom' ) .
 			"</td>
 				<td class='mw-input'>" .
-			Xml::input( 'prefix', 30, str_replace( '_', ' ', $from ), array( 'id' => 'nsfrom' ) ) .
+			Xml::input( 'prefix', 30, str_replace( '_', ' ', $from ), [ 'id' => 'nsfrom' ] ) .
 			"</td>
 			</tr>
 			<tr>
@@ -122,13 +119,13 @@ class SpecialPrefixindex extends SpecialAllpages {
 			Xml::label( $this->msg( 'namespace' )->text(), 'namespace' ) .
 			"</td>
 				<td class='mw-input'>" .
-			Html::namespaceSelector( array(
+			Html::namespaceSelector( [
 				'selected' => $namespace,
-			), array(
+			], [
 				'name' => 'namespace',
 				'id' => 'namespace',
 				'class' => 'namespaceselector',
-			) ) .
+			] ) .
 			Xml::checkLabel(
 				$this->msg( 'allpages-hide-redirects' )->text(),
 				'hideredirects',
@@ -141,7 +138,7 @@ class SpecialPrefixindex extends SpecialAllpages {
 				'stripprefix',
 				$this->stripPrefix
 			) . ' ' .
-			Xml::submitButton( $this->msg( 'allpagessubmit' )->text() ) .
+			Xml::submitButton( $this->msg( 'prefixindex-submit' )->text() ) .
 			"</td>
 			</tr>";
 		$out .= Xml::closeElement( 'table' );
@@ -153,9 +150,9 @@ class SpecialPrefixindex extends SpecialAllpages {
 	}
 
 	/**
-	 * @param $namespace Integer, default NS_MAIN
-	 * @param $prefix String
-	 * @param string $from list all pages from this name (default FALSE)
+	 * @param int $namespace Default NS_MAIN
+	 * @param string $prefix
+	 * @param string $from List all pages from this name (default false)
 	 */
 	protected function showPrefixChunk( $namespace = NS_MAIN, $prefix, $from = null ) {
 		global $wgContLang;
@@ -167,6 +164,7 @@ class SpecialPrefixindex extends SpecialAllpages {
 		$fromList = $this->getNamespaceKeyAndText( $namespace, $from );
 		$prefixList = $this->getNamespaceKeyAndText( $namespace, $prefix );
 		$namespaces = $wgContLang->getNamespaces();
+		$res = null;
 
 		if ( !$prefixList || !$fromList ) {
 			$out = $this->msg( 'allpagesbadtitle' )->parseAsBlock();
@@ -178,36 +176,36 @@ class SpecialPrefixindex extends SpecialAllpages {
 			list( $namespace, $prefixKey, $prefix ) = $prefixList;
 			list( /* $fromNS */, $fromKey, ) = $fromList;
 
-			### @todo FIXME: Should complain if $fromNs != $namespace
+			# ## @todo FIXME: Should complain if $fromNs != $namespace
 
 			$dbr = wfGetDB( DB_SLAVE );
 
-			$conds = array(
+			$conds = [
 				'page_namespace' => $namespace,
 				'page_title' . $dbr->buildLike( $prefixKey, $dbr->anyString() ),
 				'page_title >= ' . $dbr->addQuotes( $fromKey ),
-			);
+			];
 
 			if ( $this->hideRedirects ) {
 				$conds['page_is_redirect'] = 0;
 			}
 
 			$res = $dbr->select( 'page',
-				array( 'page_namespace', 'page_title', 'page_is_redirect' ),
+				[ 'page_namespace', 'page_title', 'page_is_redirect' ],
 				$conds,
 				__METHOD__,
-				array(
+				[
 					'ORDER BY' => 'page_title',
 					'LIMIT' => $this->maxPerPage + 1,
 					'USE INDEX' => 'name_title',
-				)
+				]
 			);
 
-			### @todo FIXME: Side link to previous
+			// @todo FIXME: Side link to previous
 
 			$n = 0;
 			if ( $res->numRows() > 0 ) {
-				$out = Xml::openElement( 'table', array( 'class' => 'mw-prefixindex-list-table' ) );
+				$out = Html::openElement( 'ul', [ 'class' => 'mw-prefixindex-list' ] );
 
 				$prefixLength = strlen( $prefix );
 				while ( ( $n < $this->maxPerPage ) && ( $s = $res->fetchObject() ) ) {
@@ -222,83 +220,90 @@ class SpecialPrefixindex extends SpecialAllpages {
 							Linker::linkKnown(
 								$t,
 								htmlspecialchars( $displayed ),
-								$s->page_is_redirect ? array( 'class' => 'mw-redirect' ) : array()
+								$s->page_is_redirect ? [ 'class' => 'mw-redirect' ] : []
 							) .
 							( $s->page_is_redirect ? '</div>' : '' );
 					} else {
 						$link = '[[' . htmlspecialchars( $s->page_title ) . ']]';
 					}
-					if ( $n % $this->columns == 0 ) {
-						$out .= '<tr>';
-					}
-					$out .= "<td>$link</td>";
+
+					$out .= "<li>$link</li>\n";
 					$n++;
-					if ( $n % $this->columns == 0 ) {
-						$out .= '</tr>';
-					}
-				}
 
-				if ( $n % $this->columns != 0 ) {
-					$out .= '</tr>';
 				}
+				$out .= Html::closeElement( 'ul' );
 
-				$out .= Xml::closeElement( 'table' );
+				if ( $res->numRows() > 2 ) {
+					// Only apply CSS column styles if there's more than 2 entries.
+					// Otherwise rendering is broken as "mw-prefixindex-body"'s CSS column count is 3.
+					$out = Html::rawElement( 'div', [ 'class' => 'mw-prefixindex-body' ], $out );
+				}
 			} else {
 				$out = '';
 			}
 		}
 
-		$footer = '';
+		$output = $this->getOutput();
+
 		if ( $this->including() ) {
-			$out2 = '';
-		} else {
-			$nsForm = $this->namespacePrefixForm( $namespace, $prefix );
-			$self = $this->getPageTitle();
-			$out2 = Xml::openElement( 'table', array( 'id' => 'mw-prefixindex-nav-table' ) ) .
-				'<tr>
-					<td>' .
-				$nsForm .
-				'</td>
-				<td id="mw-prefixindex-nav-form" class="mw-prefixindex-nav">';
-
-			if ( isset( $res ) && $res && ( $n == $this->maxPerPage ) &&
-				( $s = $res->fetchObject() )
-			) {
-				$query = array(
-					'from' => $s->page_title,
-					'prefix' => $prefix,
-					'hideredirects' => $this->hideRedirects,
-					'stripprefix' => $this->stripPrefix,
-					'columns' => $this->columns,
-				);
-
-				if ( $namespace || $prefix == '' ) {
-					// Keep the namespace even if it's 0 for empty prefixes.
-					// This tells us we're not just a holdover from old links.
-					$query['namespace'] = $namespace;
-				}
-
-				$nextLink = Linker::linkKnown(
-					$self,
-					$this->msg( 'nextpage', str_replace( '_', ' ', $s->page_title ) )->escaped(),
-					array(),
-					$query
-				);
-
-				$out2 .= $nextLink;
-
-				$footer = "\n" . Html::element( 'hr' ) .
-					Html::rawElement(
-						'div',
-						array( 'class' => 'mw-prefixindex-nav' ),
-						$nextLink
-					);
-			}
-			$out2 .= "</td></tr>" .
-				Xml::closeElement( 'table' );
+			// We don't show the nav-links and the form when included into other
+			// pages so let's just finish here.
+			$output->addHTML( $out );
+			return;
 		}
 
-		$this->getOutput()->addHTML( $out2 . $out . $footer );
+		$topOut = $this->namespacePrefixForm( $namespace, $prefix );
+
+		if ( $res && ( $n == $this->maxPerPage ) && ( $s = $res->fetchObject() ) ) {
+			$query = [
+				'from' => $s->page_title,
+				'prefix' => $prefix,
+				'hideredirects' => $this->hideRedirects,
+				'stripprefix' => $this->stripPrefix,
+			];
+
+			if ( $namespace || $prefix == '' ) {
+				// Keep the namespace even if it's 0 for empty prefixes.
+				// This tells us we're not just a holdover from old links.
+				$query['namespace'] = $namespace;
+			}
+
+			$nextLink = Linker::linkKnown(
+				$this->getPageTitle(),
+				$this->msg( 'nextpage', str_replace( '_', ' ', $s->page_title ) )->escaped(),
+				[],
+				$query
+			);
+
+			// Link shown at the top of the page below the form
+			$topOut .= Html::rawElement( 'div',
+				[ 'class' => 'mw-prefixindex-nav' ],
+				$nextLink
+			);
+
+			// Link shown at the footer
+			$out .= "\n" . Html::element( 'hr' ) .
+				Html::rawElement(
+					'div',
+					[ 'class' => 'mw-prefixindex-nav' ],
+					$nextLink
+				);
+
+		}
+
+		$output->addHTML( $topOut . $out );
+	}
+
+	/**
+	 * Return an array of subpages beginning with $search that this special page will accept.
+	 *
+	 * @param string $search Prefix to search for
+	 * @param int $limit Maximum number of results to return (usually 10)
+	 * @param int $offset Number of results to skip (usually 0)
+	 * @return string[] Matching subpages
+	 */
+	public function prefixSearchSubpages( $search, $limit, $offset ) {
+		return $this->prefixSearchString( $search, $limit, $offset );
 	}
 
 	protected function getGroupName() {
