@@ -1,8 +1,7 @@
 <?php
 /** Simplified Gan script (赣语（简体）‎)
  *
- * See MessagesQqq.php for message documentation incl. usage of parameters
- * To improve a translation please visit http://translatewiki.net
+ * To improve a translation please visit https://translatewiki.net
  *
  * @ingroup Language
  * @file
@@ -11,7 +10,7 @@
 
 $fallback = 'zh-hans';
 
-$namespaceNames = array(
+$namespaceNames = [
 	NS_TALK             => '谈詑',
 	NS_USER_TALK        => '用户谈詑',
 	NS_PROJECT_TALK     => '$1谈詑',
@@ -20,5 +19,5 @@ $namespaceNames = array(
 	NS_TEMPLATE_TALK    => '模板谈詑',
 	NS_HELP_TALK        => '帮助谈詑',
 	NS_CATEGORY_TALK    => '分类谈詑',
-);
+];
 
