@@ -22,7 +22,7 @@
  * @author Rob Church <robchur@gmail.com>
  */
 
-require_once( __DIR__ . '/Maintenance.php' );
+require_once __DIR__ . '/Maintenance.php';
 
 /**
  * Maintenance script that purges old text records from the database.
@@ -32,7 +32,7 @@ require_once( __DIR__ . '/Maintenance.php' );
 class PurgeOldText extends Maintenance {
 	public function __construct() {
 		parent::__construct();
-		$this->mDescription = "Purge old text records from the database";
+		$this->addDescription( 'Purge old text records from the database' );
 		$this->addOption( 'purge', 'Performs the deletion' );
 	}
 
@@ -42,4 +42,4 @@ class PurgeOldText extends Maintenance {
 }
 
 $maintClass = "PurgeOldText";
-require_once( RUN_MAINTENANCE_IF_MAIN );
+require_once RUN_MAINTENANCE_IF_MAIN;
