@@ -22,6 +22,7 @@
  */
 
 if ( !defined( 'MEDIAWIKI' ) ) {
+	$optionsWithoutArgs = [ 'fix' ];
 	require_once __DIR__ . '/../commandLine.inc';
 
 	$cs = new CheckStorage;
@@ -39,6 +40,7 @@ if ( !defined( 'MEDIAWIKI' ) ) {
 /**
  * Maintenance script to do various checks on external storage.
  *
+ * @fixme this should extend the base Maintenance class
  * @ingroup Maintenance ExternalStorage
  */
 class CheckStorage {
@@ -46,13 +48,13 @@ class CheckStorage {
 	public $oldIdMap, $errors;
 	public $dbStore = null;
 
-	public $errorDescriptions = array(
+	public $errorDescriptions = [
 		'restore text' => 'Damaged text, need to be restored from a backup',
 		'restore revision' => 'Damaged revision row, need to be restored from a backup',
 		'unfixable' => 'Unexpected errors with no automated fixing method',
 		'fixed' => 'Errors already fixed',
 		'fixable' => 'Errors which would already be fixed if --fix was specified',
-	);
+	];
 
 	function check( $fix = false, $xml = '' ) {
 		$dbr = wfGetDB( DB_SLAVE );
@@ -63,26 +65,26 @@ class CheckStorage {
 		}
 		$maxRevId = $dbr->selectField( 'revision', 'MAX(rev_id)', false, __METHOD__ );
 		$chunkSize = 1000;
-		$flagStats = array();
-		$objectStats = array();
-		$knownFlags = array( 'external', 'gzip', 'object', 'utf-8' );
-		$this->errors = array(
-			'restore text' => array(),
-			'restore revision' => array(),
-			'unfixable' => array(),
-			'fixed' => array(),
-			'fixable' => array(),
-		);
+		$flagStats = [];
+		$objectStats = [];
+		$knownFlags = [ 'external', 'gzip', 'object', 'utf-8' ];
+		$this->errors = [
+			'restore text' => [],
+			'restore revision' => [],
+			'unfixable' => [],
+			'fixed' => [],
+			'fixable' => [],
+		];
 
 		for ( $chunkStart = 1; $chunkStart < $maxRevId; $chunkStart += $chunkSize ) {
 			$chunkEnd = $chunkStart + $chunkSize - 1;
 			// print "$chunkStart of $maxRevId\n";
 
 			// Fetch revision rows
-			$this->oldIdMap = array();
+			$this->oldIdMap = [];
 			$dbr->ping();
-			$res = $dbr->select( 'revision', array( 'rev_id', 'rev_text_id' ),
-				array( "rev_id BETWEEN $chunkStart AND $chunkEnd" ), __METHOD__ );
+			$res = $dbr->select( 'revision', [ 'rev_id', 'rev_text_id' ],
+				[ "rev_id BETWEEN $chunkStart AND $chunkEnd" ], __METHOD__ );
 			foreach ( $res as $row ) {
 				$this->oldIdMap[$row->rev_id] = $row->rev_text_id;
 			}
@@ -94,9 +96,9 @@ class CheckStorage {
 
 			// Fetch old_flags
 			$missingTextRows = array_flip( $this->oldIdMap );
-			$externalRevs = array();
-			$objectRevs = array();
-			$res = $dbr->select( 'text', array( 'old_id', 'old_flags' ),
+			$externalRevs = [];
+			$objectRevs = [];
+			$res = $dbr->select( 'text', [ 'old_id', 'old_flags' ],
 				'old_id IN (' . implode( ',', $this->oldIdMap ) . ')', __METHOD__ );
 			foreach ( $res as $row ) {
 				/**
@@ -106,7 +108,7 @@ class CheckStorage {
 				$id = $row->old_id;
 
 				// Create flagStats row if it doesn't exist
-				$flagStats = $flagStats + array( $flags => 0 );
+				$flagStats = $flagStats + [ $flags => 0 ];
 				// Increment counter
 				$flagStats[$flags]++;
 
@@ -115,7 +117,7 @@ class CheckStorage {
 
 				// Check for external or object
 				if ( $flags == '' ) {
-					$flagArray = array();
+					$flagArray = [];
 				} else {
 					$flagArray = explode( ',', $flags );
 				}
@@ -133,8 +135,8 @@ class CheckStorage {
 						$this->error( 'fixed', "Warning: old_flags set to 0", $id );
 						$dbw = wfGetDB( DB_MASTER );
 						$dbw->ping();
-						$dbw->update( 'text', array( 'old_flags' => '' ),
-							array( 'old_id' => $id ), __METHOD__ );
+						$dbw->update( 'text', [ 'old_flags' => '' ],
+							[ 'old_id' => $id ], __METHOD__ );
 						echo "Fixed\n";
 					} else {
 						$this->error( 'fixable', "Warning: old_flags set to 0", $id );
@@ -151,11 +153,11 @@ class CheckStorage {
 			}
 
 			// Verify external revisions
-			$externalConcatBlobs = array();
-			$externalNormalBlobs = array();
+			$externalConcatBlobs = [];
+			$externalNormalBlobs = [];
 			if ( count( $externalRevs ) ) {
-				$res = $dbr->select( 'text', array( 'old_id', 'old_flags', 'old_text' ),
-					array( 'old_id IN (' . implode( ',', $externalRevs ) . ')' ), __METHOD__ );
+				$res = $dbr->select( 'text', [ 'old_id', 'old_flags', 'old_text' ],
+					[ 'old_id IN (' . implode( ',', $externalRevs ) . ')' ], __METHOD__ );
 				foreach ( $res as $row ) {
 					$urlParts = explode( '://', $row->old_text, 2 );
 					if ( count( $urlParts ) !== 2 || $urlParts[1] == '' ) {
@@ -192,8 +194,8 @@ class CheckStorage {
 					$extDb =& $this->dbStore->getSlave( $cluster );
 					$blobsTable = $this->dbStore->getTable( $extDb );
 					$res = $extDb->select( $blobsTable,
-						array( 'blob_id' ),
-						array( 'blob_id IN( ' . implode( ',', $blobIds ) . ')' ), __METHOD__ );
+						[ 'blob_id' ],
+						[ 'blob_id IN( ' . implode( ',', $blobIds ) . ')' ], __METHOD__ );
 					foreach ( $res as $row ) {
 						unset( $xBlobIds[$row->blob_id] );
 					}
@@ -207,15 +209,19 @@ class CheckStorage {
 
 			// Check local objects
 			$dbr->ping();
-			$concatBlobs = array();
-			$curIds = array();
+			$concatBlobs = [];
+			$curIds = [];
 			if ( count( $objectRevs ) ) {
 				$headerLength = 300;
-				$res = $dbr->select( 'text', array( 'old_id', 'old_flags', "LEFT(old_text, $headerLength) AS header" ),
-					array( 'old_id IN (' . implode( ',', $objectRevs ) . ')' ), __METHOD__ );
+				$res = $dbr->select(
+					'text',
+					[ 'old_id', 'old_flags', "LEFT(old_text, $headerLength) AS header" ],
+					[ 'old_id IN (' . implode( ',', $objectRevs ) . ')' ],
+					__METHOD__
+				);
 				foreach ( $res as $row ) {
 					$oldId = $row->old_id;
-					$matches = array();
+					$matches = [];
 					if ( !preg_match( '/^O:(\d+):"(\w+)"/', $row->header, $matches ) ) {
 						$this->error( 'restore text', "Error: invalid object header", $oldId );
 						continue;
@@ -223,11 +229,15 @@ class CheckStorage {
 
 					$className = strtolower( $matches[2] );
 					if ( strlen( $className ) != $matches[1] ) {
-						$this->error( 'restore text', "Error: invalid object header, wrong class name length", $oldId );
+						$this->error(
+							'restore text',
+							"Error: invalid object header, wrong class name length",
+							$oldId
+						);
 						continue;
 					}
 
-					$objectStats = $objectStats + array( $className => 0 );
+					$objectStats = $objectStats + [ $className => 0 ];
 					$objectStats[$className]++;
 
 					switch ( $className ) {
@@ -259,11 +269,15 @@ class CheckStorage {
 			}
 
 			// Check local concat blob validity
-			$externalConcatBlobs = array();
+			$externalConcatBlobs = [];
 			if ( count( $concatBlobs ) ) {
 				$headerLength = 300;
-				$res = $dbr->select( 'text', array( 'old_id', 'old_flags', "LEFT(old_text, $headerLength) AS header" ),
-					array( 'old_id IN (' . implode( ',', array_keys( $concatBlobs ) ) . ')' ), __METHOD__ );
+				$res = $dbr->select(
+					'text',
+					[ 'old_id', 'old_flags', "LEFT(old_text, $headerLength) AS header" ],
+					[ 'old_id IN (' . implode( ',', array_keys( $concatBlobs ) ) . ')' ],
+					__METHOD__
+				);
 				foreach ( $res as $row ) {
 					$flags = explode( ',', $row->old_flags );
 					if ( in_array( 'external', $flags ) ) {
@@ -271,24 +285,36 @@ class CheckStorage {
 						if ( in_array( 'object', $flags ) ) {
 							$urlParts = explode( '/', $row->header );
 							if ( $urlParts[0] != 'DB:' ) {
-								$this->error( 'unfixable', "Error: unrecognised external storage type \"{$urlParts[0]}", $row->old_id );
+								$this->error(
+									'unfixable',
+									"Error: unrecognised external storage type \"{$urlParts[0]}",
+									$row->old_id
+								);
 							} else {
 								$cluster = $urlParts[2];
 								$id = $urlParts[3];
 								if ( !isset( $externalConcatBlobs[$cluster][$id] ) ) {
-									$externalConcatBlobs[$cluster][$id] = array();
+									$externalConcatBlobs[$cluster][$id] = [];
 								}
 								$externalConcatBlobs[$cluster][$id] = array_merge(
 									$externalConcatBlobs[$cluster][$id], $concatBlobs[$row->old_id]
 								);
 							}
 						} else {
-							$this->error( 'unfixable', "Error: invalid flags \"{$row->old_flags}\" on concat bulk row {$row->old_id}",
+							$this->error(
+								'unfixable',
+								"Error: invalid flags \"{$row->old_flags}\" on concat bulk row {$row->old_id}",
 								$concatBlobs[$row->old_id] );
 						}
-					} elseif ( strcasecmp( substr( $row->header, 0, strlen( self::CONCAT_HEADER ) ), self::CONCAT_HEADER ) ) {
-						$this->error( 'restore text', "Error: Incorrect object header for concat bulk row {$row->old_id}",
-							$concatBlobs[$row->old_id] );
+					} elseif ( strcasecmp(
+						substr( $row->header, 0, strlen( self::CONCAT_HEADER ) ),
+						self::CONCAT_HEADER
+					) ) {
+						$this->error(
+							'restore text',
+							"Error: Incorrect object header for concat bulk row {$row->old_id}",
+							$concatBlobs[$row->old_id]
+						);
 					} # else good
 
 					unset( $concatBlobs[$row->old_id] );
@@ -298,7 +324,6 @@ class CheckStorage {
 
 			// Check targets of unresolved stubs
 			$this->checkExternalConcatBlobs( $externalConcatBlobs );
-
 			// next chunk
 		}
 
@@ -335,7 +360,7 @@ class CheckStorage {
 			$ids = reset( $ids );
 		}
 		if ( is_array( $ids ) ) {
-			$revIds = array();
+			$revIds = [];
 			foreach ( $ids as $id ) {
 				$revIds = array_merge( $revIds, array_keys( $this->oldIdMap, $id ) );
 			}
@@ -368,21 +393,27 @@ class CheckStorage {
 			$blobsTable = $this->dbStore->getTable( $extDb );
 			$headerLength = strlen( self::CONCAT_HEADER );
 			$res = $extDb->select( $blobsTable,
-				array( 'blob_id', "LEFT(blob_text, $headerLength) AS header" ),
-				array( 'blob_id IN( ' . implode( ',', $blobIds ) . ')' ), __METHOD__ );
+				[ 'blob_id', "LEFT(blob_text, $headerLength) AS header" ],
+				[ 'blob_id IN( ' . implode( ',', $blobIds ) . ')' ], __METHOD__ );
 			foreach ( $res as $row ) {
 				if ( strcasecmp( $row->header, self::CONCAT_HEADER ) ) {
-					$this->error( 'restore text', "Error: invalid header on target $cluster/{$row->blob_id} of two-part ES URL",
-						$oldIds[$row->blob_id] );
+					$this->error(
+						'restore text',
+						"Error: invalid header on target $cluster/{$row->blob_id} of two-part ES URL",
+						$oldIds[$row->blob_id]
+					);
 				}
 				unset( $oldIds[$row->blob_id] );
-
 			}
 			$extDb->freeResult( $res );
 
 			// Print errors for missing blobs rows
 			foreach ( $oldIds as $blobId => $oldIds2 ) {
-				$this->error( 'restore text', "Error: missing target $cluster/$blobId for two-part ES URL", $oldIds2 );
+				$this->error(
+					'restore text',
+					"Error: missing target $cluster/$blobId for two-part ES URL",
+					$oldIds2
+				);
 			}
 		}
 	}
@@ -403,6 +434,7 @@ class CheckStorage {
 		// Write revision list
 		if ( !file_put_contents( $revFileName, implode( "\n", $revIds ) ) ) {
 			echo "Error writing revision list, can't restore text\n";
+
 			return;
 		}
 
@@ -419,12 +451,14 @@ class CheckStorage {
 
 		if ( $exitStatus ) {
 			echo "mwdumper died with exit status $exitStatus\n";
+
 			return;
 		}
 
 		$file = fopen( $filteredXmlFileName, 'r' );
 		if ( !$file ) {
 			echo "Unable to open filtered XML file\n";
+
 			return;
 		}
 
@@ -434,8 +468,11 @@ class CheckStorage {
 		$dbw->ping();
 
 		$source = new ImportStreamSource( $file );
-		$importer = new WikiImporter( $source );
-		$importer->setRevisionCallback( array( &$this, 'importRevision' ) );
+		$importer = new WikiImporter(
+			$source,
+			ConfigFactory::getDefaultInstance()->makeConfig( 'main' )
+		);
+		$importer->setRevisionCallback( [ $this, 'importRevision' ] );
 		$importer->doImport();
 	}
 
@@ -446,6 +483,7 @@ class CheckStorage {
 
 		if ( $content === null ) {
 			echo "Revision $id is broken, we have no content available\n";
+
 			return;
 		}
 
@@ -457,20 +495,23 @@ class CheckStorage {
 			// be safe, we'll skip it and leave it broken
 
 			echo "Revision $id is blank in the dump, may have been broken before export\n";
+
 			return;
 		}
 
 		if ( !$id ) {
 			// No ID, can't import
 			echo "No id tag in revision, can't import\n";
+
 			return;
 		}
 
 		// Find text row again
 		$dbr = wfGetDB( DB_SLAVE );
-		$oldId = $dbr->selectField( 'revision', 'rev_text_id', array( 'rev_id' => $id ), __METHOD__ );
+		$oldId = $dbr->selectField( 'revision', 'rev_text_id', [ 'rev_id' => $id ], __METHOD__ );
 		if ( !$oldId ) {
 			echo "Missing revision row for rev_id $id\n";
+
 			return;
 		}
 
@@ -480,9 +521,9 @@ class CheckStorage {
 		// Update the text row
 		$dbw = wfGetDB( DB_MASTER );
 		$dbw->update( 'text',
-			array( 'old_flags' => $flags, 'old_text' => $text ),
-			array( 'old_id' => $oldId ),
-			__METHOD__, array( 'LIMIT' => 1 )
+			[ 'old_flags' => $flags, 'old_text' => $text ],
+			[ 'old_id' => $oldId ],
+			__METHOD__, [ 'LIMIT' => 1 ]
 		);
 
 		// Remove it from the unfixed list and add it to the fixed list
