@@ -22,19 +22,23 @@
  * @ingroup Maintenance ExternalStorage
  */
 
+use MediaWiki\Logger\LegacyLogger;
+
 $optionsWithArgs = RecompressTracked::getOptionsWithArgs();
 require __DIR__ . '/../commandLine.inc';
 
 if ( count( $args ) < 1 ) {
 	echo "Usage: php recompressTracked.php [options] <cluster> [... <cluster>...]
-Moves blobs indexed by trackBlobs.php to a specified list of destination clusters, and recompresses them in the process. Restartable.
+Moves blobs indexed by trackBlobs.php to a specified list of destination clusters,
+and recompresses them in the process. Restartable.
 
 Options:
-	--procs <procs>         Set the number of child processes (default 1)
-	--copy-only             Copy only, do not update the text table. Restart without this option to complete.
-	--debug-log <file>      Log debugging data to the specified file
-	--info-log <file>       Log progress messages to the specified file
-	--critical-log <file>   Log error messages to the specified file
+	--procs <procs>       Set the number of child processes (default 1)
+	--copy-only           Copy only, do not update the text table. Restart
+	                      without this option to complete.
+	--debug-log <file>    Log debugging data to the specified file
+	--info-log <file>     Log progress messages to the specified file
+	--critical-log <file> Log error messages to the specified file
 ";
 	exit( 1 );
 }
@@ -54,6 +58,7 @@ class RecompressTracked {
 	public $orphanBatchSize = 1000;
 	public $reportingInterval = 10;
 	public $numProcs = 1;
+	public $numBatches = 0;
 	public $useDiff, $pageBlobClass, $orphanBlobClass;
 	public $slavePipes, $slaveProcs, $prevSlaveId;
 	public $copyOnly = false;
@@ -63,8 +68,15 @@ class RecompressTracked {
 	public $debugLog, $infoLog, $criticalLog;
 	public $store;
 
-	static $optionsWithArgs = array( 'procs', 'slave-id', 'debug-log', 'info-log', 'critical-log' );
-	static $cmdLineOptionMap = array(
+	private static $optionsWithArgs = [
+		'procs',
+		'slave-id',
+		'debug-log',
+		'info-log',
+		'critical-log'
+	];
+
+	private static $cmdLineOptionMap = [
 		'no-count' => 'noCount',
 		'procs' => 'numProcs',
 		'copy-only' => 'copyOnly',
@@ -73,19 +85,20 @@ class RecompressTracked {
 		'debug-log' => 'debugLog',
 		'info-log' => 'infoLog',
 		'critical-log' => 'criticalLog',
-	);
+	];
 
 	static function getOptionsWithArgs() {
 		return self::$optionsWithArgs;
 	}
 
 	static function newFromCommandLine( $args, $options ) {
-		$jobOptions = array( 'destClusters' => $args );
+		$jobOptions = [ 'destClusters' => $args ];
 		foreach ( self::$cmdLineOptionMap as $cmdOption => $classOption ) {
 			if ( isset( $options[$cmdOption] ) ) {
 				$jobOptions[$classOption] = $options[$cmdOption];
 			}
 		}
+
 		return new self( $jobOptions );
 	}
 
@@ -109,7 +122,6 @@ class RecompressTracked {
 		if ( $this->debugLog ) {
 			$this->logToFile( $msg, $this->debugLog );
 		}
-
 	}
 
 	function info( $msg ) {
@@ -132,7 +144,7 @@ class RecompressTracked {
 			$header .= "({$this->slaveId})";
 		}
 		$header .= ' ' . wfWikiID();
-		wfErrorLog( sprintf( "%-50s %s\n", $header, $msg ), $file );
+		LegacyLogger::emit( sprintf( "%-50s %s\n", $header, $msg ), $file );
 	}
 
 	/**
@@ -181,13 +193,16 @@ class RecompressTracked {
 		$dbr = wfGetDB( DB_SLAVE );
 		if ( !$dbr->tableExists( 'blob_tracking' ) ) {
 			$this->critical( "Error: blob_tracking table does not exist" );
+
 			return false;
 		}
-		$row = $dbr->selectRow( 'blob_tracking', '*', false, __METHOD__ );
+		$row = $dbr->selectRow( 'blob_tracking', '*', '', __METHOD__ );
 		if ( !$row ) {
 			$this->info( "Warning: blob_tracking table contains no rows, skipping this wiki." );
+
 			return false;
 		}
+
 		return true;
 	}
 
@@ -212,17 +227,17 @@ class RecompressTracked {
 			' --wiki ' . wfEscapeShellArg( wfWikiID() ) .
 			' ' . call_user_func_array( 'wfEscapeShellArg', $this->destClusters );
 
-		$this->slavePipes = $this->slaveProcs = array();
+		$this->slavePipes = $this->slaveProcs = [];
 		for ( $i = 0; $i < $this->numProcs; $i++ ) {
-			$pipes = false;
-			$spec = array(
-				array( 'pipe', 'r' ),
-				array( 'file', 'php://stdout', 'w' ),
-				array( 'file', 'php://stderr', 'w' )
-			);
-			wfSuppressWarnings();
+			$pipes = [];
+			$spec = [
+				[ 'pipe', 'r' ],
+				[ 'file', 'php://stdout', 'w' ],
+				[ 'file', 'php://stderr', 'w' ]
+			];
+			MediaWiki\suppressWarnings();
 			$proc = proc_open( "$cmd --slave-id $i", $spec, $pipes );
-			wfRestoreWarnings();
+			MediaWiki\restoreWarnings();
 			if ( !$proc ) {
 				$this->critical( "Error opening slave process: $cmd" );
 				exit( 1 );
@@ -257,7 +272,7 @@ class RecompressTracked {
 	function dispatch( /*...*/ ) {
 		$args = func_get_args();
 		$pipes = $this->slavePipes;
-		$numPipes = stream_select( $x = array(), $pipes, $y = array(), 3600 );
+		$numPipes = stream_select( $x = [], $pipes, $y = [], 3600 );
 		if ( !$numPipes ) {
 			$this->critical( "Error waiting to write to slaves. Aborting" );
 			exit( 1 );
@@ -267,6 +282,7 @@ class RecompressTracked {
 			if ( isset( $pipes[$slaveId] ) ) {
 				$this->prevSlaveId = $slaveId;
 				$this->dispatchToSlave( $slaveId, $args );
+
 				return;
 			}
 		}
@@ -276,6 +292,8 @@ class RecompressTracked {
 
 	/**
 	 * Dispatch a command to a specified slave
+	 * @param int $slaveId
+	 * @param array|string $args
 	 */
 	function dispatchToSlave( $slaveId, $args ) {
 		$args = (array)$args;
@@ -296,7 +314,7 @@ class RecompressTracked {
 			$numPages = $dbr->selectField( 'blob_tracking',
 				'COUNT(DISTINCT bt_page)',
 				# A condition is required so that this query uses the index
-				array( 'bt_moved' => 0 ),
+				[ 'bt_moved' => 0 ],
 				__METHOD__
 			);
 		}
@@ -307,26 +325,26 @@ class RecompressTracked {
 		}
 		while ( true ) {
 			$res = $dbr->select( 'blob_tracking',
-				array( 'bt_page' ),
-				array(
+				[ 'bt_page' ],
+				[
 					'bt_moved' => 0,
 					'bt_page > ' . $dbr->addQuotes( $startId )
-				),
+				],
 				__METHOD__,
-				array(
+				[
 					'DISTINCT',
 					'ORDER BY' => 'bt_page',
 					'LIMIT' => $this->batchSize,
-				)
+				]
 			);
 			if ( !$res->numRows() ) {
 				break;
 			}
 			foreach ( $res as $row ) {
+				$startId = $row->bt_page;
 				$this->dispatch( 'doPage', $row->bt_page );
 				$i++;
 			}
-			$startId = $row->bt_page;
 			$this->report( 'pages', $i, $numPages );
 		}
 		$this->report( 'pages', $i, $numPages );
@@ -339,13 +357,16 @@ class RecompressTracked {
 
 	/**
 	 * Display a progress report
+	 * @param string $label
+	 * @param int $current
+	 * @param int $end
 	 */
 	function report( $label, $current, $end ) {
 		$this->numBatches++;
 		if ( $current == $end || $this->numBatches >= $this->reportingInterval ) {
 			$this->numBatches = 0;
 			$this->info( "$label: $current / $end" );
-			$this->waitForSlaves();
+			wfWaitForSlaves();
 		}
 	}
 
@@ -361,7 +382,7 @@ class RecompressTracked {
 		} else {
 			$numOrphans = $dbr->selectField( 'blob_tracking',
 				'COUNT(DISTINCT bt_text_id)',
-				array( 'bt_moved' => 0, 'bt_page' => 0 ),
+				[ 'bt_moved' => 0, 'bt_page' => 0 ],
 				__METHOD__ );
 			if ( !$numOrphans ) {
 				return;
@@ -375,24 +396,25 @@ class RecompressTracked {
 
 		while ( true ) {
 			$res = $dbr->select( 'blob_tracking',
-				array( 'bt_text_id' ),
-				array(
+				[ 'bt_text_id' ],
+				[
 					'bt_moved' => 0,
 					'bt_page' => 0,
 					'bt_text_id > ' . $dbr->addQuotes( $startId )
-				),
+				],
 				__METHOD__,
-				array(
+				[
 					'DISTINCT',
 					'ORDER BY' => 'bt_text_id',
 					'LIMIT' => $this->batchSize
-				)
+				]
 			);
 			if ( !$res->numRows() ) {
 				break;
 			}
-			$ids = array();
+			$ids = [];
 			foreach ( $res as $row ) {
+				$startId = $row->bt_text_id;
 				$ids[] = $row->bt_text_id;
 				$i++;
 			}
@@ -403,15 +425,14 @@ class RecompressTracked {
 				$args = array_slice( $ids, 0, $this->orphanBatchSize );
 				$ids = array_slice( $ids, $this->orphanBatchSize );
 				array_unshift( $args, 'doOrphanList' );
-				call_user_func_array( array( $this, 'dispatch' ), $args );
+				call_user_func_array( [ $this, 'dispatch' ], $args );
 			}
 			if ( count( $ids ) ) {
 				$args = $ids;
 				array_unshift( $args, 'doOrphanList' );
-				call_user_func_array( array( $this, 'dispatch' ), $args );
+				call_user_func_array( [ $this, 'dispatch' ], $args );
 			}
 
-			$startId = $row->bt_text_id;
 			$this->report( 'orphans', $i, $numOrphans );
 		}
 		$this->report( 'orphans', $i, $numOrphans );
@@ -434,24 +455,26 @@ class RecompressTracked {
 			$args = explode( ' ', $line );
 			$cmd = array_shift( $args );
 			switch ( $cmd ) {
-			case 'doPage':
-				$this->doPage( intval( $args[0] ) );
-				break;
-			case 'doOrphanList':
-				$this->doOrphanList( array_map( 'intval', $args ) );
-				break;
-			case 'quit':
-				return;
+				case 'doPage':
+					$this->doPage( intval( $args[0] ) );
+					break;
+				case 'doOrphanList':
+					$this->doOrphanList( array_map( 'intval', $args ) );
+					break;
+				case 'quit':
+					return;
 			}
-			$this->waitForSlaves();
+			wfWaitForSlaves();
 		}
 	}
 
 	/**
 	 * Move tracked text in a given page
+	 *
+	 * @param int $pageId
 	 */
 	function doPage( $pageId ) {
-		$title = Title::newFromId( $pageId );
+		$title = Title::newFromID( $pageId );
 		if ( $title ) {
 			$titleText = $title->getPrefixedText();
 		} else {
@@ -461,7 +484,7 @@ class RecompressTracked {
 
 		// Finish any incomplete transactions
 		if ( !$this->copyOnly ) {
-			$this->finishIncompleteMoves( array( 'bt_page' => $pageId ) );
+			$this->finishIncompleteMoves( [ 'bt_page' => $pageId ] );
 			$this->syncDBs();
 		}
 
@@ -470,20 +493,20 @@ class RecompressTracked {
 
 		while ( true ) {
 			$res = $dbr->select(
-				array( 'blob_tracking', 'text' ),
+				[ 'blob_tracking', 'text' ],
 				'*',
-				array(
+				[
 					'bt_page' => $pageId,
 					'bt_text_id > ' . $dbr->addQuotes( $startId ),
 					'bt_moved' => 0,
 					'bt_new_url IS NULL',
 					'bt_text_id=old_id',
-				),
+				],
 				__METHOD__,
-				array(
+				[
 					'ORDER BY' => 'bt_text_id',
 					'LIMIT' => $this->batchSize
-				)
+				]
 			);
 			if ( !$res->numRows() ) {
 				break;
@@ -491,6 +514,7 @@ class RecompressTracked {
 
 			$lastTextId = 0;
 			foreach ( $res as $row ) {
+				$startId = $row->bt_text_id;
 				if ( $lastTextId == $row->bt_text_id ) {
 					// Duplicate (null edit)
 					continue;
@@ -508,10 +532,9 @@ class RecompressTracked {
 					$this->debug( "$titleText: committing blob with " . $trx->getSize() . " items" );
 					$trx->commit();
 					$trx = new CgzCopyTransaction( $this, $this->pageBlobClass );
-					$this->waitForSlaves();
+					wfWaitForSlaves();
 				}
 			}
-			$startId = $row->bt_text_id;
 		}
 
 		$this->debug( "$titleText: committing blob with " . $trx->getSize() . " items" );
@@ -527,6 +550,9 @@ class RecompressTracked {
 	 * without data loss.
 	 *
 	 * The transaction is kept short to reduce locking.
+	 *
+	 * @param int $textId
+	 * @param string $url
 	 */
 	function moveTextRow( $textId, $url ) {
 		if ( $this->copyOnly ) {
@@ -536,18 +562,18 @@ class RecompressTracked {
 		$dbw = wfGetDB( DB_MASTER );
 		$dbw->begin( __METHOD__ );
 		$dbw->update( 'text',
-			array( // set
+			[ // set
 				'old_text' => $url,
 				'old_flags' => 'external,utf-8',
-			),
-			array( // where
+			],
+			[ // where
 				'old_id' => $textId
-			),
+			],
 			__METHOD__
 		);
 		$dbw->update( 'blob_tracking',
-			array( 'bt_moved' => 1 ),
-			array( 'bt_text_id' => $textId ),
+			[ 'bt_moved' => 1 ],
+			[ 'bt_text_id' => $textId ],
 			__METHOD__
 		);
 		$dbw->commit( __METHOD__ );
@@ -560,36 +586,38 @@ class RecompressTracked {
 	 *
 	 * This function completes any moves that only have done bt_new_url. This
 	 * can happen when the script is interrupted, or when --copy-only is used.
+	 *
+	 * @param array $conds
 	 */
 	function finishIncompleteMoves( $conds ) {
 		$dbr = wfGetDB( DB_SLAVE );
 
 		$startId = 0;
-		$conds = array_merge( $conds, array(
+		$conds = array_merge( $conds, [
 			'bt_moved' => 0,
 			'bt_new_url IS NOT NULL'
-		) );
+		] );
 		while ( true ) {
 			$res = $dbr->select( 'blob_tracking',
 				'*',
-				array_merge( $conds, array( 'bt_text_id > ' . $dbr->addQuotes( $startId ) ) ),
+				array_merge( $conds, [ 'bt_text_id > ' . $dbr->addQuotes( $startId ) ] ),
 				__METHOD__,
-				array(
+				[
 					'ORDER BY' => 'bt_text_id',
 					'LIMIT' => $this->batchSize,
-				)
+				]
 			);
 			if ( !$res->numRows() ) {
 				break;
 			}
 			$this->debug( 'Incomplete: ' . $res->numRows() . ' rows' );
 			foreach ( $res as $row ) {
+				$startId = $row->bt_text_id;
 				$this->moveTextRow( $row->bt_text_id, $row->bt_new_url );
 				if ( $row->bt_text_id % 10 == 0 ) {
-					$this->waitForSlaves();
+					wfWaitForSlaves();
 				}
 			}
-			$startId = $row->bt_text_id;
 		}
 	}
 
@@ -602,41 +630,45 @@ class RecompressTracked {
 		if ( $cluster === false ) {
 			$cluster = reset( $this->destClusters );
 		}
+
 		return $cluster;
 	}
 
 	/**
 	 * Gets a DB master connection for the given external cluster name
-	 * @param $cluster string
+	 * @param string $cluster
 	 * @return DatabaseBase
 	 */
 	function getExtDB( $cluster ) {
 		$lb = wfGetLBFactory()->getExternalLB( $cluster );
+
 		return $lb->getConnection( DB_MASTER );
 	}
 
 	/**
 	 * Move an orphan text_id to the new cluster
+	 *
+	 * @param array $textIds
 	 */
 	function doOrphanList( $textIds ) {
 		// Finish incomplete moves
 		if ( !$this->copyOnly ) {
-			$this->finishIncompleteMoves( array( 'bt_text_id' => $textIds ) );
+			$this->finishIncompleteMoves( [ 'bt_text_id' => $textIds ] );
 			$this->syncDBs();
 		}
 
 		$trx = new CgzCopyTransaction( $this, $this->orphanBlobClass );
 
 		$res = wfGetDB( DB_SLAVE )->select(
-			array( 'text', 'blob_tracking' ),
-			array( 'old_id', 'old_text', 'old_flags' ),
-			array(
+			[ 'text', 'blob_tracking' ],
+			[ 'old_id', 'old_text', 'old_flags' ],
+			[
 				'old_id' => $textIds,
 				'bt_text_id=old_id',
 				'bt_moved' => 0,
-			),
+			],
 			__METHOD__,
-			array( 'DISTINCT' )
+			[ 'DISTINCT' ]
 		);
 
 		foreach ( $res as $row ) {
@@ -650,52 +682,42 @@ class RecompressTracked {
 				$this->debug( "[orphan]: committing blob with " . $trx->getSize() . " rows" );
 				$trx->commit();
 				$trx = new CgzCopyTransaction( $this, $this->orphanBlobClass );
-				$this->waitForSlaves();
+				wfWaitForSlaves();
 			}
 		}
 		$this->debug( "[orphan]: committing blob with " . $trx->getSize() . " rows" );
 		$trx->commit();
 	}
-
-	/**
-	 * Wait for slaves (quietly)
-	 */
-	function waitForSlaves() {
-		$lb = wfGetLB();
-		while ( true ) {
-			list( $host, $maxLag ) = $lb->getMaxLag();
-			if ( $maxLag < 2 ) {
-				break;
-			}
-			sleep( 5 );
-		}
-	}
 }
 
 /**
  * Class to represent a recompression operation for a single CGZ blob
  */
 class CgzCopyTransaction {
+	/** @var RecompressTracked */
 	public $parent;
 	public $blobClass;
+	/** @var ConcatenatedGzipHistoryBlob */
 	public $cgz;
 	public $referrers;
 
 	/**
 	 * Create a transaction from a RecompressTracked object
+	 * @param RecompressTracked $parent
+	 * @param string $blobClass
 	 */
 	function __construct( $parent, $blobClass ) {
 		$this->blobClass = $blobClass;
 		$this->cgz = false;
-		$this->texts = array();
+		$this->texts = [];
 		$this->parent = $parent;
 	}
 
 	/**
 	 * Add text.
 	 * Returns false if it's ready to commit.
-	 * @param $text string
-	 * @param $textId
+	 * @param string $text
+	 * @param int $textId
 	 * @return bool
 	 */
 	function addItem( $text, $textId ) {
@@ -706,6 +728,7 @@ class CgzCopyTransaction {
 		$hash = $this->cgz->addItem( $text );
 		$this->referrers[$textId] = $hash;
 		$this->texts[$textId] = $text;
+
 		return $this->cgz->isHappy();
 	}
 
@@ -719,7 +742,7 @@ class CgzCopyTransaction {
 	function recompress() {
 		$class = $this->blobClass;
 		$this->cgz = new $class;
-		$this->referrers = array();
+		$this->referrers = [];
 		foreach ( $this->texts as $textId => $text ) {
 			$hash = $this->cgz->addItem( $text );
 			$this->referrers[$textId] = $hash;
@@ -737,19 +760,20 @@ class CgzCopyTransaction {
 			return;
 		}
 
-		// Check to see if the target text_ids have been moved already.
-		//
-		// We originally read from the slave, so this can happen when a single
-		// text_id is shared between multiple pages. It's rare, but possible
-		// if a delete/move/undelete cycle splits up a null edit.
-		//
-		// We do a locking read to prevent closer-run race conditions.
+		/* Check to see if the target text_ids have been moved already.
+		 *
+		 * We originally read from the slave, so this can happen when a single
+		 * text_id is shared between multiple pages. It's rare, but possible
+		 * if a delete/move/undelete cycle splits up a null edit.
+		 *
+		 * We do a locking read to prevent closer-run race conditions.
+		 */
 		$dbw = wfGetDB( DB_MASTER );
 		$dbw->begin( __METHOD__ );
 		$res = $dbw->select( 'blob_tracking',
-			array( 'bt_text_id', 'bt_moved' ),
-			array( 'bt_text_id' => array_keys( $this->referrers ) ),
-			__METHOD__, array( 'FOR UPDATE' ) );
+			[ 'bt_text_id', 'bt_moved' ],
+			[ 'bt_text_id' => array_keys( $this->referrers ) ],
+			__METHOD__, [ 'FOR UPDATE' ] );
 		$dirty = false;
 		foreach ( $res as $row ) {
 			if ( $row->bt_moved ) {
@@ -766,9 +790,11 @@ class CgzCopyTransaction {
 				// All have been moved already
 				if ( $originalCount > 1 ) {
 					// This is suspcious, make noise
-					$this->critical( "Warning: concurrent operation detected, are there two conflicting " .
+					$this->parent->critical(
+						"Warning: concurrent operation detected, are there two conflicting " .
 						"processes running, doing the same job?" );
 				}
+
 				return;
 			}
 			$this->recompress();
@@ -786,11 +812,11 @@ class CgzCopyTransaction {
 		foreach ( $this->referrers as $textId => $hash ) {
 			$url = $baseUrl . '/' . $hash;
 			$dbw->update( 'blob_tracking',
-				array( 'bt_new_url' => $url ),
-				array(
+				[ 'bt_new_url' => $url ],
+				[
 					'bt_text_id' => $textId,
 					'bt_moved' => 0, # Check for concurrent conflicting update
-				),
+				],
 				__METHOD__
 			);
 		}
